/* Polynomial.h
 *
 * Copyright (C) 2008 Laboratoire Statistique & Genome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef POLYNOMIAL_H
#define P0LYNOMIAL_H

#include <Drifting_Markov.h>

using namespace std;

//===============================

class Polynomial : public Drifting_Markov
{

private:
 
  int _degree;                     //Degree of polynom
  int _nbM;                        //Number of points of support (or degree of polynom +1)
  int _K;                          //Number of parameters of the model (_K = _nbM * nbl * (nbc-1))
                                   //nbl = _alphabet_size^_order, nbc = _alphabet_size
  double *** _TM;                  //Drifting Markov : table of all matrices Pi_i 
  double ** _A;                    //Coefficients of t : table for each matrix Pi_i

  void init_poly();
  void init_poly(double * pos);
  void alloc_table(double *** M, double *** B);  
  void init_table(const Sequence & seq, double *** M, double *** B);
  void solve_systeme(double *** M, double *** B);
  void normalize_TM(double *** TM);
  void alloc_TM(double *** TM);
  void init_TM(double *** TM);


public:

  //---------------//
  // Constructeurs //
  //---------------//
  
  //DMM from one sequence
  Polynomial(const Sequence & seq, int degree);
  
  //DMM from a group of sequences (mean Pi_t)
  Polynomial(const SequenceSet & seqset, int degree);

  //DMM from a group of sequences (sum over sequences)
  Polynomial(int degree ,const SequenceSet & seqset);

  //DMM from a config_file
  Polynomial(char * config_file);

  //Copy constructor
  Polynomial(Polynomial & C);


  //-------------//
  // Destructeur //
  //-------------// 

  virtual ~Polynomial();

  //----------//
  // Mthodes //
  //----------//
  
  //Number of parameters = k
  virtual int nb_param();

  //+++++++++++++
  //Quadratic distance between this._TM and C._TM
  void mean_distance(Polynomial & C);  
  //+++++++++++++  

  //Pi_t(u,v) where code=code(uv)
  //Warning: Code of aaa = 0.
  //0 <= t <= _n-1
  virtual double Pi(int t, int code);

  //Pi_t(i,j)
  //0<=i<=nbl-1 and 0<=j<=nbc-1
  //0<=t<=_n-1
  virtual double Pi(int t, int i, int j); 

  //Degree
  virtual int degree();

  //Edit a configuration file of the DMM
  virtual int config_file(ofstream& cf);

};
//===========================

#endif //POLYNOMIAL_H
