/* Polynomial_M.cc
 *
 * Copyright (C) 2008 Laboratoire Statistique & Genome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "Polynomial_M.h"

//===================================================================================
//===================================================================================

//////////////////
// CONSTRUCTORS //
//////////////////

//###################################################################################

//Allocation of system matrices

void Polynomial_M::alloc_table(double *** M, double *** B)
{
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int i,j,c;

  for(c=0;c<nbl;c++)
    {
      M[c] = new double * [_nbM];
      M[c][0] = new double [_nbM*_nbM];
      for(i=1;i<_nbM;i++)
 	{
 	  M[c][i] = M[c][i-1]+_nbM;
 	}
      for(i=0;i<_nbM;i++)
 	{
 	  for(j=0;j<_nbM;j++)
 	    {
 	      M[c][i][j]=0;
	    } 
  	} 
    }
  for(c=0;c<nbl;c++)
    {
      B[c] = new double * [nbc];
      B[c][0]= new double [nbc*_nbM];
      for(i=1;i<nbc;i++)
	{
	  B[c][i] = B[c][i-1]+ _nbM;
	}
      for(i=0;i<nbc;i++)
 	{
 	  for(j=0;j<_nbM;j++)
 	    {
 	      B[c][i][j]=0;
	    }
	}
    }
}

//###################################################################################

//Initialization of system matrices

void Polynomial_M::init_table(const Sequence & seq, double *** M, double *** B)
{
  int k,lo,ha; 
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int i,j,c;

  unsigned long size= seq.tell_length();
  double * a = new double [_nbM];
  
  //Order 0 Markov model  
  if(_order==0)
    { 
      //Warning nd = sequence size - 1
      double nd=double(size-1);
      int v,cpt;
      double kd;
      for(k=_order;(unsigned long)(k)<size;k++)
	{ 
	  kd=double(k);
	  //Coefficients of M_i
	  for(i=0;i<_nbM;i++)
	    {
	      a[i]=pow(kd/nd,i);
	    }
	  //Matrices
	  cpt=0;
	  for(ha=0;ha<_nbM;ha++)
	    {
	      for(lo=0;lo<_nbM;lo++)
		{
		  M[cpt][ha][lo]+=a[ha]*a[lo];
		}
	    }	   
	  for(v=0;v<nbc;v++)
	    {
	      if(v==seq.tell_int(k))
		{
		  for(ha=0;ha<_nbM;ha++)
		    {
		      B[cpt][v][ha]+=a[ha];
		    }
		}
	    }	    	     
	}
    }
  else
    {
      //Code of words of size _order between ud and uf
      int ud=0;
      for(i=1;i<_order;i++)
	{
	  ud+=int(pow(double(_alphabet_size),i));
	}  
      int uf=ud+int(pow(double(_alphabet_size),i));
      
      //Warning nd = sequence size - 1
      double nd=double(size-1);
      int u,v,cpt;
      double kd;
      for(k=_order;(unsigned long)(k)<size;k++)
	{ 
	  kd=double(k);
	  //Coefficients of M_i
	  for(i=0;i<_nbM;i++)
	    {
	      a[i]=pow(kd/nd,i);
	    }
	  //Matrices
	  cpt=0;
	  for(u=ud;u<uf;u++)
	    {
	      if(u == seq.tell_code(k)/_alphabet_size-1)
		{
		  for(ha=0;ha<_nbM;ha++)
		    {
		      for(lo=0;lo<_nbM;lo++)
			{
			  M[cpt][ha][lo]+=a[ha]*a[lo];
			}
		    }	   
		  for(v=0;v<nbc;v++)
		    {
		      if(v==seq.tell_int(k))
			{
			  for(ha=0;ha<_nbM;ha++)
			    {
			      B[cpt][v][ha]+=a[ha];
			    }
			}
		    }	    	     
		}
	      cpt++;
	    } 
	}
    }
  delete[] a; 
}

//###################################################################################

//Solve of the system MX=B

void Polynomial_M::solve_systeme(double *** M, double *** B)
{
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int i;
  //Lapack solve (matrix is not positive definite) 
  char uplo = 'U'; 
  int N=_nbM; 
  int nrhs=nbc; 
  int lda=N;
  int ldb=N;
  int lwork;
  int info;
  double * work;
  int * ipiv = new int[_nbM];
  for(i=0;i<nbl;i++)
    {
      //Computing of lwork
      work=new double [_nbM];
      lwork=-1;
      dsysv_(uplo, N, nrhs, M[i][0], lda, ipiv, B[i][0], ldb, work, lwork, info);
      lwork=int(work[0]);
      delete[] work;

      work = new double [lwork];
      //Solve
      dsysv_(uplo, N, nrhs, M[i][0], lda, ipiv, B[i][0], ldb, work, lwork, info);
      delete[] work;
    }
  delete[] ipiv;
}

//###################################################################################

//Allocation of matrices table
void Polynomial_M::alloc_TM(double *** TM) 
{
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int i,j,c;
  for(c=0;c<_nbM;c++)
    {
      TM[c] = new double * [nbl];
      TM[c][0]= new double [nbl*nbc];
      for(i=1;i<nbl;i++)
	{
	  TM[c][i] = TM[c][i-1]+ nbc;
	}
    }
}

//###################################################################################

//Initialization of matrices table _TM to zero
void Polynomial_M::init_TM(double *** TM) 
{
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int i,j,c;
  for(c=0; c<_nbM; c++)
    {  
      for(i=0;i<nbl;i++)
	{
	  for(j=0;j<nbc;j++)
	    {
	      TM[c][i][j] = 0;
	    }
	}
    }
}

//###################################################################################

// Constructor 1 : 1 sequence.

Polynomial_M::Polynomial_M(const Sequence & seq, int degree): Drifting_Markov(seq)
{
  _degree=degree;
  _nbM=degree+1; 
 
  int i,j,c;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  _K=_nbM*nbl*(nbc-1); 

  //Allocation of matrices table
  _TM = new double ** [_nbM];
  alloc_TM(_TM);

  double *** M = new double ** [nbl];
  double *** B = new double ** [nbl];
  alloc_table(M,B);
  init_table(seq,M,B);
  solve_systeme(M,B);

  for(c=0;c<nbl;c++)
    {
      for(i=0;i<nbc;i++)
 	{
 	  for(j=0;j<_nbM;j++)
 	    {
	      _TM[j][c][i]= B[c][i][j];
	    }
	}
    }
 
   for (i=0;i<nbl;i++)
     {
       delete[] M[i][0];
       delete[] M[i];
       delete[] B[i][0];
       delete[] B[i];
     }
   delete[] M;
   delete[] B;
}

//###################################################################################

// Constructor 2 : mean of many sequences (mean Pi_t).

Polynomial_M::Polynomial_M (const SequenceSet & seqset, int degree): Drifting_Markov(seqset)
{
  _degree=degree;
  _nbM=degree+1; 
 
  int i,j,c;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  _K=_nbM*nbl*(nbc-1); 

  //Allocation of matrices table
  _TM = new double ** [_nbM];
  alloc_TM(_TM);

  //Allocation of matrices table
  double *** TM = new double ** [_nbM];
  alloc_TM(TM);

  //Initialization of matrices table _TM to zero
  init_TM(_TM);

  int ns;
  int nbs=seqset.tell_nb_sequence(); 
  for(ns=0;ns<nbs;ns++)
    { 
      double *** M = new double ** [nbl];
      double *** B = new double ** [nbl];
      alloc_table(M,B);
      init_table(seqset(ns),M,B);
      solve_systeme(M,B);

      for(c=0;c<nbl;c++)
	{
	  for(i=0;i<nbc;i++)
	    {
	      for(j=0;j<_nbM;j++)
		{
		  TM[j][c][i]= B[c][i][j];
		}
	    }
	}
      
      //Add TM in _TM 
      for(c=0; c<_nbM; c++)
	{  
	  for(i=0;i<nbl;i++)
	    {
	      for(j=0;j<nbc;j++)
		{
		  _TM[c][i][j]+= TM[c][i][j];
		}
	    }
	}

      for (i=0;i<nbl;i++)
	{
	  delete[] M[i][0];
	  delete[] M[i];
	  delete[] B[i][0];
	  delete[] B[i];
	}
      delete[] M;
      delete[] B;
    }

  //Divide _TM by the number of sequences
  for(c=0; c<_nbM; c++)
    {  
      for(i=0;i<nbl;i++)
	{
	  for(j=0;j<nbc;j++)
	    {
	      _TM[c][i][j]=_TM[c][i][j]/double(nbs) ;
	    }
	}
    }
      
  for (c=0;c<_nbM;c++)
    {
      delete[] TM[c][0];
    }
}

//###################################################################################

//Constructor 3 : sum of many sequences (Sum over sequences)

Polynomial_M::Polynomial_M
(int degree, const SequenceSet & seqset)
: Drifting_Markov(seqset)
{
  //Initialization of private members
  _degree=degree;
  _nbM=degree+1; 

  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int i,j,c;
  _K=_nbM*nbl*(nbc-1); 

  //Allocation of matrices table
  _TM = new double ** [_nbM];
  alloc_TM(_TM); 
      
  double *** M = new double ** [nbl];
  double *** B = new double ** [nbl];
  alloc_table(M,B);

  int ns;
  int nbs=seqset.tell_nb_sequence(); 
  for(ns=0;ns<nbs;ns++)
    {  
       init_table(seqset(ns),M,B);   
    }
  solve_systeme(M,B);

  for(c=0;c<nbl;c++)
    {
      for(i=0;i<nbc;i++)
	{
	  for(j=0;j<_nbM;j++)
	    {
	      _TM[j][c][i]= B[c][i][j];
	    }
	}
    }

  for (i=0;i<nbl;i++)
    {
      delete[] M[i][0];
      delete[] M[i];
      delete[] B[i][0];
      delete[] B[i];
    }
  delete[] M;
  delete[] B;  
}

//###################################################################################

// Constructor 4 : configuration file

Polynomial_M::Polynomial_M(char * config_file)
{
  ifstream cf(config_file);
  int method;
  //method = 4
  cf>>method;
  cf>>_alphabet_size;
  cf>>_n;
  cf>>_order;
  cf>>_degree;
  _nbM=_degree+1;;
  
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));

  _tPi=_alphabet_size * int(pow(double(_alphabet_size),_order));
  _K=_nbM*nbl*(nbc-1); 

  int c,i,j;
  int k; 
     
   //Allocation of matrices table
  _TM = new double ** [_nbM];
  alloc_TM(_TM);

  //Reading of TM
  for(k=0;k<_nbM;k++)
    {
      for (i=0; i<nbl;i++)
	{
	  for (j=0;j<nbc;j++)
	    {
	      cf>>_TM[k][i][j];
	    }	  
	}
    }
  
  cf.close();
}

//###################################################################################

// Constructor 5 : copy 

Polynomial_M::Polynomial_M(Polynomial_M & C) : Drifting_Markov(C)
{
  int c,i,j;
  _degree = C._degree;
  _nbM = C._nbM;
  _K=C._K; 

  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order)); 

  //Allocation of matrices table
  _TM = new double ** [_nbM];
  alloc_TM(_TM);
   
  //Copy
  for(c=0; c<_nbM; c++)
    {  
      for(i=0;i<nbl;i++)
	{
	  for(j=0;j<nbc;j++)
	    {
	      _TM[c][i][j] = C._TM[c][i][j];
	    }
	}
    }
}


////////////////
// DESTRUCTOR //
////////////////

//###########################################################################################

//Polynomial_M::~Polynomial_M(){}
Polynomial_M::~Polynomial_M()
{
  int c;
  for (c=0;c<_nbM;c++)
    {
      delete[] _TM[c][0];
      delete[] _TM[c];
    }
  delete[] _TM; 
}

//===========================================================================================
//===========================================================================================


//Number of parameters
int Polynomial_M::nb_param()
{
  return _K;
}

//###########################################################################################
 
//Quadratic distance between this._TM and C._TM
void Polynomial_M::mean_distance(Polynomial_M & C)
{
  //No comparaisons if not the same order
  if((_tPi!=C._tPi)||(_nbM!=C._nbM))
    { 
      cout<<"==============================================================="<<endl;
      cout<<"                         mean_distance                         "<<endl;
      cout<<"                                                               "<<endl;
      cout<<"          Error : Not the same order, not comparable           "<<endl;
      cout<<"==============================================================="<<endl;
    }
  else 
    {
      int c,i,j;
      double D1=0, D2=0; 
      int nbc=_alphabet_size;
      int nbl= int(pow(double(_alphabet_size),_order));
      for(c=0;c<_nbM;c++)
	{
	  for(i=0;i<nbl;i++)
	    {
	      for(j=0;j<nbc;j++)
		{
		  D1+=abs(_TM[c][i][j]-C._TM[c][i][j]);
		  D2+=pow(_TM[c][i][j]-C._TM[c][i][j],double(2));
		}
	    }
	}
      double ne = double(_nbM*nbl*nbc);
      cout<<"   Sum             = "<<D1<<endl;
      cout<<"   Mean            = "<<D1/ne<<endl;
      cout<<"   Sum of squares  = "<<D2<<endl;
      cout<<"   Mean of squares = "<<D2/ne<<endl;
    }
}

//###########################################################################################

//Pi[code] at position t 
//Begin at 0 and end at _tPi-1

double Polynomial_M::Pi(int t, int code)
{
  int nbc=_alphabet_size;
  double Pi_t = Pi(t,code/nbc,code%nbc);
  return Pi_t;
}

//###########################################################################################

//Pi(i,j) at position t
//i is the line number
//j is the column number
//0 <= i < nbl
//0 <= j < nbc

double Polynomial_M::Pi(int t, int i, int j)
{
  unsigned long seqlen = _n;
  double Pi_t;
  int d1,l;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));  
  //Warning: we divide by sequence size - 1 
  //Warning convert double to int...
  double k;
  if(seqlen%2==1)
    {
      k=double(t)/double(seqlen-1);
    }
  else
    {
      k=double(t)/double(seqlen);
    }
  if ((i<0)||(j<0)||(i>=nbl)||(j>=nbc)||(k>1))
    {
      cout<<"Error: code or (i,j) not correct or t not correct"<<endl;
      return 0;
    }
  else
    {
      double * vect = new double [nbc];
      double sum;
      sum=0;
      for(l=0;l<nbc;l++)
	{	  
	  vect[l]=0;
	  for(d1=0;d1<_nbM;d1++)
	    {
	      vect[l]+=_TM[d1][i][l]*pow(k,double(d1));
	    }
  	  //Normalization
  	  vect[l]=abs(vect[l]);
  	  sum+=vect[l];
	}
        for(l=0;l<nbc;l++)
  	{
  	  vect[l]=vect[l]/sum;
  	}
      Pi_t=vect[j];     
  
      delete[] vect;
      return Pi_t;
    } 
}


//###########################################################################################

//Degree
int Polynomial_M::degree()
{
  return _degree;
}

//###########################################################################################

int Polynomial_M::config_file(ofstream& cf)
{
  int i,j,k,t;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int method=4;

  cf<<method<<endl;
  cf<<_alphabet_size<<endl;
  cf<<_n<<endl;
  cf<<_order<<endl;
  cf<<_degree<<endl;
  cf<<endl;

  for(k=0;k<_nbM;k++)
    {
      for(i=0;i<nbl;i++)
	{
	  for(j=0;j<nbc;j++)
	    {
	      cf<<_TM[k][i][j]<<" "; 
	    }  
	  cf<<endl;
	}
      cf<<endl;
    }
  return 1;
}

//###########################################################################################

