/* Splines.cc
 *
 * Copyright (C) 2008 Laboratoire Statistique & Genome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "Splines.h"

//===================================================================================
//===================================================================================

//////////////////
// CONSTRUCTORS //
//////////////////

//###################################################################################

  void Splines::alloc_o(double **** Mo, double *** Bo)
{
  int u,v,i,j,dim;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  dim=4;
  //M initialisation
  for(u=0;u<nbl;u++)
    {
      Mo[u] = new double ** [nbc];
      for(v=0;v<nbc;v++)
	{
	  Mo[u][v]= new double * [dim];
	  Mo[u][v][0]= new double [dim*dim];
	  for(i=1;i<dim;i++)
	    {
	      Mo[u][v][i] = Mo[u][v][i-1]+ dim;
	    }
	  for(i=0;i<dim;i++)
	    {
	      for(j=0;j<dim;j++)
		{
		  Mo[u][v][i][j] = 0;
		}
	    }
	}
    }

  dim=4;
  //B initialisation
  for(u=0;u<nbl;u++)
    {
      Bo[u] = new double * [nbc];
      Bo[u][0]= new double [nbc*dim];
      for(v=1;v<nbc;v++)
	{
	  Bo[u][v] = Bo[u][v-1]+ dim;
	}
      for(v=0;v<nbc;v++)
	{
	  for(i=0;i<dim;i++)
	    {
	      Bo[u][v][i] = 0;
	    }
	}
    }
}
//###################################################################################
  void Splines::alloc_a(double ***** Ma, double **** Ba)
{
  int u,v,i,j,m,dim;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  dim=2;
  //M aller
  for(m=0;m<_N;m++)
    {
      Ma[m] = new double *** [nbl];
      for(u=0;u<nbl;u++)
	{
	  Ma[m][u] = new double ** [nbc];
	  for(v=0;v<nbc;v++)
	    {
	      Ma[m][u][v]= new double * [dim];
	      Ma[m][u][v][0]= new double [dim*dim];
	      for(i=1;i<dim;i++)
		{
		  Ma[m][u][v][i] = Ma[m][u][v][i-1]+ dim;
		}
	      for(i=0;i<dim;i++)
		{
		  for(j=0;j<dim;j++)
		    {
		      Ma[m][u][v][i][j] = 0;
		    }
		}
	    }
	}
    }
  dim=2;
  //B aller
  for(m=0;m<_N;m++)
    {
      Ba[m] = new double ** [nbl];
      for(u=0;u<nbl;u++)
	{
	  Ba[m][u] = new double * [nbc];
	  Ba[m][u][0]= new double [nbc*dim];
	  for(v=1;v<nbc;v++)
	    {
	      Ba[m][u][v] = Ba[m][u][v-1]+ dim;
	    }
	  for(v=0;v<nbc;v++)
	    {
	      for(i=0;i<dim;i++)
		{
		  Ba[m][u][v][i] = 0;
		}
	    }
	}
    }
}
//###################################################################################
  void Splines::alloc_r(double ***** Mr, double **** Br)
{ 
  int u,v,i,j,m,dim;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  dim =2;
  //M retour
  for(m=0;m<_N;m++)
    {
      Mr[m] = new double *** [nbl];
      for(u=0;u<nbl;u++)
	{
	  Mr[m][u] = new double ** [nbc];
	  for(v=0;v<nbc;v++)
	    {
	      Mr[m][u][v]= new double * [dim];
	      Mr[m][u][v][0]= new double [dim*dim];
	      for(i=1;i<dim;i++)
		{
		  Mr[m][u][v][i] = Mr[m][u][v][i-1]+ dim;
		}
	      for(i=0;i<dim;i++)
		{
		  for(j=0;j<dim;j++)
		    {
		      Mr[m][u][v][i][j] = 0;
		    }
		}
	    }
	}
    }
  //B retour
  for(m=0;m<_N;m++)
    {
      Br[m] = new double ** [nbl];
      for(u=0;u<nbl;u++)
	{
	  Br[m][u] = new double * [nbc];
	  Br[m][u][0]= new double [nbc*dim];
	  for(v=1;v<nbc;v++)
	    {
	      Br[m][u][v] = Br[m][u][v-1]+ dim;
	    }
	  for(v=0;v<nbc;v++)
	    {
	      for(i=0;i<dim;i++)
		{
		  Br[m][u][v][i] = 0;
		}
	    }
	}
    }
}
//###################################################################################
  void Splines::alloc_sum_o(double ** sumMo, double *** sumBo)
{
  int u,v,i;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));

  int sizeMo=10;
  //sumMo initialisation
  for(u=0;u<nbl;u++)
    {     
      sumMo[u]= new double [sizeMo];	 
      for(i=0;i<sizeMo;i++)
	{
	  sumMo[u][i] = 0;
	}
    }

  int sizeBo=4;
  //sumBo initialisation
  for(u=0;u<nbl;u++)
    {
      sumBo[u] = new double * [nbc];
      sumBo[u][0]= new double [nbc*sizeBo];	 
      for(v=1;v<nbc;v++)
	{
	  sumBo[u][v] = sumBo[u][v-1]+ sizeBo;	     
	}
      for(v=0;v<nbc;v++)
	{
	  for(i=0;i<sizeBo;i++)
	    {
	      sumBo[u][v][i] = 0;		  
	    }
	}
    }
}
//###################################################################################
  void Splines::alloc_sum_a(double *** sumMa, double **** sumBa)
{
  int u,v,i,m;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));

  int sizeM=3;
  //sumMa aller
  for(m=0;m<_N;m++)
    {
      sumMa[m] = new double * [nbl];
      for(u=0;u<nbl;u++)
	{	  
	  sumMa[m][u]= new double [sizeM];	 
	  for(i=0;i<sizeM;i++)
	    {
	      sumMa[m][u][i] = 0;
	    }
	}
    }

  int sizeB=2;
  //sumBa aller
  for(m=0;m<_N;m++)
    {
      sumBa[m] = new double ** [nbl];
     
      for(u=0;u<nbl;u++)
	{
	  sumBa[m][u] = new double * [nbc];
	  sumBa[m][u][0]= new double [nbc*sizeB];	 
	  for(v=1;v<nbc;v++)
	    {
	      sumBa[m][u][v] = sumBa[m][u][v-1]+ sizeB;	     
	    }
	  for(v=0;v<nbc;v++)
	    {
	      for(i=0;i<sizeB;i++)
		{
		  sumBa[m][u][v][i] = 0;		  
		}
	    }
	}
    }
}
//###################################################################################
  void Splines::alloc_sum_r(double *** sumMr, double **** sumBr)
{
  int u,v,i,m;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));

  int sizeM=3;
  //sumMr retour
  for(m=0;m<_N;m++)
    {
      sumMr[m] = new double * [nbl];
      for(u=0;u<nbl;u++)
	{
	  sumMr[m][u]= new double [sizeM];	 	 
	  for(i=0;i<sizeM;i++)
	    {
	      sumMr[m][u][i] = 0;
	    }
	}
    }

  int sizeB=2;
  //sumBr retour
  for(m=0;m<_N;m++)
    {
      sumBr[m] = new double ** [nbl];
     
      for(u=0;u<nbl;u++)
	{
	  sumBr[m][u] = new double * [nbc];
	  sumBr[m][u][0]= new double [nbc*sizeB];	 
	  for(v=1;v<nbc;v++)
	    {
	      sumBr[m][u][v] = sumBr[m][u][v-1]+ sizeB;	     
	    }
	  for(v=0;v<nbc;v++)
	    {
	      for(i=0;i<sizeB;i++)
		{
		  sumBr[m][u][v][i] = 0;		  
		}
	    }
	}
    }
}

//###################################################################################

void Splines::alloc_sumB_arB(double *** sumBaB, double *** sumBrB)
{
  int m,u,i;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));

  int sizeBB=4;
  //sumBaB aller
  for(m=0;m<_N;m++)
    {
      sumBaB[m] = new double * [nbl];
     
      for(u=0;u<nbl;u++)
	{
	  sumBaB[m][u]= new double [sizeBB];	 
	 
	  for(i=0;i<sizeBB;i++)
	    {
	      sumBaB[m][u][i] = 0;		  
	    }	   
	}
    }

  //sumBrB retour
  for(m=0;m<_N;m++)
    {
      sumBrB[m] = new double * [nbl];
     
      for(u=0;u<nbl;u++)
	{
	  sumBrB[m][u]= new double [sizeBB];	 
	 
	  for(i=0;i<sizeBB;i++)
	    {
	      sumBrB[m][u][i] = 0;		  
	    }	   
	}
    }
}


//###################################################################################


void Splines::free_o(double **** Mo, double *** Bo)
{
  int u,v;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  for(u=0;u<nbl;u++)
    {
      for(v=0;v<nbc;v++)
	{
	  delete[] Mo[u][v][0];
	  delete[] Mo[u][v];	    
	}
      delete[] Mo[u];
    }

  for(u=0;u<nbl;u++)
    {
      delete[] Bo[u][0];
      delete[] Bo[u];
    }
}
//###################################################################################
  void Splines::free_ar(double ***** Ma, double **** Ba, double ***** Mr, double **** Br)
{
  int u,v,m;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  for(m=0;m<_N;m++)
    {
      for(u=0;u<nbl;u++)
	{
	  for(v=0;v<nbc;v++)
	    {
	      delete[] Ma[m][u][v][0];
	      delete[] Ma[m][u][v];
	      delete[] Mr[m][u][v][0];
	      delete[] Mr[m][u][v];	   	    
	    }
	  delete[] Ma[m][u];
	  delete[] Mr[m][u];
	}

      delete[] Ma[m];
      delete[] Mr[m];
    }

  for(m=0;m<_N;m++)
    {
      for(u=0;u<nbl;u++)
	{
	  delete[] Ba[m][u][0];
	  delete[] Ba[m][u];
	  delete[] Br[m][u][0];
	  delete[] Br[m][u];
	}
      delete[] Ba[m];
      delete[] Br[m];
    }
}

//###################################################################################
  void Splines::free_sum_o(double ** sumMo, double *** sumBo)
{
  int u;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  for(u=0;u<nbl;u++)
    {
      delete[] sumMo[u];	  
    }

  for(u=0;u<nbl;u++)
    {

      delete[] sumBo[u][0];
      delete[] sumBo[u];     
    }
}
//###################################################################################
void Splines::free_sum_ar(double *** sumMa, double **** sumBa, double *** sumMr, double **** sumBr)
{
  int u,m;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  for(m=0;m<_N;m++)
    {
      for(u=0;u<nbl;u++)
	{
	  delete[] sumMa[m][u];	
	  delete[] sumMr[m][u];	  	  
	}
      delete[] sumMa[m];
      delete[] sumMr[m];
    }

  for(m=0;m<_N;m++)
    {
      for(u=0;u<nbl;u++)
	{
	  delete[] sumBa[m][u][0];
	  delete[] sumBa[m][u];
	  delete[] sumBr[m][u][0];
	  delete[] sumBr[m][u];	  
	}
      delete[] sumBa[m];
      delete[] sumBr[m];
    }
}

//###################################################################################

void Splines::free_sumB_arB(double *** sumBaB, double *** sumBrB)
{
  int m,u;
  int nbl= int(pow(double(_alphabet_size),_order));
  for(m=0;m<_N;m++)
    {
      for(u=0;u<nbl;u++)
	{       
	  delete[] sumBaB[m][u];
	  delete[] sumBrB[m][u];	  
	}
      delete[] sumBaB[m];
      delete[] sumBrB[m];
    }
}

//###################################################################################

void Splines::init_tab(unsigned long * tab, unsigned long seqlen)
{
  //Sequence cut at tab[i]
  //Warning:
  //segment 1={tab[0],tab[1]}, 
  //segment 2={tab[1]+1,tab[2]}, 
  //segment N={tab[N-1]+1,tab[N]}.
  int i;
  tab[0]=0;
  for(i=1;i<_N;i++)
    {
      tab[i]=(unsigned long)(i)*(unsigned long)(seqlen/_N)-1;
    }
  tab[_N]=seqlen-1;
}

//###################################################################################

void Splines::init_sums(const Sequence & seq, double ** sumMo, double *** sumBo, double *** sumMa, double **** sumBa,double *** sumMr, double **** sumBr, double *** sumBaB, double *** sumBrB) 
{
  int i,k,u,v,t;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));

  unsigned long seqlen= seq.tell_length();
  double td; 
  double tkap,tdel,nu; 
  double nd; 
  //Warning nd = sequence size - 1
  nd=double(seqlen)-1;
  unsigned long * tab= new unsigned long [_N+1];
  init_tab(tab,seqlen);
 
  //=================================
  //Initialisation of sumM and sumB
  //=================================
  if(_order==0)
    {
      //sumMo and sumBo
      for(t=_order;(unsigned long)(t)<=tab[1];t++)
	{ 
	  td= double(t);
	  tkap=(td-double(tab[0]))/(double(tab[1])-double(tab[0]));
	  tdel=(double(tab[1])-td)/(double(tab[1])-double(tab[0]));
	  sumMo[0][0]+=amat(tkap)*amat(tkap);
	  sumMo[0][1]+=amat(tkap)*bmat(tkap);
	  sumMo[0][2]+=amat(tkap)*amat(tdel);
	  sumMo[0][3]+=amat(tkap)*bmat(tdel);
	  sumMo[0][4]+=amat(tdel)*bmat(tkap);
	  sumMo[0][5]+=amat(tdel)*amat(tdel);
	  sumMo[0][6]+=amat(tdel)*bmat(tdel);
	  sumMo[0][7]+=bmat(tdel)*bmat(tdel);
	  sumMo[0][8]+=bmat(tdel)*bmat(tkap);
	  sumMo[0][9]+=bmat(tkap)*bmat(tkap);
	  for(v=0;v<nbc;v++)
	    {
	      //tell_code(t) gives code of t
	      if(v==seq.tell_int(t))
		{ 		  
		  sumBo[0][v][0]+=amat(tkap);
		  sumBo[0][v][1]+=bmat(tkap);
		  sumBo[0][v][2]+=amat(tdel);
		  sumBo[0][v][3]+=bmat(tdel);		     	     
		}
	    }	    	     	
	}
      //sumMa, sumMr and sumBa, sumBr
      //First segment
      for(t=_order;(unsigned long)(t)<=tab[1];t++)
	{ 
	  td= double(t);  
	  tkap=(td-double(tab[0]))/(double(tab[1])-double(tab[0]));
	  tdel=(double(tab[1])-td)/(double(tab[1])-double(tab[0]));
	  sumMa[0][0][0]+=amat(tdel)*amat(tdel);
	  sumMa[0][0][1]+=amat(tdel)*bmat(tdel);
	  sumMa[0][0][2]+=bmat(tdel)*bmat(tdel);
		      
	  sumMr[0][0][0]+=amat(tkap)*amat(tkap);
	  sumMr[0][0][1]+=amat(tkap)*bmat(tkap);
	  sumMr[0][0][2]+=bmat(tkap)*bmat(tkap);

	  sumBaB[0][0][0]+=amat(tdel)*amat(tkap);
	  sumBaB[0][0][1]+=amat(tdel)*bmat(tkap);
	  sumBaB[0][0][2]+=bmat(tdel)*amat(tkap); 
	  sumBaB[0][0][3]+=bmat(tdel)*bmat(tkap);

	  sumBrB[0][0][0]+=amat(tkap)*amat(tdel);
	  sumBrB[0][0][1]+=amat(tkap)*bmat(tdel);
	  sumBrB[0][0][2]+=bmat(tkap)*amat(tdel); 
	  sumBrB[0][0][3]+=bmat(tkap)*bmat(tdel);
	  for(v=0;v<nbc;v++)
	    {
	      //tell_code(t) gives code of t
	      if(v==seq.tell_int(t))
		{ 		  		      
		  sumBa[0][0][v][0]+=amat(tdel);
		  sumBa[0][0][v][1]+=bmat(tdel);
		     
		  sumBr[0][0][v][0]+=amat(tkap);
		  sumBr[0][0][v][1]+=bmat(tkap);		    
		}	
	    }
	}
      //Others segments
      for(k=1;k<_N;k++)
	{
	  for(t=tab[k]+1;(unsigned long)(t)<=tab[k+1];t++)
	    {  
	      td= double(t);
	      tkap=(td-double(tab[k]))/(double(tab[k+1])-double(tab[k]));
	      tdel=(double(tab[k+1])-td)/(double(tab[k+1])-double(tab[k]));
	      sumMa[k][0][0]+=amat(tdel)*amat(tdel);
	      sumMa[k][0][1]+=amat(tdel)*bmat(tdel);
	      sumMa[k][0][2]+=bmat(tdel)*bmat(tdel);
		      
	      sumMr[k][0][0]+=amat(tkap)*amat(tkap);
	      sumMr[k][0][1]+=amat(tkap)*bmat(tkap);
	      sumMr[k][0][2]+=bmat(tkap)*bmat(tkap);

	      sumBaB[k][0][0]+=amat(tdel)*amat(tkap);
	      sumBaB[k][0][1]+=amat(tdel)*bmat(tkap);
	      sumBaB[k][0][2]+=bmat(tdel)*amat(tkap); 
	      sumBaB[k][0][3]+=bmat(tdel)*bmat(tkap);

	      sumBrB[k][0][0]+=amat(tkap)*amat(tdel);
	      sumBrB[k][0][1]+=amat(tkap)*bmat(tdel);
	      sumBrB[k][0][2]+=bmat(tkap)*amat(tdel); 
	      sumBrB[k][0][3]+=bmat(tkap)*bmat(tdel);				 

	      for(v=0;v<nbc;v++)
		{ 
		  //tell_code(t) gives code of t
		  if(v==seq.tell_int(t))
		    { 		      
		      sumBa[k][0][v][0]+=amat(tdel);
		      sumBa[k][0][v][1]+=bmat(tdel);

		      sumBr[k][0][v][0]+=amat(tkap);
		      sumBr[k][0][v][1]+=bmat(tkap);
		    }
		}	    	     
	    }
	}
    }
  else
    {
      int ud=0;
      for(i=1;i<_order;i++)
	{
	  ud+=int(pow(double(_alphabet_size),i));
	}  
      int uf=ud+int(pow(double(_alphabet_size),i));
      //sumMo and sumBo
      for(t=_order;(unsigned long)(t)<=tab[1];t++)
	{ 
	  td= double(t);
	  tkap=(td-double(tab[0]))/(double(tab[1])-double(tab[0]));
	  tdel=(double(tab[1])-td)/(double(tab[1])-double(tab[0]));
	  for(u=ud;u<uf;u++)
	    {
	      //Code of markovian past: 
	      //Example A={a,b} 
	      //Order 1: a=0, b=1; order 2: aa=2, ab=3, ba=4, bb=5; order 3:  aaa=6, etc...
	      //tell_code(t) gives code of t-order...t
	      if(u == seq.tell_code(t)/_alphabet_size-1)
		{	   	      
		  sumMo[u-ud][0]+=amat(tkap)*amat(tkap);
		  sumMo[u-ud][1]+=amat(tkap)*bmat(tkap);
		  sumMo[u-ud][2]+=amat(tkap)*amat(tdel);
		  sumMo[u-ud][3]+=amat(tkap)*bmat(tdel);
		  sumMo[u-ud][4]+=amat(tdel)*bmat(tkap);
		  sumMo[u-ud][5]+=amat(tdel)*amat(tdel);
		  sumMo[u-ud][6]+=amat(tdel)*bmat(tdel);
		  sumMo[u-ud][7]+=bmat(tdel)*bmat(tdel);
		  sumMo[u-ud][8]+=bmat(tdel)*bmat(tkap);
		  sumMo[u-ud][9]+=bmat(tkap)*bmat(tkap);
		  for(v=0;v<nbc;v++)
		    {
		      //tell_code(t) gives code of t
		      if(v==seq.tell_int(t))
			{ 		  
			  sumBo[u-ud][v][0]+=amat(tkap);
			  sumBo[u-ud][v][1]+=bmat(tkap);
			  sumBo[u-ud][v][2]+=amat(tdel);
			  sumBo[u-ud][v][3]+=bmat(tdel);		     	     
			}
		    }	    	     
		}
	    }
	}

      //sumMa, sumMr and sumBa, sumBr
      //First segment
      for(t=_order;(unsigned long)(t)<=tab[1];t++)
	{ 
	  td= double(t);  
	  tkap=(td-double(tab[0]))/(double(tab[1])-double(tab[0]));
	  tdel=(double(tab[1])-td)/(double(tab[1])-double(tab[0]));
	  for(u=ud;u<uf;u++)
	    {
	      //Code of markovian past: 
	      //Example A={a,b} 
	      //Order 1: a=0, b=1; order 2: aa=2, ab=3, ba=4, bb=5; order 3:  aaa=6, etc...
	      //tell_code(t) gives code of t-order...t
	      if(u == seq.tell_code(t)/_alphabet_size-1)
		{	   
		  sumMa[0][u-ud][0]+=amat(tdel)*amat(tdel);
		  sumMa[0][u-ud][1]+=amat(tdel)*bmat(tdel);
		  sumMa[0][u-ud][2]+=bmat(tdel)*bmat(tdel);
		      
		  sumMr[0][u-ud][0]+=amat(tkap)*amat(tkap);
		  sumMr[0][u-ud][1]+=amat(tkap)*bmat(tkap);
		  sumMr[0][u-ud][2]+=bmat(tkap)*bmat(tkap);

		  sumBaB[0][u-ud][0]+=amat(tdel)*amat(tkap);
		  sumBaB[0][u-ud][1]+=amat(tdel)*bmat(tkap);
		  sumBaB[0][u-ud][2]+=bmat(tdel)*amat(tkap); 
		  sumBaB[0][u-ud][3]+=bmat(tdel)*bmat(tkap);

		  sumBrB[0][u-ud][0]+=amat(tkap)*amat(tdel);
		  sumBrB[0][u-ud][1]+=amat(tkap)*bmat(tdel);
		  sumBrB[0][u-ud][2]+=bmat(tkap)*amat(tdel); 
		  sumBrB[0][u-ud][3]+=bmat(tkap)*bmat(tdel);
		  for(v=0;v<nbc;v++)
		    {
		      //tell_code(t) gives code of t
		      if(v==seq.tell_int(t))
			{ 		  		      
			  sumBa[0][u-ud][v][0]+=amat(tdel);
			  sumBa[0][u-ud][v][1]+=bmat(tdel);
		     
			  sumBr[0][u-ud][v][0]+=amat(tkap);
			  sumBr[0][u-ud][v][1]+=bmat(tkap);		    
			}
		    }	    	     
		}
	    }
	}

      //Others segments
      for(k=1;k<_N;k++)
	{
	  for(t=tab[k]+1;(unsigned long)(t)<=tab[k+1];t++)
	    {  
	      td= double(t);
	      tkap=(td-double(tab[k]))/(double(tab[k+1])-double(tab[k]));
	      tdel=(double(tab[k+1])-td)/(double(tab[k+1])-double(tab[k]));
	      for(u=ud;u<uf;u++)
		{
		  //Code of markovian past: 
		  //Example A={a,b} 
		  //Order 1: a=0, b=1; order 2: aa=2, ab=3, ba=4, bb=5; order 3:  aaa=6, etc...
		  //tell_code(t) gives code of t-order...t
		  if(u == seq.tell_code(t)/_alphabet_size-1)
		    {	 
		      sumMa[k][u-ud][0]+=amat(tdel)*amat(tdel);
		      sumMa[k][u-ud][1]+=amat(tdel)*bmat(tdel);
		      sumMa[k][u-ud][2]+=bmat(tdel)*bmat(tdel);
		      
		      sumMr[k][u-ud][0]+=amat(tkap)*amat(tkap);
		      sumMr[k][u-ud][1]+=amat(tkap)*bmat(tkap);
		      sumMr[k][u-ud][2]+=bmat(tkap)*bmat(tkap);

		      sumBaB[k][u-ud][0]+=amat(tdel)*amat(tkap);
		      sumBaB[k][u-ud][1]+=amat(tdel)*bmat(tkap);
		      sumBaB[k][u-ud][2]+=bmat(tdel)*amat(tkap); 
		      sumBaB[k][u-ud][3]+=bmat(tdel)*bmat(tkap);

		      sumBrB[k][u-ud][0]+=amat(tkap)*amat(tdel);
		      sumBrB[k][u-ud][1]+=amat(tkap)*bmat(tdel);
		      sumBrB[k][u-ud][2]+=bmat(tkap)*amat(tdel); 
		      sumBrB[k][u-ud][3]+=bmat(tkap)*bmat(tdel);				 

		      for(v=0;v<nbc;v++)
			{ 
			  //tell_code(t) gives code of t
			  if(v==seq.tell_int(t))
			    { 		      
			      sumBa[k][u-ud][v][0]+=amat(tdel);
			      sumBa[k][u-ud][v][1]+=bmat(tdel);

			      sumBr[k][u-ud][v][0]+=amat(tkap);
			      sumBr[k][u-ud][v][1]+=bmat(tkap);
			    }
			}	    	     
		    }
		}
	    }
	}
    }
  delete[] tab;
}

//###################################################################################
 
void Splines::init_o(double **** Mo, double *** Bo, double ** sumMo, double *** sumBo)
{
  int u,v,i;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  double nu;

  for(u=0;u<nbl;u++)
    {   
      nu=(double(_tab[0]-double(_tab[1])))/(double(_tab[2])-double(_tab[1]));
      for(v=0;v<nbc;v++)
	{
	  Mo[u][v][0][0] = sumMo[u][0];
	  Mo[u][v][1][0] = sumMo[u][1];
	  Mo[u][v][2][0] = sumMo[u][2];
	  Mo[u][v][3][0] = nu*sumMo[u][3];
	  Mo[u][v][0][1] = sumMo[u][1];
	  Mo[u][v][1][1] = sumMo[u][9];
	  Mo[u][v][2][1] = sumMo[u][4];
	  Mo[u][v][3][1] = nu*sumMo[u][8];
	  Mo[u][v][0][2] = sumMo[u][2];
	  Mo[u][v][1][2] = sumMo[u][4];
	  Mo[u][v][2][2] = sumMo[u][5];
	  Mo[u][v][3][2] = nu*sumMo[u][6];
	  Mo[u][v][0][3] = sumMo[u][3];
	  Mo[u][v][1][3] = sumMo[u][8];
	  Mo[u][v][2][3] = sumMo[u][6];
	  Mo[u][v][3][3] = nu*sumMo[u][7];
     
	  Bo[u][v][0] = sumBo[u][v][0];
	  Bo[u][v][1] = sumBo[u][v][1];	
	  Bo[u][v][2] = sumBo[u][v][2];
	  Bo[u][v][3] = sumBo[u][v][3];	  	
	}
    }
}

//###################################################################################


void Splines::solve_o(double **** Mo, double *** Bo)
{
  int u,i,v;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int dim=4;
  int NN=dim;
  int nrhs=1;
  int lda=dim;
  int ldb=dim;
  int info;
  int * ipiv = new int[NN];
  for(u=0;u<nbl;u++)
    {
      for(v=0;v<nbc;v++)
	{
	  dgesv_( NN, nrhs, Mo[u][v][0], lda, ipiv, Bo[u][v], ldb, info );
	}
    }
  delete[] ipiv;
}

//###################################################################################

 void Splines::init_a(double ***** Ma, double **** Ba,double *** sumMa, double **** sumBa, double *** sumBaB, double **** Br, int m)
{
  int u,v,i;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  double nu;

  if(m==0)
    {
      //first segment
      nu=(double(_tab[0]-double(_tab[1])))/(double(_tab[2])-double(_tab[1]));
      for(u=0;u<nbl;u++)
	{
	  for(v=0;v<nbc;v++)
	    {
	      Ma[0][u][v][0][0] = sumMa[0][u][0];
	      Ma[0][u][v][1][0] = nu*sumMa[0][u][1];
	      Ma[0][u][v][0][1] = sumMa[0][u][1];
	      Ma[0][u][v][1][1] = nu* sumMa[0][u][2]; 
	      
	      Ba[0][u][v][0] = sumBa[0][u][v][0]- Br[0][u][v][0]*sumBaB[0][u][0]- Br[0][u][v][1]*sumBaB[0][u][1];
	      Ba[0][u][v][1] = sumBa[0][u][v][1]- Br[0][u][v][0]*sumBaB[0][u][2]- Br[0][u][v][1]*sumBaB[0][u][3];	  
	    }
	}
      solve(Ma,Ba,0);
    }
  else 
    {
      if(m==_N-1)
	{
	  //Last segment
	  for(u=0;u<nbl;u++)
	    {
	      for(v=0;v<nbc;v++)
		{
		  Ma[_N-1][u][v][0][0] = sumMa[_N-1][u][0];
		  Ma[_N-1][u][v][1][0] = sumMa[_N-1][u][1];
		  Ma[_N-1][u][v][0][1] = sumMa[_N-1][u][1];
		  Ma[_N-1][u][v][1][1] = sumMa[_N-1][u][2]; 

		  Ba[_N-1][u][v][0] = sumBa[_N-1][u][v][0]- Ba[_N-2][u][v][0]*sumBaB[_N-1][u][0]- Ba[_N-2][u][v][1]*sumBaB[_N-1][u][1];
		  Ba[_N-1][u][v][1] = sumBa[_N-1][u][v][1]- Ba[_N-2][u][v][0]*sumBaB[_N-1][u][2]- Ba[_N-2][u][v][1]*sumBaB[_N-1][u][3];
		}
	    }
	  solve(Ma,Ba,_N-1);
	}
      else
	{
	  //other segments
	  nu=(double(_tab[m]-double(_tab[m+1])))/(double(_tab[m+2])-double(_tab[m+1]));
	  for(u=0;u<nbl;u++)
	    {
	      for(v=0;v<nbc;v++)
		{
		  Ma[m][u][v][0][0] = sumMa[m][u][0];
		  Ma[m][u][v][1][0] = nu*sumMa[m][u][1];
		  Ma[m][u][v][0][1] = sumMa[m][u][1];
		  Ma[m][u][v][1][1] = nu* sumMa[m][u][2]; 

		  Ba[m][u][v][0] = sumBa[m][u][v][0]- Ba[m-1][u][v][0]*sumBaB[m][u][0]- Ba[m-1][u][v][1]*sumBaB[m][u][1];
		  Ba[m][u][v][1] = sumBa[m][u][v][1]- Ba[m-1][u][v][0]*sumBaB[m][u][2]- Ba[m-1][u][v][1]*sumBaB[m][u][3];
		}
	    }
	  solve(Ma,Ba,m);
	}
    }
}

//###################################################################################

void Splines::init_r(double ***** Mr, double **** Br,double *** sumMr, double **** sumBr, double *** sumBrB, double **** Ba, int m)
{
  int u,v,i;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  double nu;
 
  if(m==_N-1)
    {
      //Last segment
      for(u=0;u<nbl;u++)
    	{
    	  for(v=0;v<nbc;v++)
    	    {
    	      Mr[_N-1][u][v][0][0] = sumMr[_N-1][u][0];
    	      Mr[_N-1][u][v][1][0] = sumMr[_N-1][u][1];
    	      Mr[_N-1][u][v][0][1] = sumMr[_N-1][u][1];
    	      Mr[_N-1][u][v][1][1] = sumMr[_N-1][u][2]; 

    	      Br[_N-1][u][v][0] = sumBr[_N-1][u][v][0]- Ba[_N-1][u][v][0]*sumBrB[_N-1][u][0]- Ba[_N-1][u][v][1]*sumBrB[_N-1][u][1];
    	      Br[_N-1][u][v][1] = sumBr[_N-1][u][v][1]- Ba[_N-1][u][v][0]*sumBrB[_N-1][u][2]- Ba[_N-1][u][v][1]*sumBrB[_N-1][u][3]; 	       
    	    }
    	}
      solve(Mr,Br,_N-1);
    }
  else
    {
      //other segments
      nu=(double(_tab[m]-double(_tab[m+1])))/(double(_tab[m+2])-double(_tab[m+1]));
      for(u=0;u<nbl;u++)
	{	 
	  for(v=0;v<nbc;v++)
	    {
	      Mr[m][u][v][0][0] = sumMr[m][u][0];
	      Mr[m][u][v][1][0] = sumMr[m][u][1];
	      Mr[m][u][v][0][1] = sumMr[m][u][1];
	      Mr[m][u][v][1][1] = sumMr[m][u][2]; 
	      
	      Br[m][u][v][0] = sumBr[m][u][v][0]- Br[m+1][u][v][0]*sumBrB[m][u][0]- Br[m+1][u][v][1]*sumBrB[m][u][1]*nu;
	      Br[m][u][v][1] = sumBr[m][u][v][1]- Br[m+1][u][v][0]*sumBrB[m][u][2]- Br[m+1][u][v][1]*sumBrB[m][u][3]*nu;
	    }
	}
      solve(Mr,Br,m);
    }
}

//###################################################################################
void Splines::solve(double ***** M, double **** B, int m)
{ 
  double AMIN=0.05;
  double BMAX=1;
  double BMIN=-BMAX;

  int u,v,info;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int dim=2;
  int NN=dim;
  int nrhs=1;
  int lda=dim;
  int ldb=dim;
  int * ipiv = new int[NN];
  for(u=0;u<nbl;u++)
    {	 
      for(v=0;v<nbc;v++)
	{	
	  dgesv_( NN, nrhs, M[m][u][v][0], lda, ipiv, B[m][u][v], ldb, info );
	}	 
    }	
  delete[] ipiv;
  
  normalA(B[m],nbl,nbc, AMIN);
  normalB(B[m],nbl,nbc, BMIN, BMAX);
}
//###################################################################################

// Constructor 1 : 1 sequence.

Splines::Splines
(const Sequence & seq, int N, int Nar)
: Drifting_Markov(seq)
{
  //Initialization of private members
  _degree=3;
  _N=N; 
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int i,j,k,c,m,u,v,t;
  _K=2*(_N+1)*(nbc-1)*nbl;
  _Nar=Nar;
  int dim;
  dim=2;

  _tab= new unsigned long [_N+1];
  init_tab(_tab,_n);

  //Allocation of matrices table
  _TM= new double *** [_N+1];
  for(m=0;m<_N+1;m++)
    {
      _TM[m] = new double ** [dim];
      for(c=0;c<dim;c++)
	{
	  _TM[m][c] = new double * [nbl];
	  _TM[m][c][0]= new double [nbl*nbc];
	  for(i=1;i<nbl;i++)
	    {
	      _TM[m][c][i] = _TM[m][c][i-1]+ nbc;
	    }
	}
    }

  //M B initialisation
  double **** Mo= new double *** [nbl];
  double *** Bo= new double ** [nbl];  
  alloc_o(Mo,Bo);

  //M B aller
  double ***** Ma= new double **** [_N];
  double **** Ba= new double *** [_N];
  alloc_a(Ma,Ba);

  //M B retour
  double ***** Mr= new double **** [_N];
  double **** Br= new double *** [_N];
  alloc_r(Mr,Br);

  //M B initialisation
  double ** sumMo= new double * [nbl];
  double *** sumBo= new double ** [nbl];
  alloc_sum_o(sumMo,sumBo);
  
  //M B aller
  double *** sumMa= new double ** [_N];
  double **** sumBa= new double *** [_N];
  alloc_sum_a(sumMa,sumBa);
  
  //M B retour
  double *** sumMr= new double ** [_N];
  double **** sumBr= new double *** [_N]; 
  alloc_sum_r(sumMr,sumBr);

  //sumBaB aller  sumBrB retour
  double *** sumBaB= new double ** [_N];
  double *** sumBrB= new double ** [_N];
  alloc_sumB_arB(sumBaB,sumBrB);

  //Initialisation of sums
  init_sums(seq,sumMo,sumBo,sumMa,sumBa,sumMr,sumBr,sumBaB,sumBrB);

  //Mo and Bo
  init_o(Mo,Bo,sumMo,sumBo);

  //Solve of the first system
  solve_o(Mo,Bo);
 
  //Copy in Br
  dim=2;
  for(u=0;u<nbl;u++)
    {
      for(v=0;v<nbc;v++)
	{
	  for(i=0;i<dim;i++)
	    {
	      Br[0][u][v][i]=Bo[u][v][i];
	    }
	}
    }
 
  //BOUCLE ALLER RETOUR
  for(i=0;i<_Nar;i++)
    {
      //ALLER
      //Ma Ba
      for(m=0;m<_N;m++)
	{
	  init_a(Ma,Ba,sumMa,sumBa,sumBaB,Br,m);
	} 
      //RETOUR
      //Mr Br
      for(m=_N-1;m>=0;m--)
	{
	  init_r(Mr,Br,sumMr,sumBr,sumBrB,Ba,m);
	}
    }
 
  //after the come-back
  dim=2; 
  for(m=0;m<_N;m++)
    {
      for(c=0;c<dim;c++)
   	{
   	  for(i=0;i<nbl;i++)
   	    {
   	      for(j=0;j<nbc;j++)
   		{
   		  _TM[m][c][i][j] = Br[m][i][j][c];
   		}
   	    }
   	}
    }

  for(c=0;c<dim;c++)
    {
      for(i=0;i<nbl;i++)
   	{
   	  for(j=0;j<nbc;j++)
   	    {
   	      _TM[N][c][i][j] = Ba[_N-1][i][j][c];
   	    }
   	}
    }

  //======= DELETE =======

 free_o(Mo,Bo);
 delete[] Mo;
 delete[] Bo;
 free_ar(Ma,Ba,Mr, Br);
 delete[] Ma;
 delete[] Ba;
 delete[] Mr;
 delete[] Br;
 free_sum_o(sumMo,sumBo);
 delete[] sumMo;
 delete[] sumBo;
 free_sum_ar(sumMa,sumBa,sumMr,sumBr);
 delete[] sumMa;
 delete[] sumBa;
 delete[] sumMr;
 delete[] sumBr;
 free_sumB_arB(sumBaB,sumBrB);
 delete[] sumBaB;
 delete[] sumBrB;
}

//###################################################################################

// Constructor 2 : mean of many sequences (mean Pi_t).

Splines::Splines
(const SequenceSet & seqset, int N, int Nar)
  : Drifting_Markov(seqset)
{
  //Initialization of private members
  _degree=3;
  _N=N; 
  _Nar=Nar;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int i,j,k,c,m,u,v,t,dim;
  _K=2*(_N+1)*(nbc-1)*nbl; 

  _tab= new unsigned long [_N+1];
  init_tab(_tab,_n);

  dim=2;
  //Allocation of matrices table
  _TM= new double *** [_N+1];
  for(m=0;m<_N+1;m++)
    {
      _TM[m] = new double ** [dim];
      for(c=0;c<2;c++)
	{
	  _TM[m][c] = new double * [nbl];
	  _TM[m][c][0]= new double [nbl*nbc];
	  for(i=1;i<nbl;i++)
	    {
	      _TM[m][c][i] = _TM[m][c][i-1]+ nbc;
	    }
	}
    }

  //Allocation of matrices table
  double **** TM = new double *** [_N+1];  
  for(m=0;m<_N+1;m++)
    {
      TM[m] = new double ** [dim];
      for(c=0;c<2;c++)
	{
	  TM[m][c] = new double * [nbl];
	  TM[m][c][0]= new double [nbl*nbc];
	  for(i=1;i<nbl;i++)
	    {
	      TM[m][c][i] = TM[m][c][i-1]+ nbc;
	    }
	}
    }
  //Initialization of matrices table _TM to zero
  for(m=0;m<_N+1;m++)
    {
      for(c=0;c<dim;c++)
	{
	  for(i=0;i<nbl;i++)
	    {
	      for(j=0;j<nbc;j++)
		{
		  _TM[m][c][i][j] = 0;
		}
	    }
	}
    }

  int ns;
  int nbs=seqset.tell_nb_sequence();   
  for(ns=0;ns<nbs;ns++)//begin seqloop
    { 
      //BEGIN Of INITIALISATION      
      // M and B and sumM and sumB to zero!
      //M B initialisation
      double **** Mo= new double *** [nbl];
      double *** Bo= new double ** [nbl];  
      alloc_o(Mo,Bo);

      //M B aller
      double ***** Ma= new double **** [_N];
      double **** Ba= new double *** [_N];
      alloc_a(Ma,Ba);

      //M B retour
      double ***** Mr= new double **** [_N];
      double **** Br= new double *** [_N];
      alloc_r(Mr,Br);

      //M B initialisation
      double ** sumMo= new double * [nbl];
      double *** sumBo= new double ** [nbl];
      alloc_sum_o(sumMo,sumBo);
  
      //M B aller
      double *** sumMa= new double ** [_N];
      double **** sumBa= new double *** [_N];
      alloc_sum_a(sumMa,sumBa);
  
      //M B retour
      double *** sumMr= new double ** [_N];
      double **** sumBr= new double *** [_N]; 
      alloc_sum_r(sumMr,sumBr);

      //sumBaB aller  sumBrB retour
      double *** sumBaB= new double ** [_N];
      double *** sumBrB= new double ** [_N];
      alloc_sumB_arB(sumBaB,sumBrB);
      //END OF INITIALISATION

      //Initialisation of sums
      init_sums(seqset(ns),sumMo,sumBo,sumMa,sumBa,sumMr,sumBr,sumBaB,sumBrB);

      //Mo and Bo
      init_o(Mo,Bo,sumMo,sumBo);

      //Solve of the first system
      solve_o(Mo,Bo);
 
      //Copy in Br
      dim=2;
      for(u=0;u<nbl;u++)
	{
	  for(v=0;v<nbc;v++)
	    {
	      for(i=0;i<dim;i++)
		{
		  Br[0][u][v][i]=Bo[u][v][i];
		}
	    }
	}
 
      //BOUCLE ALLER RETOUR
      for(i=0;i<_Nar;i++)
	{
	  //ALLER
	  //Ma Ba
	  for(m=0;m<_N;m++)
	    {
	      init_a(Ma,Ba,sumMa,sumBa,sumBaB,Br,m);
	    } 
	  //RETOUR
	  //Mr Br
	  for(m=_N-1;m>=0;m--)
	    {
	      init_r(Mr,Br,sumMr,sumBr,sumBrB,Ba,m);
	    }
	}
 
      //after the come-back
      dim=2; 
      for(m=0;m<_N;m++)
	{
	  for(c=0;c<dim;c++)
	    {
	      for(i=0;i<nbl;i++)
		{
		  for(j=0;j<nbc;j++)
		    {
		      TM[m][c][i][j] = Br[m][i][j][c];
		      _TM[m][c][i][j]+=TM[m][c][i][j]; 		   
		    }
		}
	    }
	} 
      for(c=0;c<dim;c++)
	{
	  for(i=0;i<nbl;i++)
	    {
	      for(j=0;j<nbc;j++)
		{
		  TM[N][c][i][j] = Ba[_N-1][i][j][c]; 
		  _TM[N][c][i][j]+=TM[N][c][i][j];
		}
	    }
	}

      //free
      free_o(Mo,Bo);
      delete[] Mo;
      delete[] Bo;
      free_ar(Ma,Ba,Mr, Br);
      delete[] Ma;
      delete[] Ba;
      delete[] Mr;
      delete[] Br;
      free_sum_o(sumMo,sumBo);
      delete[] sumMo;
      delete[] sumBo;
      free_sum_ar(sumMa,sumBa,sumMr,sumBr);
      delete[] sumMa;
      delete[] sumBa;
      delete[] sumMr;
      delete[] sumBr;
      free_sumB_arB(sumBaB,sumBrB);
      delete[] sumBaB;
      delete[] sumBrB;
    }//end seqloop

  //Divide _TM by the number of sequences
  dim=2; 
 
  for(m=0;m<_N;m++)
    {
      for(c=0;c<dim;c++)
	{
	  for(i=0;i<nbl;i++)
	    {
	      for(j=0;j<nbc;j++)
		{
		  _TM[m][c][i][j]=_TM[m][c][i][j]/double(nbs);
		}
	    }
	}
    }
  
 for(c=0;c<dim;c++)
    {
      for(i=0;i<nbl;i++)
	{
	  for(j=0;j<nbc;j++)
	    {
	      _TM[_N][c][i][j]=_TM[_N][c][i][j]/double(nbs);
	    }
	}
    }

  for(m=0;m<_N+1;m++)
    {
      for (c=0;c<2;c++)
	{
	  delete[] TM[m][c][0];
	  delete[] TM[m][c];
	}
      delete[] TM[m]; 
    }
  delete[] TM;
}

//###################################################################################

//Constructor 3 : sum of many sequences (Sum over sequences)

Splines::Splines(int N, const SequenceSet & seqset, int Nar) : Drifting_Markov(seqset)
{
  //Initialization of private members
  _degree=3;
  _N=N; 
  _Nar=Nar;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int i,j,k,c,m,u,v,t,dim;
  _K=2*(_N+1)*(nbc-1)*nbl; 

  _tab= new unsigned long [_N+1];
  init_tab(_tab,_n);

  dim=2;
  //Allocation of matrices table
  _TM= new double *** [_N+1];
  for(m=0;m<_N+1;m++)
    {
      _TM[m] = new double ** [dim];
      for(c=0;c<2;c++)
	{
	  _TM[m][c] = new double * [nbl];
	  _TM[m][c][0]= new double [nbl*nbc];
	  for(i=1;i<nbl;i++)
	    {
	      _TM[m][c][i] = _TM[m][c][i-1]+ nbc;
	    }
	}
    }

  //Initialization of matrices table _TM to zero
  for(m=0;m<_N+1;m++)
    {
      for(c=0;c<dim;c++)
	{
	  for(i=0;i<nbl;i++)
	    {
	      for(j=0;j<nbc;j++)
		{
		  _TM[m][c][i][j] = 0;
		}
	    }
	}
    }

  //BEGIN Of INITIALISATION      
  // M and B and sumM and sumB to zero!
  //M B initialisation
  double **** Mo= new double *** [nbl];
  double *** Bo= new double ** [nbl];  
  alloc_o(Mo,Bo);

  //M B aller
  double ***** Ma= new double **** [_N];
  double **** Ba= new double *** [_N];
  alloc_a(Ma,Ba);

  //M B retour
  double ***** Mr= new double **** [_N];
  double **** Br= new double *** [_N];
  alloc_r(Mr,Br);

  //M B initialisation
  double ** sumMo= new double * [nbl];
  double *** sumBo= new double ** [nbl];
  alloc_sum_o(sumMo,sumBo);
  
  //M B aller
  double *** sumMa= new double ** [_N];
  double **** sumBa= new double *** [_N];
  alloc_sum_a(sumMa,sumBa);
  
  //M B retour
  double *** sumMr= new double ** [_N];
  double **** sumBr= new double *** [_N]; 
  alloc_sum_r(sumMr,sumBr);

  //sumBaB aller  sumBrB retour
  double *** sumBaB= new double ** [_N];
  double *** sumBrB= new double ** [_N];
  alloc_sumB_arB(sumBaB,sumBrB);
  //END OF INITIALISATION

  int ns;
  int nbs=seqset.tell_nb_sequence();   
  for(ns=0;ns<nbs;ns++)//begin seqloop
    { 
      //Initialisation of sums
      init_sums(seqset(ns),sumMo,sumBo,sumMa,sumBa,sumMr,sumBr,sumBaB,sumBrB);
    }

  //Mo and Bo
  init_o(Mo,Bo,sumMo,sumBo);

  //Solve of the first system
  solve_o(Mo,Bo);
 
  //Copy in Br
  dim=2;
  for(u=0;u<nbl;u++)
    {
      for(v=0;v<nbc;v++)
	{
	  for(i=0;i<dim;i++)
	    {
	      Br[0][u][v][i]=Bo[u][v][i];
	    }
	}
    }
 
  //BOUCLE ALLER RETOUR
  for(i=0;i<_Nar;i++)
    {
      //ALLER
      //Ma Ba
      for(m=0;m<_N;m++)
	{
	  init_a(Ma,Ba,sumMa,sumBa,sumBaB,Br,m);
	} 
      //RETOUR
      //Mr Br
      for(m=_N-1;m>=0;m--)
	{
	  init_r(Mr,Br,sumMr,sumBr,sumBrB,Ba,m);
	}
    }
 
  //after the come-back
  dim=2; 
  for(m=0;m<_N;m++)
    {
      for(c=0;c<dim;c++)
	{
	  for(i=0;i<nbl;i++)
	    {
	      for(j=0;j<nbc;j++)
		{
		  _TM[m][c][i][j] = Br[m][i][j][c];
		}
	    }
	}
    } 
  for(c=0;c<dim;c++)
    {
      for(i=0;i<nbl;i++)
	{
	  for(j=0;j<nbc;j++)
	    {
	      _TM[N][c][i][j] = Ba[_N-1][i][j][c]; 
	    }
	}
    }

  //free
  free_o(Mo,Bo);
  delete[] Mo;
  delete[] Bo;
  free_ar(Ma,Ba,Mr, Br);
  delete[] Ma;
  delete[] Ba;
  delete[] Mr;
  delete[] Br;
  free_sum_o(sumMo,sumBo);
  delete[] sumMo;
  delete[] sumBo;
  free_sum_ar(sumMa,sumBa,sumMr,sumBr);
  delete[] sumMa;
  delete[] sumBa;
  delete[] sumMr;
  delete[] sumBr;
  free_sumB_arB(sumBaB,sumBrB);
  delete[] sumBaB;
  delete[] sumBrB;
}



//###################################################################################

// Constructor 4 : configuration file

Splines::Splines(char * config_file)
{
  ifstream cf(config_file);
  int method;
  //method = 3
  cf>>method;
  cf>>_alphabet_size;
  cf>>_n;
  cf>>_order;
  cf>>_degree;
  cf>>_N;

  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));

  _tPi=_alphabet_size * int(pow(double(_alphabet_size),_order));
  _K=2*(_N+1)*(nbc-1)*nbl; 

  int c,i,j,m;

  _tab= new unsigned long [_N+1];
  for (i=0;i<_N+1;i++)
    {
      cf>>_tab[i];
    }

  //Allocation of matrices table
  _TM= new double *** [_N+1];
  for(m=0;m<_N+1;m++)
    {
      _TM[m] = new double ** [2];
      for(c=0;c<2;c++)
	{
	  _TM[m][c] = new double * [nbl];
	  _TM[m][c][0]= new double [nbl*nbc];
	  for(i=1;i<nbl;i++)
	    {
	      _TM[m][c][i] = _TM[m][c][i-1]+ nbc;
	    }
	}
    }   

  //Reading of _TM
  for(m=0;m<_N+1;m++)
    {
      for(c=0; c<2; c++)
	{  
	  for(i=0;i<nbl;i++)
	    {
	      for(j=0;j<nbc;j++)
		{
		  cf>>_TM[m][c][i][j];
		}
	    }
	}
    }
  cf.close();
}


//====================================================================================
//====================================================================================


Splines::Splines(Splines & C) : Drifting_Markov(C)
{
  int c,i,j,m;

  _degree = C._degree;
  _K=C._K; 

  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order)); 

  _tab= new unsigned long [_N+1];
  for (i=0;i<_N+1;i++)
    {
      _tab[i]=C._tab[i];
    }

  //Allocation of matrices table
  _TM= new double *** [_N+1];
  for(m=0;m<_N+1;m++)
    {
      _TM[m] = new double ** [2];
      for(c=0;c<2;c++)
	{
	  _TM[m][c] = new double * [nbl];
	  _TM[m][c][0]= new double [nbl*nbc];
	  for(i=1;i<nbl;i++)
	    {
	      _TM[m][c][i] = _TM[m][c][i-1]+ nbc;
	    }
	}
    }   

  //Copy 
  for(m=0;m<_N+1;m++)
    {
      for(c=0; c<2; c++)
	{  
	  for(i=0;i<nbl;i++)
	    {
	      for(j=0;j<nbc;j++)
		{
		  _TM[m][c][i][j] = C._TM[m][c][i][j];
		}
	    }
	}
    }
}



//===========================================================================================
//===========================================================================================

////////////////
// DESTRUCTOR //
////////////////

//###########################################################################################

//Splines::~Splines(){}
Splines::~Splines()
{
  int c,m;
  for(m=0;m<_N+1;m++)
    {
      for (c=0;c<2;c++)
	{
	  delete[] _TM[m][c][0];
	  delete[] _TM[m][c];
	}
      delete[] _TM[m]; 
    }
  delete[] _TM;
  delete[] _tab;
}


//###########################################################################################
//Number of parameters
int Splines::nb_param()
{
  return _K;
}

//###########################################################################################
 
//Quadratic distance between this._TM and C._TM
void Splines::mean_distance(Splines & C)
{
  //No comparaisons if not the same order
  if((_tPi!=C._tPi)||(_degree!=C._degree))
    { 
      cout<<"==============================================================="<<endl;
      cout<<"                         mean_distance                         "<<endl;
      cout<<"                                                               "<<endl;
      cout<<"          Error : Not the same order, not comparable           "<<endl;
      cout<<"==============================================================="<<endl;
    }
  else
    {
      int c,i,j,m;
      double D1=0, D2=0; 
      int nbM=_degree+1;
      int nbc=_alphabet_size;
      int nbl= int(pow(double(_alphabet_size),_order));
      for(m=0;m<_N+1;m++)
	{
	  for(c=0;c<2;c++)
	    {
	      for(i=0;i<nbl;i++)
		{
		  for(j=0;j<nbc;j++)
		    {
		      D1+=abs(_TM[m][c][i][j]-C._TM[m][c][i][j]);
		      D2+=pow(_TM[m][c][i][j]-C._TM[m][c][i][j],double(2));
		    }
		}
	    }
	}
      double ne = double(nbM*nbl*nbc);
      cout<<"   Sum             = "<<D1<<endl;
      cout<<"   Mean            = "<<D1/ne<<endl;
      cout<<"   Sum of squares  = "<<D2<<endl;
      cout<<"   Mean of squares = "<<D2/ne<<endl;
    }
}

//###########################################################################################

//Pi[code] at position t 
//Begin at 0 and end at _tPi-1

double Splines::Pi(int t, int code)
{
  int nbc=_alphabet_size;
  double Pi_t = Pi(t,code/nbc,code%nbc);
  return Pi_t;
}

//###########################################################################################

//Pi(i,j) at position t
//i is the line number
//j is the column number
//0 <= i < nbl
//0 <= j < nbc

double Splines::Pi(int t, int i, int j)
{
  double Pi_t;
  int l;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));

  double tal0,tal1,al;
  int nseg=t/(_n/_N);
  if(nseg==_N)
    {
      nseg=nseg-1;
    }
  if ((i<0)||(j<0)||(i>=nbl)||(j>=nbc)||((unsigned long)(t)>_n-1))
     {
      cout<<"Error: (i,j) not correct or t not correct"<<endl;
      return 0;
     }
  else
    { 
      double * vect = new double [nbc];
      double sum;
      double td = double(t);
 
      if (nseg==_N-1)
	{
	  tal0=(td-double(_tab[nseg]))/(double(_tab[nseg+1])-double(_tab[nseg]));
	  tal1=(double(_tab[nseg+1])-td)/(double(_tab[nseg+1])-double(_tab[nseg]));
	  sum=0;
	  for(l=0;l<nbc;l++)
	    { 
	      vect[l]=0; 	     
	      vect[l]+=_TM[nseg][0][i][l]*amat(tal0);
	      vect[l]+=_TM[nseg][1][i][l]*bmat(tal0);
	      vect[l]+=_TM[nseg+1][0][i][l]*amat(tal1);
	      vect[l]+=_TM[nseg+1][1][i][l]*bmat(tal1);		  
		  
	      //Normalization
	      vect[l]=abs(vect[l]);
	      sum+=vect[l];
	    }
	}    
      else 
	{
	  tal0=(td-double(_tab[nseg]))/(double(_tab[nseg+1])-double(_tab[nseg]));
	  tal1=(double(_tab[nseg+1])-td)/(double(_tab[nseg+1])-double(_tab[nseg]));
	  al=(double(_tab[nseg]-double(_tab[nseg+1])))/(double(_tab[nseg+2])-double(_tab[nseg+1]));
	  sum=0;
	  for(l=0;l<nbc;l++)
	    { 
	      vect[l]=0; 	     
	      vect[l]+=_TM[nseg][0][i][l]*amat(tal0);
	      vect[l]+=_TM[nseg][1][i][l]*bmat(tal0);	  
	      vect[l]+=_TM[nseg+1][0][i][l]*amat(tal1);
	      vect[l]+=_TM[nseg+1][1][i][l]*bmat(tal1)*al;
	      
	      //Normalization
	      vect[l]=abs(vect[l]);
	      sum+=vect[l];
	    }
	}

      for(l=0;l<nbc;l++)
	{
	  vect[l]=vect[l]/sum;
	}
      Pi_t=vect[j];  
      
      delete[] vect;
      return Pi_t;
    }
}

//###########################################################################################

//Degree
int Splines::degree()
{
  return _degree;
}

//===========================================================================================

int Splines::config_file(ofstream& cf)
{
  int c,i,j,m;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int method = 3;
  
  cf<<method<<endl;
  cf<<_alphabet_size<<endl;
  cf<<_n<<endl;
  cf<<_order<<endl;
  cf<<_degree<<endl;
  cf<<_N<<endl;
  for (i=0;i<_N+1;i++)
    {
      cf<<_tab[i]<<" ";
    }
  cf<<endl<<endl;;
 
  for(m=0;m<_N+1;m++)
    {
      for(c=0; c<2; c++)
	{  
	  for(i=0;i<nbl;i++)
	    {
	      for(j=0;j<nbc;j++)
		{
		  cf<<_TM[m][c][i][j]<<" ";
		}
	      cf<<endl;
	    }
	  cf<<endl;
	}
      cf<<endl;
    }
  cf<<endl;
  return 1;
}

//###########################################################################################

