/* Splines.h
 *
 * Copyright (C) 2008 Laboratoire Statistique & Genome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef SPLINES_H
#define SPLINES_H

#include <Drifting_Markov.h>

using namespace std;

//===============================

class Splines : public Drifting_Markov
{

private:
  int _degree;           //Degree d of splines = 3 
  int _N;                //Number N of regions  
  int _Nar;              //Number of aller retour
  int _K;                //Number of parameters of the model                  
  double **** _TM;       //Drifting Markov : table of all matrices A_i, B_i
  unsigned long * _tab;  //Position of knots

  void alloc_o(double **** Mo, double *** Bo);
  void alloc_a(double ***** Ma, double **** Ba);
  void alloc_r(double ***** Mr, double **** Br);
  void alloc_sum_o(double ** sumMo, double *** sumBo);
  void alloc_sum_a(double *** sumMa, double **** sumBa);
  void alloc_sum_r(double *** sumMr, double **** sumBr);
  void alloc_sumB_arB(double *** sumBaB, double *** sumBrB);

  void free_o(double **** Mo, double *** Bo);
  void free_ar(double ***** Ma, double **** Ba,double ***** Mr, double **** Br);
  void free_sum_o(double ** sumMo, double *** sumBo);
  void free_sum_ar(double *** sumMa, double **** sumBa,double *** sumMr, double **** sumBr);
  void free_sumB_arB(double *** sumBaB, double *** sumBrB);

  void init_tab(unsigned long * tab, unsigned long seqlen);
  void init_sums(const Sequence & seq, double ** sumMo, double *** sumBo, double *** sumMa, double **** sumBa,double *** sumMr, double **** sumBr, double *** sumBaB, double *** sumBrB);

  void init_o(double **** Mo, double *** Bo, double ** sumMo, double *** sumBo);
  void solve_o(double **** Mo, double *** Bo);
  void init_a(double ***** Ma, double **** Ba,double *** sumMa, double **** sumBa, double *** sumBaB, double **** Br, int m);
  void init_r(double ***** Mr, double **** Br,double *** sumMr, double **** sumBr, double *** sumBrB, double **** Ba, int m);
  void solve(double ***** M, double **** B, int m);
 
public:

  //---------------//
  // Constructeurs //
  //---------------//
  
  Splines(const Sequence & seq, int N, int Nar);
  Splines(const SequenceSet & seqset, int N, int Nar);
  Splines(int N, const SequenceSet & seqset, int Nar);
  Splines(char * config_file);

  //Copy constructor
  Splines(Splines & C);

  //-------------//
  // Destructeur //
  //-------------// 

  virtual ~Splines();

  //----------//
  // Mthodes //
  //----------//

  //Number of parameters = k
  virtual int nb_param();

  //+++++++++++++
  //Quadratic distance between this._TM and C._TM
  void mean_distance(Splines & C);  
  //+++++++++++++  

  //Pi_t(u,v) where code=code(uv)
  //Warning: Code of aaa = 0.
  virtual double Pi(int t, int code);

  //Pi_t(i,j)
  //0<=i<=nbl-1 and 0<=j<=nbc-1
  virtual double Pi(int t, int i, int j); 

  //Degree
  virtual int degree();
 
  //Edit a configuration file of the DMM
  virtual int config_file(ofstream& cf);

};
//===========================

#endif //SPLINES_H
