/* Splines_H.cc
 *
 * Copyright (C) 2008 Laboratoire Statistique & Genome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "Splines_H.h"

//===================================================================================
//===================================================================================

//////////////////
// CONSTRUCTORS //
//////////////////

//###################################################################################


void Splines_H::alloc_o(double **** Mo, double *** Bo)
{
  int u,v,i,j;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int dim=10;
  for(u=0;u<nbl;u++)
    {
      Mo[u] = new double ** [nbc];
      for(v=0;v<nbc;v++)
	{
	  Mo[u][v]= new double * [dim];
	  Mo[u][v][0]= new double [dim*dim];
	  for(i=1;i<dim;i++)
	    {
	      Mo[u][v][i] = Mo[u][v][i-1]+ dim;
	    }
	  for(i=0;i<dim;i++)
	    {
	      for(j=0;j<dim;j++)
		{
		  Mo[u][v][i][j] = 0;
		}
	    }
	}
    }

  for(u=0;u<nbl;u++)
    {
      Bo[u] = new double * [nbc];
      Bo[u][0]= new double [nbc*dim];
      for(v=1;v<nbc;v++)
	{
	  Bo[u][v] = Bo[u][v-1]+ dim;
	}
      for(v=0;v<nbc;v++)
	{
	  for(i=0;i<dim;i++)
	    {
	      Bo[u][v][i] = 0;
	    }
	}
    }
}

//###################################################################################

void Splines_H::alloc_a(double ***** Ma, double **** Ba)
{
  int m,u,v,i,j;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int dim=6;
  for(m=0;m<_N;m++)
    {
      Ma[m] = new double *** [nbl];
      for(u=0;u<nbl;u++)
	{
	  Ma[m][u] = new double ** [nbc];
	  for(v=0;v<nbc;v++)
	    {
	      Ma[m][u][v]= new double * [dim];
	      Ma[m][u][v][0]= new double [dim*dim];
	      for(i=1;i<dim;i++)
		{
		  Ma[m][u][v][i] = Ma[m][u][v][i-1]+ dim;
		}
	      for(i=0;i<dim;i++)
		{
		  for(j=0;j<dim;j++)
		    {
		      Ma[m][u][v][i][j] = 0;
		    }
		}
	    }
	}
    }

  for(m=0;m<_N;m++)
    {
      Ba[m] = new double ** [nbl];
      for(u=0;u<nbl;u++)
	{
	  Ba[m][u] = new double * [nbc];
	  Ba[m][u][0]= new double [nbc*dim];
	  for(v=1;v<nbc;v++)
	    {
	      Ba[m][u][v] = Ba[m][u][v-1]+ dim;
	    }
	  for(v=0;v<nbc;v++)
	    {
	      for(i=0;i<dim;i++)
		{
		  Ba[m][u][v][i] = 0;
		}
	    }
	}
    }

}

//###################################################################################
void Splines_H::alloc_r(double ***** Mr, double **** Br)
{
  int m,u,v,i,j;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int dim=6;
  for(m=0;m<_N;m++)
    {
      Mr[m] = new double *** [nbl];
      for(u=0;u<nbl;u++)
	{
	  Mr[m][u] = new double ** [nbc];
	  for(v=0;v<nbc;v++)
	    {
	      Mr[m][u][v]= new double * [dim];
	      Mr[m][u][v][0]= new double [dim*dim];
	      for(i=1;i<dim;i++)
		{
		  Mr[m][u][v][i] = Mr[m][u][v][i-1]+ dim;
		}
	      for(i=0;i<dim;i++)
		{
		  for(j=0;j<dim;j++)
		    {
		      Mr[m][u][v][i][j] = 0;
		    }
		}
	    }
	}
    }
  for(m=0;m<_N;m++)
    {
      Br[m] = new double ** [nbl];
      for(u=0;u<nbl;u++)
	{
	  Br[m][u] = new double * [nbc];
	  Br[m][u][0]= new double [nbc*dim];
	  for(v=1;v<nbc;v++)
	    {
	      Br[m][u][v] = Br[m][u][v-1]+ dim;
	    }
	  for(v=0;v<nbc;v++)
	    {
	      for(i=0;i<dim;i++)
		{
		  Br[m][u][v][i] = 0;
		}
	    }
	}
    }
}

//###################################################################################
void Splines_H::alloc_sum_o(double ** sumMo, double *** sumBo)
{
  int m,u,v,i,j;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));

  int sizeBo=8;
  int sizeMo=14;
  for(u=0;u<nbl;u++)
    {     
      sumMo[u]= new double [sizeMo];	 
      for(i=0;i<sizeMo;i++)
	{
	  sumMo[u][i] = 0;
	}
    }

  for(u=0;u<nbl;u++)
    {
      sumBo[u] = new double * [nbc];
      sumBo[u][0]= new double [nbc*sizeBo];	 
      for(v=1;v<nbc;v++)
	{
	  sumBo[u][v] = sumBo[u][v-1]+ sizeBo;	     
	}
      for(v=0;v<nbc;v++)
	{
	  for(i=0;i<sizeBo;i++)
	    {
	      sumBo[u][v][i] = 0;		  
	    }
	}
    }
}

//###################################################################################
void Splines_H::alloc_sum_a(double *** sumMa, double **** sumBa)
{
  int m,u,v,i,j;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));

  int sizeB=4;
  int sizeM=7;
  for(m=0;m<_N;m++)
    {
      sumMa[m] = new double * [nbl];
      for(u=0;u<nbl;u++)
	{	  
	  sumMa[m][u]= new double [sizeM];	 
	  for(i=0;i<sizeM;i++)
	    {
	      sumMa[m][u][i] = 0;
	    }
	}
    }
  for(m=0;m<_N;m++)
    {
      sumBa[m] = new double ** [nbl];
     
      for(u=0;u<nbl;u++)
	{
	  sumBa[m][u] = new double * [nbc];
	  sumBa[m][u][0]= new double [nbc*sizeB];	 
	  for(v=1;v<nbc;v++)
	    {
	      sumBa[m][u][v] = sumBa[m][u][v-1]+ sizeB;	     
	    }
	  for(v=0;v<nbc;v++)
	    {
	      for(i=0;i<sizeB;i++)
		{
		  sumBa[m][u][v][i] = 0;		  
		}
	    }
	}
    }
}

//###################################################################################
void Splines_H::alloc_sum_r(double *** sumMr, double **** sumBr)
{
  int m,u,v,i,j;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));

  int sizeB=4;
  int sizeM=7;
  for(m=0;m<_N;m++)
    {
      sumMr[m] = new double * [nbl];
      for(u=0;u<nbl;u++)
	{
	  sumMr[m][u]= new double [sizeM];	 	 
	  for(i=0;i<sizeM;i++)
	    {
	      sumMr[m][u][i] = 0;
	    }
	}
    }

  for(m=0;m<_N;m++)
    {
      sumBr[m] = new double ** [nbl];
     
      for(u=0;u<nbl;u++)
	{
	  sumBr[m][u] = new double * [nbc];
	  sumBr[m][u][0]= new double [nbc*sizeB];	 
	  for(v=1;v<nbc;v++)
	    {
	      sumBr[m][u][v] = sumBr[m][u][v-1]+ sizeB;	     
	    }
	  for(v=0;v<nbc;v++)
	    {
	      for(i=0;i<sizeB;i++)
		{
		  sumBr[m][u][v][i] = 0;		  
		}
	    }
	}
    }
}

//###################################################################################
void Splines_H::free_o(double **** Mo, double *** Bo)
{
  int u,v;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));

  for(u=0;u<nbl;u++)
    {
      for(v=0;v<nbc;v++)
	{
	  delete[] Mo[u][v][0];
	  delete[] Mo[u][v];	    
	}
      delete[] Mo[u];
    }
  for(u=0;u<nbl;u++)
    {
      delete[] Bo[u][0];
      delete[] Bo[u];
    }
}

//###################################################################################
void Splines_H::free_ar(double ***** Ma, double **** Ba,double ***** Mr, double **** Br)
{
  int u,v,m;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));

  for(m=0;m<_N;m++)
    {
      for(u=0;u<nbl;u++)
	{
	  for(v=0;v<nbc;v++)
	    {
	      delete[] Ma[m][u][v][0];
	      delete[] Ma[m][u][v];
	      delete[] Mr[m][u][v][0];
	      delete[] Mr[m][u][v];	   	    
	    }
	  delete[] Ma[m][u];
	  delete[] Mr[m][u];
	}

      delete[] Ma[m];
      delete[] Mr[m];
    }
  for(m=0;m<_N;m++)
    {
      for(u=0;u<nbl;u++)
	{
	  delete[] Ba[m][u][0];
	  delete[] Ba[m][u];
	  delete[] Br[m][u][0];
	  delete[] Br[m][u];
	}
      delete[] Ba[m];
      delete[] Br[m];
    }
}

//###################################################################################
void Splines_H::free_sum_o(double ** sumMo, double *** sumBo)
{
  int u,v;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));

  for(u=0;u<nbl;u++)
    {
      delete[] sumMo[u];	  
    }
  for(u=0;u<nbl;u++)
    {

      delete[] sumBo[u][0];
      delete[] sumBo[u];     
    }
}

//###################################################################################
void Splines_H::free_sum_ar(double *** sumMa, double **** sumBa,double *** sumMr, double **** sumBr)
{
  int u,v,m;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));

  for(m=0;m<_N;m++)
    {
      for(u=0;u<nbl;u++)
	{
	  delete[] sumMa[m][u];	
	  delete[] sumMr[m][u];	  	  
	}
      delete[] sumMa[m];
      delete[] sumMr[m];
    }
  for(m=0;m<_N;m++)
    {
      for(u=0;u<nbl;u++)
	{

	  delete[] sumBa[m][u][0];
	  delete[] sumBa[m][u];
	  delete[] sumBr[m][u][0];
	  delete[] sumBr[m][u];	  
	}
      delete[] sumBa[m];
      delete[] sumBr[m];
    }
}

//###################################################################################

void Splines_H::init_tab(unsigned long * tab, unsigned long seqlen)
{
  //Sequence cut at tab[i]
  //Warning:
  //segment 1={tab[0],tab[1]}, 
  //segment 2={tab[1]+1,tab[2]}, 
  //segment N={tab[N-1]+1,tab[N]}.
  int i;
  tab[0]=0;
  for(i=1;i<_N;i++)
    {
      tab[i]=(unsigned long)(i)*(unsigned long)(seqlen/_N)-1;
    }
  tab[_N]=seqlen-1;
}

//###################################################################################
void Splines_H::init_alpha(unsigned long * tab, double ** alpha)
{
  int i,j;
  int nbM=_degree+1;
  double nd; 
  //Warning nd = sequence size - 1
  nd=double(_n)-1;
   
  //Table of power of tab[i]
  //alpha[i][j]= (tab[j])^i/n^i
  alpha[0]= new double [nbM*(_N+1)];
  for(i=1;i<nbM;i++)
    {
      alpha[i] = alpha[i-1]+(_N+1);
    }
  for(j=0;j<_N+1;j++)
    {  
      alpha[0][j]=1;
      for(i=1;i<nbM;i++)
	{
	  alpha[i][j]=alpha[i-1][j]*tab[j]/nd;
	}
    }
}
//###################################################################################

void Splines_H::init_sums(const Sequence & seq, double ** sumMo, double *** sumBo, double *** sumMa, double **** sumBa,double *** sumMr, double **** sumBr)
{
  int u,v,t,k,i;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int nbM=_degree+1;
  
  unsigned long seqlen= seq.tell_length();
  double nd=double(seqlen)-1;
  unsigned long * tab= new unsigned long [_N+1];
  double ** alpha = new double * [nbM];
  init_tab(tab,seqlen);
  init_alpha(tab,alpha);

  if(_order==0)
    {
      double td;
      //sumMo and sumBo
      for(t=_order;(unsigned long)(t)<=tab[1];t++)
	{ 
	  td= double(t);
	  sumMo[0][0]+=2*1;
	  sumMo[0][1]+=2*td/nd;
	  sumMo[0][2]+=2*pow(td/nd,double(2));
	  sumMo[0][3]+=2*pow(td/nd,double(3));
	  sumMo[0][4]+=2*pow(td/nd,double(4));
	  sumMo[0][5]+=2*pow(td/nd,double(5));
	  sumMo[0][6]+=2*pow(td/nd,double(6));
	  for(v=0;v<nbc;v++)
	    {
	      //tell_code(t) gives code of t
	      if(v==seq.tell_int(t))
		{ 		  
		  sumBo[0][v][0]+=2*1;
		  sumBo[0][v][1]+=2*td/nd;
		  sumBo[0][v][2]+=2*pow(td/nd,double(2));
		  sumBo[0][v][3]+=2*pow(td/nd,double(3));
		}
	    }
	}
      for(t=tab[1]+1;(unsigned long)(t)<=tab[2];t++)
	{ 
	  td= double(t);
	  sumMo[0][7]+=2*1;
	  sumMo[0][8]+=2*td/nd;
	  sumMo[0][9]+=2*pow(td/nd,double(2));
	  sumMo[0][10]+=2*pow(td/nd,double(3));
	  sumMo[0][11]+=2*pow(td/nd,double(4));
	  sumMo[0][12]+=2*pow(td/nd,double(5));
	  sumMo[0][13]+=2*pow(td/nd,double(6));		 
	  for(v=0;v<nbc;v++)
	    {
	      //tell_code(t) gives code of t
	      if(v==seq.tell_int(t))
		{ 		  
		  sumBo[0][v][4]+=2*1;
		  sumBo[0][v][5]+=2*td/nd;
		  sumBo[0][v][6]+=2*pow(td/nd,double(2));
		  sumBo[0][v][7]+=2*pow(td/nd,double(3));
		}
	    }
	}

      //sumMa and sumBa, sumMr and sumBr
      for(k=0;k<_N;k++)
	{
	  for(t=tab[k]+1;(unsigned long)(t)<=tab[k+1];t++)
	    {  
	      td= double(t);
	      sumMa[k][0][0]+=2*1;
	      sumMa[k][0][1]+=2*td/nd;
	      sumMa[k][0][2]+=2*pow(td/nd,double(2));
	      sumMa[k][0][3]+=2*pow(td/nd,double(3));
	      sumMa[k][0][4]+=2*pow(td/nd,double(4));
	      sumMa[k][0][5]+=2*pow(td/nd,double(5));
	      sumMa[k][0][6]+=2*pow(td/nd,double(6));
		      
	      sumMr[k][0][0]+=2*1;
	      sumMr[k][0][1]+=2*td/nd;
	      sumMr[k][0][2]+=2*pow(td/nd,double(2));
	      sumMr[k][0][3]+=2*pow(td/nd,double(3));
	      sumMr[k][0][4]+=2*pow(td/nd,double(4));
	      sumMr[k][0][5]+=2*pow(td/nd,double(5));
	      sumMr[k][0][6]+=2*pow(td/nd,double(6));

	      for(v=0;v<nbc;v++)
		{ 
		  //tell_code(t) gives code of t
		  if(v==seq.tell_int(t))
		    { 		      
		      sumBa[k][0][v][0]+=2*1;
		      sumBa[k][0][v][1]+=2*td/nd;
		      sumBa[k][0][v][2]+=2*pow(td/nd,double(2));
		      sumBa[k][0][v][3]+=2*pow(td/nd,double(3));

		      sumBr[k][0][v][0]+=2*1;
		      sumBr[k][0][v][1]+=2*td/nd;
		      sumBr[k][0][v][2]+=2*pow(td/nd,double(2));
		      sumBr[k][0][v][3]+=2*pow(td/nd,double(3));
		    }
		}
	    }
	}
    }

  else
    {
      int ud=0;
      for(i=1;i<_order;i++)
	{
	  ud+=int(pow(double(_alphabet_size),i));
	}  
      int uf=ud+int(pow(double(_alphabet_size),i));

      double td;
      //sumMo and sumBo
      for(t=_order;(unsigned long)(t)<=tab[1];t++)
	{ 
	  td= double(t);
	  for(u=ud;u<uf;u++)
	    {
	      //Code of markovian past: 
	      //Example A={a,b} 
	      //Order 1: a=0, b=1; order 2: aa=2, ab=3, ba=4, bb=5; order 3:  aaa=6, etc...
	      //tell_code(t) gives code of t-order...t
	      if(u == seq.tell_code(t)/_alphabet_size-1)
		{	   	      
		  sumMo[u-ud][0]+=2*1;
		  sumMo[u-ud][1]+=2*td/nd;
		  sumMo[u-ud][2]+=2*pow(td/nd,double(2));
		  sumMo[u-ud][3]+=2*pow(td/nd,double(3));
		  sumMo[u-ud][4]+=2*pow(td/nd,double(4));
		  sumMo[u-ud][5]+=2*pow(td/nd,double(5));
		  sumMo[u-ud][6]+=2*pow(td/nd,double(6));
		  for(v=0;v<nbc;v++)
		    {
		      //tell_code(t) gives code of t
		      if(v==seq.tell_int(t))
			{ 		  
			  sumBo[u-ud][v][0]+=2*1;
			  sumBo[u-ud][v][1]+=2*td/nd;
			  sumBo[u-ud][v][2]+=2*pow(td/nd,double(2));
			  sumBo[u-ud][v][3]+=2*pow(td/nd,double(3));
			}
		    }	    	     
		}
	    }
	}
      for(t=tab[1]+1;(unsigned long)(t)<=tab[2];t++)
	{ 
	  td= double(t);
	  for(u=ud;u<uf;u++)
	    {
	      //Code of markovian past: 
	      //Example A={a,b} 
	      //Order 1: a=0, b=1; order 2: aa=2, ab=3, ba=4, bb=5; order 3:  aaa=6, etc...
	      //tell_code(t) gives code of t-order...t
	      if(u == seq.tell_code(t)/_alphabet_size-1)
		{	   	      
		  sumMo[u-ud][7]+=2*1;
		  sumMo[u-ud][8]+=2*td/nd;
		  sumMo[u-ud][9]+=2*pow(td/nd,double(2));
		  sumMo[u-ud][10]+=2*pow(td/nd,double(3));
		  sumMo[u-ud][11]+=2*pow(td/nd,double(4));
		  sumMo[u-ud][12]+=2*pow(td/nd,double(5));
		  sumMo[u-ud][13]+=2*pow(td/nd,double(6));		 
		  for(v=0;v<nbc;v++)
		    {
		      //tell_code(t) gives code of t
		      if(v==seq.tell_int(t))
			{ 		  
			  sumBo[u-ud][v][4]+=2*1;
			  sumBo[u-ud][v][5]+=2*td/nd;
			  sumBo[u-ud][v][6]+=2*pow(td/nd,double(2));
			  sumBo[u-ud][v][7]+=2*pow(td/nd,double(3));
			}
		    }	    	     
		}
	    }
	}


      //sumMa and sumBa, sumMr and sumBr
      for(k=0;k<_N;k++)
	{
	  for(t=tab[k]+1;(unsigned long)(t)<=tab[k+1];t++)
	    {  
	      td= double(t);
	      for(u=ud;u<uf;u++)
		{
		  //Code of markovian past: 
		  //Example A={a,b} 
		  //Order 1: a=0, b=1; order 2: aa=2, ab=3, ba=4, bb=5; order 3:  aaa=6, etc...
		  //tell_code(t) gives code of t-order...t
		  if(u == seq.tell_code(t)/_alphabet_size-1)
		    {	 
		      sumMa[k][u-ud][0]+=2*1;
		      sumMa[k][u-ud][1]+=2*td/nd;
		      sumMa[k][u-ud][2]+=2*pow(td/nd,double(2));
		      sumMa[k][u-ud][3]+=2*pow(td/nd,double(3));
		      sumMa[k][u-ud][4]+=2*pow(td/nd,double(4));
		      sumMa[k][u-ud][5]+=2*pow(td/nd,double(5));
		      sumMa[k][u-ud][6]+=2*pow(td/nd,double(6));
		      
		      sumMr[k][u-ud][0]+=2*1;
		      sumMr[k][u-ud][1]+=2*td/nd;
		      sumMr[k][u-ud][2]+=2*pow(td/nd,double(2));
		      sumMr[k][u-ud][3]+=2*pow(td/nd,double(3));
		      sumMr[k][u-ud][4]+=2*pow(td/nd,double(4));
		      sumMr[k][u-ud][5]+=2*pow(td/nd,double(5));
		      sumMr[k][u-ud][6]+=2*pow(td/nd,double(6));

		      for(v=0;v<nbc;v++)
			{ 
			  //tell_code(t) gives code of t
			  if(v==seq.tell_int(t))
			    { 		      
			      sumBa[k][u-ud][v][0]+=2*1;
			      sumBa[k][u-ud][v][1]+=2*td/nd;
			      sumBa[k][u-ud][v][2]+=2*pow(td/nd,double(2));
			      sumBa[k][u-ud][v][3]+=2*pow(td/nd,double(3));

			      sumBr[k][u-ud][v][0]+=2*1;
			      sumBr[k][u-ud][v][1]+=2*td/nd;
			      sumBr[k][u-ud][v][2]+=2*pow(td/nd,double(2));
			      sumBr[k][u-ud][v][3]+=2*pow(td/nd,double(3));
			    }
			}	    	     
		    }
		}
	    }
	}
    }

  delete[] alpha[0];
  delete[] alpha;
  delete[] tab;
}

//###################################################################################

void Splines_H::init_o(double **** Mo, double *** Bo, double ** sumMo, double *** sumBo)
{
  int u,v,i,j,t;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int nbM=_degree+1;

  double nd=double(_n)-1;
  double ** alpha = new double * [nbM];
  init_alpha(_tab,alpha);

  //Mo and Bo
  for(u=0;u<nbl;u++)
    {   
      for(v=0;v<nbc;v++)
	{
	  for(i=0;i<4;i++)
	    {
	      for(j=0;j<4;j++)
		{
		  Mo[u][v][i][j]=sumMo[u][i+j];
		  Mo[u][v][i+4][j+4]=sumMo[u][i+j+7];
		}
	    }
	  for(j=0;j<4;j++)
	    {
	      Mo[u][v][8][j]=-alpha[j][1];
	      Mo[u][v][8][j+4]=alpha[j][1]; 
	      Mo[u][v][j][8]=-alpha[j][1];
	      Mo[u][v][j+4][8]=alpha[j][1];		 
	    }
	  for(j=1;j<4;j++)
	    {
	      Mo[u][v][9][j]=-j*alpha[j-1][1]/nd;
	      Mo[u][v][9][j+4]=j*alpha[j-1][1]/nd;
	      Mo[u][v][j][9]=-j*alpha[j-1][1]/nd;
	      Mo[u][v][j+4][9]=j*alpha[j-1][1]/nd;
	    }

	  for(i=0;i<8;i++)
	    {
	      Bo[u][v][i]=sumBo[u][v][i];
	    }
	  Bo[u][v][8]=0;
	  Bo[u][v][9]=0;
	}
    }
  delete[] alpha[0];
  delete[] alpha;
}

//###################################################################################
void Splines_H::solve_o(double **** Mo, double *** Bo)
{
  int u,v;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));    
  int dim=10;
  int NN=dim;
  int nrhs=1;
  int lda=dim;
  int ldb=dim;
  int info;
  int * ipiv = new int[NN];
  for(u=0;u<nbl;u++)
    {
      for(v=0;v<nbc;v++)
	{
	  dgesv_( NN, nrhs, Mo[u][v][0], lda, ipiv, Bo[u][v], ldb, info );
	}
    }
  delete[] ipiv;
}

//###################################################################################

void Splines_H::init_a(double ***** Ma, double **** Ba,double *** sumMa, double **** sumBa, double *** Bo, double **** Br, int m, int ar)
{
  int u,v,i,j,t;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int nbM=_degree+1;

  double nd=double(_n)-1;
  double ** alpha = new double * [nbM];
  init_alpha(_tab,alpha);

  for(u=0;u<nbl;u++)
    {
      for(v=0;v<nbc;v++)
	{
	  for(i=0;i<4;i++)
	    {
	      for(j=0;j<4;j++)
		{
		  Ma[m][u][v][i][j]=sumMa[m+1][u][i+j];
		}
	    }
	  for(j=0;j<4;j++)
	    {
	      Ma[m][u][v][4][j]= alpha[j][m+1];
	      Ma[m][u][v][j][4]= alpha[j][m+1];
	    }
	  for(j=1;j<4;j++)
	    {
	      Ma[m][u][v][5][j]= j*alpha[j-1][m+1]/nd;
	      Ma[m][u][v][j][5]= j*alpha[j-1][m+1]/nd;
	    }
	  for(i=0;i<4;i++)
	    {
	      Ba[m][u][v][i]=sumBa[m+1][u][v][i];
	    }
	}
    }
  if(m==0)
    {
      if(ar==0)
	{
	  for(u=0;u<nbl;u++)
	    {
	      for(v=0;v<nbc;v++)
		{   
		  Ba[0][u][v][4]=Bo[u][v][0]+Bo[u][v][1]*alpha[1][1]+Bo[u][v][2]*alpha[2][1]+Bo[u][v][3]*alpha[3][1]; 
		  Ba[0][u][v][5]=Bo[u][v][1]*alpha[0][1]/nd+Bo[u][v][2]*2*alpha[1][1]/nd+Bo[u][v][3]*3*alpha[2][1]/nd;
		}
	    }
	}
      else
	{
	  for(u=0;u<nbl;u++)
	    {
	      for(v=0;v<nbc;v++)
		{   
		  Ba[0][u][v][4]=Br[1][u][v][0]+Br[1][u][v][1]*alpha[1][1]
		    +Br[1][u][v][2]*alpha[2][1]+Br[1][u][v][3]*alpha[3][1];
		  
		  Ba[0][u][v][5]=Br[1][u][v][1]*alpha[0][1]/nd
		    +Br[1][u][v][2]*2*alpha[1][1]/nd + Br[1][u][v][3]*3*alpha[2][1]/nd;	 
		}
	    }
	}
    }
  else
    {
      for(u=0;u<nbl;u++)
	{
	  for(v=0;v<nbc;v++)
	    {
	      Ba[m+1][u][v][4]=Ba[m][u][v][0]+Ba[m][u][v][1]*alpha[1][m+2]
		+Ba[m][u][v][2]*alpha[2][m+2]
		+Ba[m][u][v][3]*alpha[3][m+2];
	      Ba[m+1][u][v][5]=Ba[m][u][v][1]*alpha[0][m+2]/nd
		+Ba[m][u][v][2]*2*alpha[1][m+2]/nd
		+Ba[m][u][v][3]*3*alpha[2][m+2]/nd;
	    }
	}
    }
  delete[] alpha[0];
  delete[] alpha;
}

//###################################################################################
void Splines_H::init_r(double ***** Mr, double **** Br,double *** sumMr, double **** sumBr,double **** Ba, int m)
{
  int u,v,i,j,t;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int nbM=_degree+1;

  double nd=double(_n)-1;
  double ** alpha = new double * [nbM];
  init_alpha(_tab,alpha);

  for(u=0;u<nbl;u++)
    {	 
      for(v=0;v<nbc;v++)
	{
	  for(i=0;i<4;i++)
	    {
	      for(j=0;j<4;j++)
		{
		  Mr[m][u][v][i][j]=sumMr[m-1][u][i+j];
		}
	    }
	  for(j=0;j<4;j++)
	    {
	      Mr[m][u][v][4][j]=-alpha[j][m];
	      Mr[m][u][v][j][4]=-alpha[j][m];
	    }
	  for(j=1;j<4;j++)
	    {
	      Mr[m][u][v][5][j]=-j*alpha[j-1][m]/nd;
	      Mr[m][u][v][j][5]=-j*alpha[j-1][m]/nd;
	    }
	  for(i=0;i<4;i++)
	    {
	      Br[m][u][v][i]=sumBr[m-1][u][v][i];
	    }
	}
    }
  if(m==_N-1)
    {
      for(u=0;u<nbl;u++)
	{
	  for(v=0;v<nbc;v++)
	    {     
	      Br[_N-1][u][v][4]=-Ba[_N-2][u][v][0]-Ba[_N-2][u][v][1]*alpha[1][_N-1]
		-Ba[_N-2][u][v][2]*alpha[2][_N-1]
		-Ba[_N-2][u][v][3]*alpha[3][_N-1];
	      Br[_N-1][u][v][5]=-Ba[_N-2][u][v][1]*alpha[0][_N-1]/nd
		-Ba[_N-2][u][v][2]*2*alpha[1][_N-1]/nd
		-Ba[_N-2][u][v][3]*3*alpha[2][_N-1]/nd;
	    }
	}
    }
  else
    {
      for(u=0;u<nbl;u++)
	{
	  for(v=0;v<nbc;v++)
	    {
	      Br[m-1][u][v][4]=-Br[m][u][v][0]-Br[m][u][v][1]*alpha[1][m-1]
		-Br[m][u][v][2]*alpha[2][m-1]
		-Br[m][u][v][3]*alpha[3][m-1];
	      Br[m-1][u][v][5]=-Br[m][u][v][1]*alpha[0][m-1]/nd
		-Br[m][u][v][2]*2*alpha[1][m-1]/nd
		-Br[m][u][v][3]*3*alpha[2][m-1]/nd;
	    }
	}
    }
  delete[] alpha[0];
  delete[] alpha;
}

//###################################################################################
void Splines_H::solve(double ***** M, double **** B, int m)
{
  int u,v,info;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int dim=6;
  int NN=dim;
  int nrhs=1;
  int lda=dim;
  int ldb=dim;
  int * ipiv = new int[NN];
  for(u=0;u<nbl;u++)
    {	 
      for(v=0;v<nbc;v++)
	{	
	  dgesv_( NN, nrhs, M[m][u][v][0], lda, ipiv, B[m][u][v], ldb, info );
	}	 
    }
  delete[] ipiv;
}

//###################################################################################

// Constructor 1 : 1 sequence.

Splines_H::Splines_H
(const Sequence & seq, int N, int Nar)
  : Drifting_Markov(seq)
{
  //Initialization of private members
  _degree=3; 
  _N=N;
  int nbM=_degree+1;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int i,j,k,c,m,u,v,t,ar;
  _K=nbM*nbl*(nbc-1)*_N; 
  _Nar = Nar;
  int dim;

 _tab= new unsigned long [_N+1];
  init_tab(_tab,_n);
 
  //Allocation of matrices table
  _TM= new double *** [_N];
  for(m=0;m<_N;m++)
    {
      _TM[m] = new double ** [nbM];
      for(c=0;c<nbM;c++)
	{
	  _TM[m][c] = new double * [nbl];
	  _TM[m][c][0]= new double [nbl*nbc];
	  for(i=1;i<nbl;i++)
	    {
	      _TM[m][c][i] = _TM[m][c][i-1]+ nbc;
	    }
	}
    }

  //M B initialisation
  double **** Mo= new double *** [nbl];
  double *** Bo= new double ** [nbl];
  alloc_o(Mo,Bo);

  //M B aller
  double ***** Ma= new double **** [_N];
  double **** Ba= new double *** [_N];
  alloc_a(Ma,Ba);

  //M B retour
  double ***** Mr= new double **** [_N];
  double **** Br= new double *** [_N];
  alloc_r(Mr,Br);

  //sumMo sumBo initialisation
  double ** sumMo= new double * [nbl];
  double *** sumBo= new double ** [nbl];
  alloc_sum_o(sumMo,sumBo);

  //sumMa sumBa aller
  double *** sumMa= new double ** [_N];
  double **** sumBa= new double *** [_N];
  alloc_sum_a(sumMa,sumBa);

  //sumMr sumBr retour
  double *** sumMr= new double ** [_N];
  double **** sumBr= new double *** [_N];
  alloc_sum_r(sumMr,sumBr);

  //initialisation of sums
  init_sums(seq,sumMo,sumBo,sumMa,sumBa,sumMr,sumBr);

  //Mo and Bo
  init_o(Mo,Bo,sumMo,sumBo);
  solve_o(Mo,Bo);

  //BOUCLE ALLER RETOUR
  for(ar=0;ar<_Nar;ar++)
    {
      //ALLER
      //Ma Ba 
      for(m=0;m<_N-1;m++)
  	{
	  init_a(Ma,Ba,sumMa,sumBa,Bo,Br,m,ar);
	  solve(Ma,Ba,m);
 	}
      //RETOUR  
      //Mr Br
      for(m=_N-1;m>0;m--)
	{
	  init_r(Mr,Br,sumMr,sumBr,Ba,m);
	  solve(Mr,Br,m);
	}
    }

  //after the come-back 
  for(m=0;m<_N-1;m++)
    {
      for(c=0;c<nbM;c++)
	{
	  for(i=0;i<nbl;i++)
	    {
	      for(j=0;j<nbc;j++)
		{
		  _TM[m][c][i][j] = Br[m+1][i][j][c];
		}
	    }
	}
    }
  for(c=0;c<nbM;c++)
    {
      for(i=0;i<nbl;i++)
	{
	  for(j=0;j<nbc;j++)
	    {
	      _TM[_N-1][c][i][j] = Ba[_N-2][i][j][c];
	    }
	}
    }

  //free

  //Mo Bo
  free_o(Mo,Bo);
  delete[] Mo;
  delete[] Bo;
 
  //Ma and Mr, Ba and Br
  free_ar(Ma,Ba,Mr,Br);
  delete[] Mr;
  delete[] Ma;
  delete[] Ba;
  delete[] Br;


  //sumMo sumBo
  free_sum_o(sumMo,sumBo);
  delete[] sumMo;
  delete[] sumBo;

  //sumMa and sumMr, sumBa and sumBr
  free_sum_ar(sumMa,sumBa,sumMr,sumBr);
  delete[] sumMa;
  delete[] sumMr;
  delete[] sumBa;
  delete[] sumBr;
}

//###################################################################################

// Constructor 2 : mean of many sequences (mean Pi_t).

Splines_H::Splines_H
(const SequenceSet & seqset, int N, int Nar)
  : Drifting_Markov(seqset)
{
  //Initialization of private members
  _degree=3; 
  _N=N;
  int nbM=_degree+1;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int i,j,k,c,m,u,v,t,ar;
  _K=nbM*nbl*(nbc-1)*_N; 
  _Nar = Nar;
  int dim;

 _tab= new unsigned long [_N+1];
  init_tab(_tab,_n);

  //Allocation of matrices table
  _TM= new double *** [_N];
  for(m=0;m<_N;m++)
    {
      _TM[m] = new double ** [nbM];
      for(c=0;c<nbM;c++)
	{
	  _TM[m][c] = new double * [nbl];
	  _TM[m][c][0]= new double [nbl*nbc];
	  for(i=1;i<nbl;i++)
	    {
	      _TM[m][c][i] = _TM[m][c][i-1]+ nbc;
	    }
	}
    }

  int ns;
  int nbs=seqset.tell_nb_sequence();   

  //Allocation of matrices table
  double **** TM = new double *** [_N];  
  for(m=0;m<_N;m++)
    {
      TM[m] = new double ** [nbM];
      for(c=0;c<nbM;c++)
	{
	  TM[m][c] = new double * [nbl];
	  TM[m][c][0]= new double [nbl*nbc];
	  for(i=1;i<nbl;i++)
	    {
	      TM[m][c][i] = TM[m][c][i-1]+ nbc;
	    }
	}
    }

  //Initialization of matrices table _TM to zero
  for(m=0;m<_N;m++)
    {
      for(c=0;c<nbM;c++)
	{
	  for(i=0;i<nbl;i++)
	    {
	      for(j=0;j<nbc;j++)
		{
		  _TM[m][c][i][j] = 0;
		}
	    }
	}
    }

  for(ns=0;ns<nbs;ns++)//begin seqloop
    { 
     //BEGIN Of INITIALISATION      
      //M B initialisation
      double **** Mo= new double *** [nbl];
      double *** Bo= new double ** [nbl];
      alloc_o(Mo,Bo);
      
      //M B aller
      double ***** Ma= new double **** [_N];
      double **** Ba= new double *** [_N];
      alloc_a(Ma,Ba);
      
      //M B retour
      double ***** Mr= new double **** [_N];
      double **** Br= new double *** [_N];
      alloc_r(Mr,Br);
      
      //sumMo sumBo initialisation
      double ** sumMo= new double * [nbl];
      double *** sumBo= new double ** [nbl];
      alloc_sum_o(sumMo,sumBo);
      
      //sumMa sumBa aller
      double *** sumMa= new double ** [_N];
      double **** sumBa= new double *** [_N];
      alloc_sum_a(sumMa,sumBa);
      
      //sumMr sumBr retour
      double *** sumMr= new double ** [_N];
      double **** sumBr= new double *** [_N];
      alloc_sum_r(sumMr,sumBr);

      //initialisation of sums
      init_sums(seqset(ns),sumMo,sumBo,sumMa,sumBa,sumMr,sumBr);
      
      //Mo and Bo
      init_o(Mo,Bo,sumMo,sumBo);
      solve_o(Mo,Bo);
      
      //BOUCLE ALLER RETOUR
      for(ar=0;ar<_Nar;ar++)
	{
	  //ALLER
	  //Ma Ba 
	  for(m=0;m<_N-1;m++)
	    {
	      init_a(Ma,Ba,sumMa,sumBa,Bo,Br,m,ar);
	      solve(Ma,Ba,m);
	    }
	  //RETOUR  
	  //Mr Br
	  for(m=_N-1;m>0;m--)
	    {
	      init_r(Mr,Br,sumMr,sumBr,Ba,m);
	      solve(Mr,Br,m);
	    }
	}
	  	  
      //after the come-back 
      for(m=0;m<_N-1;m++)
	{
	  for(c=0;c<nbM;c++)
	    {
	      for(i=0;i<nbl;i++)
		{
		  for(j=0;j<nbc;j++)
		    {
		      _TM[m][c][i][j] = Br[m+1][i][j][c];
		    }
		}
	    }
	}
      for(c=0;c<nbM;c++)
	{
	  for(i=0;i<nbl;i++)
	    {
	      for(j=0;j<nbc;j++)
		{
		  _TM[_N-1][c][i][j] = Ba[_N-2][i][j][c];
		}
	    }
	}
      //free

      //Mo Bo
      free_o(Mo,Bo);
      delete[] Mo;
      delete[] Bo;
 
      //Ma and Mr, Ba and Br
      free_ar(Ma,Ba,Mr,Br);
      delete[] Mr;
      delete[] Ma;
      delete[] Ba;
      delete[] Br;


      //sumMo sumBo
      free_sum_o(sumMo,sumBo);
      delete[] sumMo;
      delete[] sumBo;

      //sumMa and sumMr, sumBa and sumBr
      free_sum_ar(sumMa,sumBa,sumMr,sumBr);
      delete[] sumMa;
      delete[] sumMr;
      delete[] sumBa;
      delete[] sumBr;
    }//end seqloop

  //Divide _TM by the number of sequences

  for(m=0;m<_N;m++)
    {
      for(c=0;c<nbM;c++)
	{
	  for(i=0;i<nbl;i++)
	    {
	      for(j=0;j<nbc;j++)
		{
		  _TM[m][c][i][j]=_TM[m][c][i][j]/double(nbs);
		}
	    }
	}
    }

  //free
  for(m=0;m<_N;m++)
    {
      for (c=0;c<nbM;c++)
	{
	  delete[] TM[m][c][0];
	  delete[] TM[m][c];
	}
      delete[] TM[m]; 
    }
  delete[] TM;

}

//###################################################################################

//Constructor 3 : sum of many sequences (Sum over sequences)

Splines_H::Splines_H
(int N, const SequenceSet & seqset, int Nar)
  : Drifting_Markov(seqset)
{
  //Initialization of private members
  _degree=3; 
  _N=N;
  int nbM=_degree+1;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int i,j,k,c,m,u,v,t,ar;
  _K=nbM*nbl*(nbc-1)*_N; 
  _Nar = Nar;
  int dim;

 _tab= new unsigned long [_N+1];
  init_tab(_tab,_n);

  //Allocation of matrices table
  _TM= new double *** [_N];
  for(m=0;m<_N;m++)
    {
      _TM[m] = new double ** [nbM];
      for(c=0;c<nbM;c++)
	{
	  _TM[m][c] = new double * [nbl];
	  _TM[m][c][0]= new double [nbl*nbc];
	  for(i=1;i<nbl;i++)
	    {
	      _TM[m][c][i] = _TM[m][c][i-1]+ nbc;
	    }
	}
    }

  //Initialization of matrices table _TM to zero
  for(m=0;m<_N;m++)
    {
      for(c=0;c<nbM;c++)
	{
	  for(i=0;i<nbl;i++)
	    {
	      for(j=0;j<nbc;j++)
		{
		  _TM[m][c][i][j] = 0;
		}
	    }
	}
    }

  //BEGIN Of INITIALISATION      
  //M B initialisation
  double **** Mo= new double *** [nbl];
  double *** Bo= new double ** [nbl];
  alloc_o(Mo,Bo);
      
  //M B aller
  double ***** Ma= new double **** [_N];
  double **** Ba= new double *** [_N];
  alloc_a(Ma,Ba);
      
  //M B retour
  double ***** Mr= new double **** [_N];
  double **** Br= new double *** [_N];
  alloc_r(Mr,Br);
      
  //sumMo sumBo initialisation
  double ** sumMo= new double * [nbl];
  double *** sumBo= new double ** [nbl];
  alloc_sum_o(sumMo,sumBo);
      
  //sumMa sumBa aller
  double *** sumMa= new double ** [_N];
  double **** sumBa= new double *** [_N];
  alloc_sum_a(sumMa,sumBa);
      
  //sumMr sumBr retour
  double *** sumMr= new double ** [_N];
  double **** sumBr= new double *** [_N];
  alloc_sum_r(sumMr,sumBr);

  int ns;
  int nbs=seqset.tell_nb_sequence();   

  for(ns=0;ns<nbs;ns++)//begin seqloop
    { 
     //initialisation of sums
      init_sums(seqset(ns),sumMo,sumBo,sumMa,sumBa,sumMr,sumBr);
    }
  //Mo and Bo
  init_o(Mo,Bo,sumMo,sumBo);
  solve_o(Mo,Bo);
      
  //BOUCLE ALLER RETOUR
  for(ar=0;ar<_Nar;ar++)
    {
      //ALLER
      //Ma Ba 
      for(m=0;m<_N-1;m++)
	{
	  init_a(Ma,Ba,sumMa,sumBa,Bo,Br,m,ar);
	  solve(Ma,Ba,m);
	}
      //RETOUR  
      //Mr Br
      for(m=_N-1;m>0;m--)
	{
	  init_r(Mr,Br,sumMr,sumBr,Ba,m);
	  solve(Mr,Br,m);
	}
    }
	  	  
  //after the come-back 
  for(m=0;m<_N-1;m++)
    {
      for(c=0;c<nbM;c++)
	{
	  for(i=0;i<nbl;i++)
	    {
	      for(j=0;j<nbc;j++)
		{
		  _TM[m][c][i][j] = Br[m+1][i][j][c];
		}
	    }
	}
    }
  for(c=0;c<nbM;c++)
    {
      for(i=0;i<nbl;i++)
	{
	  for(j=0;j<nbc;j++)
	    {
	      _TM[_N-1][c][i][j] = Ba[_N-2][i][j][c];
	    }
	}
    }
  //free

  //Mo Bo
  free_o(Mo,Bo);
  delete[] Mo;
  delete[] Bo;
 
  //Ma and Mr, Ba and Br
  free_ar(Ma,Ba,Mr,Br);
  delete[] Mr;
  delete[] Ma;
  delete[] Ba;
  delete[] Br;


  //sumMo sumBo
  free_sum_o(sumMo,sumBo);
  delete[] sumMo;
  delete[] sumBo;

  //sumMa and sumMr, sumBa and sumBr
  free_sum_ar(sumMa,sumBa,sumMr,sumBr);
  delete[] sumMa;
  delete[] sumMr;
  delete[] sumBa;
  delete[] sumBr;
}


//###################################################################################

// Constructor 4 : configuration file

Splines_H::Splines_H(char * config_file)
{
  ifstream cf(config_file);
  int method; 
  //method = 2
  cf>>method;  
  cf>>_alphabet_size;
  cf>>_n;
  cf>>_order;
  cf>>_degree;
  cf>>_N;
  cf>>_Nar;

  int nbM=_degree+1;
  int dimp=2*nbM;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));

  _tPi=_alphabet_size * int(pow(double(_alphabet_size),_order));
  _K=nbM*nbl*(nbc-1)*2*_N; 

  int c,i,j,m;
  
  _tab= new unsigned long [_N+1];
  for (i=0;i<_N+1;i++)
    {
      cf>>_tab[i];
    }

  //Allocation of matrices table
  _TM= new double *** [_N];
  for(m=0;m<_N;m++)
    {
      _TM[m] = new double ** [nbM];
      for(c=0;c<nbM;c++)
	{
	  _TM[m][c] = new double * [nbl];
	  _TM[m][c][0]= new double [nbl*nbc];
	  for(i=1;i<nbl;i++)
	    {
	      _TM[m][c][i] = _TM[m][c][i-1]+ nbc;
	    }
	}
    }   

  //Reading of _TM
 for(m=0;m<_N;m++)
    {
      for(c=0; c<nbM; c++)
	{  
	  for(i=0;i<nbl;i++)
	    {
	      for(j=0;j<nbc;j++)
		{
		  cf>>_TM[m][c][i][j];
		}
	    }
	}
    }
  cf.close();
}


//====================================================================================
//====================================================================================


Splines_H::Splines_H(Splines_H & C) : Drifting_Markov(C)
{
  int c,i,j,m;

  _degree = C._degree;
  _K=C._K; 
  _Nar=C._Nar;

  int nbM=_degree+1;
  int dimp=2*nbM;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order)); 

  _tab= new unsigned long [_N+1];
  for (i=0;i<_N+1;i++)
    {
      _tab[i]=C._tab[i];
    }

  //Allocation of matrices table
  _TM= new double *** [_N];
  for(m=0;m<_N;m++)
    {
      _TM[m] = new double ** [nbM];
      for(c=0;c<nbM;c++)
	{
	  _TM[m][c] = new double * [nbl];
	  _TM[m][c][0]= new double [nbl*nbc];
	  for(i=1;i<nbl;i++)
	    {
	      _TM[m][c][i] = _TM[m][c][i-1]+ nbc;
	    }
	}
    }

  //Copy

  for(m=0;m<_N;m++)
    {
      for(c=0;c<nbM;c++)
	{
	  for(i=0;i<nbl;i++)
	    {
	      for(j=0;j<nbc;j++)
		{
		  _TM[m][c][i][j] = C._TM[m][c][i][j];
		}
	    }
	}
    }
}



//===========================================================================================
//===========================================================================================

////////////////
// DESTRUCTOR //
////////////////

//###########################################################################################

//Splines_H::~Splines_H(){}
Splines_H::~Splines_H()
{
  int c,m;
  int nbM=_degree+1;

  for(m=0;m<_N;m++)
    {
      for (c=0;c<nbM;c++)
	{
	  delete[] _TM[m][c][0];
	  delete[] _TM[m][c];
	}
      delete[] _TM[m]; 
    }
  delete[] _TM;
  delete[] _tab;
}


//###########################################################################################
//Number of parameters
int Splines_H::nb_param()
{
  return _K;
}

//###########################################################################################
 
//Quadratic distance between this._TM and C._TM
void Splines_H::mean_distance(Splines_H & C)
{
  //No comparaisons if not the same order
  if((_tPi!=C._tPi)||(_degree!=C._degree))
    { 
      cout<<"==============================================================="<<endl;
      cout<<"                         mean_distance                         "<<endl;
      cout<<"                                                               "<<endl;
      cout<<"          Error : Not the same order, not comparable           "<<endl;
      cout<<"==============================================================="<<endl;
    }
  else
    {
      int c,i,j,m;
      double D1=0, D2=0; 
      int nbM=_degree+1;
      int nbc=_alphabet_size;
      int nbl= int(pow(double(_alphabet_size),_order));

      //Others segments
      for(m=0;m<_N;m++)
	{
	  for(c=0;c<nbM;c++)
	    {
	      for(i=0;i<nbl;i++)
		{
		  for(j=0;j<nbc;j++)
		    {
		      D1+=abs(_TM[m][c][i][j]-C._TM[m][c][i][j]);
		      D2+=pow(_TM[m][c][i][j]-C._TM[m][c][i][j],double(2));
		    }
		}
	    }
	}
      double ne = double(nbM*nbl*nbc);
      cout<<"   Sum             = "<<D1<<endl;
      cout<<"   Mean            = "<<D1/ne<<endl;
      cout<<"   Sum of squares  = "<<D2<<endl;
      cout<<"   Mean of squares = "<<D2/ne<<endl;
    }
}

//###########################################################################################

//Pi[code] at position t 
//Begin at 0 and end at _tPi-1

double Splines_H::Pi(int t, int code)
{
  int nbc=_alphabet_size;
  double Pi_t = Pi(t,code/nbc,code%nbc);
  return Pi_t;
}

//###########################################################################################

//Pi(i,j) at position t
//i is the line number
//j is the column number
//0 <= i < nbl
//0 <= j < nbc

double Splines_H::Pi(int t, int i, int j)
{
  double Pi_t;
  int c,l,m;
  int nbM=_degree+1;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));

  double ** alpha = new double * [nbM];
  init_alpha(_tab,alpha);

  //Warning: we divide by sequence size - 1
  //Warning convert double to int...
  double k;
  int nseg=t/(_n/_N);
  if(nseg==_N)
    {
      nseg=nseg-1;
    }
  if(_n%2==1)
    {
      k=double(t)/double(_n-1);
    }
  else
    {
      k=double(t)/double(_n);
    }
  if ((i<0)||(j<0)||(i>=nbl)||(j>=nbc)||(k>1))
    {
      cout<<"Error: (i,j) not correct or t not correct"<<endl;
      delete[] alpha[0];
      delete[] alpha;
      return 0;
    }
  else
    { 
      double * vect = new double [nbc];
      double sum;

      sum=0;
      for(l=0;l<nbc;l++)
	{ 
	  vect[l]=0; 
	  for(c=0;c<nbM;c++)
	    {
	      vect[l]+=_TM[nseg][c][i][l]*pow(k,double(c));
	    }	  
	  //Normalization
	  vect[l]=abs(vect[l]);
	  sum+=vect[l];
	}

      for(l=0;l<nbc;l++)
 	{
	  vect[l]=vect[l]/sum;
 	}
      Pi_t=vect[j];  
      
      delete[] alpha[0];
      delete[] alpha;
      delete[] vect;
      return Pi_t;
    }
}


//###########################################################################################

//Degree
int Splines_H::degree()
{
  return _degree;
}

//===========================================================================================

int Splines_H::config_file(ofstream& cf)
{
  int i,j,c,m;
  int nbM=_degree+1;
  int dimp= 2*nbM;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int method=2;

  cf<<method<<endl;
  cf<<_alphabet_size<<endl;
  cf<<_n<<endl;
  cf<<_order<<endl;
  cf<<_degree<<endl;
  cf<<_N<<endl;
  cf<<_Nar<<endl;
  for (i=0;i<_N+1;i++)
    {
      cf<<_tab[i]<<" ";
    }
  cf<<endl<<endl;;

  for(m=0;m<_N;m++)
    {
      for(c=0; c<nbM; c++)
	{  
	  for(i=0;i<nbl;i++)
	    {
	      for(j=0;j<nbc;j++)
		{
		  cf<<_TM[m][c][i][j]<<" ";
		}
	      cf<<endl;
	    }
	  cf<<endl;
	}
      cf<<endl;
    }
  cf<<endl;
  return 1;
}

//###########################################################################################

