/* Splines_H.h
 *
 * Copyright (C) 2008 Laboratoire Statistique & Genome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef SPLINES_H_H
#define SPLINES_H_H

#include <Drifting_Markov.h>

using namespace std;

//===============================

class Splines_H : public Drifting_Markov
{

private:
  int _degree;                     //Degree d of splines = 3
  int _N;                          //Number N of regions 
  int _K;                          //Number of parameters of the model                               
  double **** _TM;                 //Drifting Markov : table of all matrices H_i^k-, H_i^k+ 
  int _Nar;                        //Number of aller retour
  unsigned long * _tab;            //Position of knots

  void alloc_o(double **** Mo, double *** Bo);
  void alloc_a(double ***** Ma, double **** Ba);
  void alloc_r(double ***** Mr, double **** Br);
  void alloc_sum_o(double ** sumMo, double *** sumBo);
  void alloc_sum_a(double *** sumMa, double **** sumBa);
  void alloc_sum_r(double *** sumMr, double **** sumBr);

  void free_o(double **** Mo, double *** Bo);
  void free_ar(double ***** Ma, double **** Ba,double ***** Mr, double **** Br);
  void free_sum_o(double ** sumMo, double *** sumBo);
  void free_sum_ar(double *** sumMa, double **** sumBa,double *** sumMr, double **** sumBr);

  void init_tab(unsigned long * tab, unsigned long seqlen);
  void init_alpha(unsigned long * tab, double ** alpha);
  void init_sums(const Sequence & seq, double ** sumMo, double *** sumBo, double *** sumMa, double **** sumBa,double *** sumMr, double **** sumBr);

  void init_o(double **** Mo, double *** Bo, double ** sumMo, double *** sumBo);
  void solve_o(double **** Mo, double *** Bo);

  void init_a(double ***** Ma, double **** Ba,double *** sumMa, double **** sumBa, double *** Bo, double **** Br, int m, int ar);
  void init_r(double ***** Mr, double **** Br,double *** sumMr, double **** sumBr, double **** Ba, int m);
  void solve(double ***** M, double **** B, int m);

public:

  //---------------//
  // Constructeurs //
  //---------------//
  
  Splines_H(const Sequence & seq, int N, int Nar);
  Splines_H(const SequenceSet & seqset, int N, int Nar);
  Splines_H(int N, const SequenceSet & seqset, int Nar);
  Splines_H(char * config_file);

  //Copy constructor
  Splines_H(Splines_H & C);

  //-------------//
  // Destructeur //
  //-------------// 

  virtual ~Splines_H();

  //----------//
  // Mthodes //
  //----------//

  //Number of parameters = k
  virtual int nb_param();

  //+++++++++++++
  //Quadratic distance between this._TM and C._TM
  void mean_distance(Splines_H & C);  
  //+++++++++++++  

  //Pi_t(u,v) where code=code(uv)
  //Warning: Code of aaa = 0.
  virtual double Pi(int t, int code);

  //Pi_t(i,j)
  //0<=i<=nbl-1 and 0<=j<=nbc-1
  virtual double Pi(int t, int i, int j); 
 
  //Degree
  virtual int degree();

  //Edit a configuration file of the DMM
  virtual int config_file(ofstream& cf);

};
//===========================

#endif //SPLINES_H_H
