/* Splines_M.cc
 *
 * Copyright (C) 2008 Laboratoire Statistique & Genome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "Splines_M.h"

//===================================================================================
//===================================================================================

//////////////////
// CONSTRUCTORS //
//////////////////

//###################################################################################

//Allocation of system matrices

void Splines_M::alloc_table(double *** M, double *** B)
{
  int nbM=_degree+1;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int i,j,c,m;

  int dim= _N*nbM+_degree*(_N-1);
  for(c=0;c<nbl;c++)
    {
      M[c] = new double * [dim];
      M[c][0] = new double [dim*dim];
      for(i=1;i<dim;i++)
	{
	  M[c][i] = M[c][i-1]+dim;
	}
      for(i=0;i<dim;i++)
	{
	  for(j=0;j<dim;j++)
	    {
	      M[c][i][j]=0;
	    } 
	} 
    }
   
  for(c=0;c<nbl;c++)
    {
      B[c] = new double * [nbc];
      B[c][0]= new double [nbc*dim];
      for(i=1;i<nbc;i++)
	{
	  B[c][i] = B[c][i-1]+ dim;
	}
      for(i=0;i<nbc;i++)
	{
	  for(j=0;j<dim;j++)
	    {
	      B[c][i][j]=0;
	    }
	}
    }
}

//###################################################################################

void Splines_M::alloc_sum(double *** sumM, double **** sumB)
{
  int nbM=_degree+1;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int i,j,c,m,t,u,v;

  int size=2*_degree+1;

  for(u=0;u<nbl;u++)
    {
      sumM[u]= new double * [size];
      sumM[u][0]= new double [size*_N];
      for(i=1;i<size;i++)
	{
	  sumM[u][i] = sumM[u][i-1]+_N;
	}
      for(i=0;i<size;i++)
	{
	  for(j=0;j<_N;j++)
	    {
	      sumM[u][i][j] = 0;

	    }
	}
    }

  for(u=0;u<nbl;u++)
    {
      sumB[u]= new double ** [nbc];
      for(v=0;v<nbc;v++)
	{
	  sumB[u][v]= new double * [size];
	  sumB[u][v][0]= new double [size*_N];
     
	  for(i=1;i<size;i++)
	    {
	      sumB[u][v][i] = sumB[u][v][i-1]+_N;
	    } 
	  for(i=0;i<size;i++)
	    {
	      for(j=0;j<_N;j++)
		{
		  sumB[u][v][i][j] = 0;
		}
	    }
	}
    }
}

//###################################################################################

void Splines_M::init_tab(unsigned long * tab, unsigned long seqlen)
{
  //Sequence cut at tab[i]
  //Warning:
  //segment 1={tab[0],tab[1]}, 
  //segment 2={tab[1]+1,tab[2]}, 
  //segment N={tab[N-1]+1,tab[N]}.
  int i;
  tab[0]=0;
  for(i=1;i<_N;i++)
    {
      tab[i]=(unsigned long)(i)*(unsigned long)(seqlen/_N)-1;
    }
  tab[_N]=seqlen-1;
}

//###################################################################################

void Splines_M::init_sum(const Sequence & seq, double *** sumM, double **** sumB)
{
  int nbM=_degree+1;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int i,j,k,c,m,t,u,v;
  int size=2*_degree+1;
  unsigned long seqlen= seq.tell_length();
  unsigned long * tab= new unsigned long [_N+1];
  init_tab(tab,seqlen);  

  if(_order==0)
    {
      //Initialisation of sumM and sumB
      double td, nd; 
      //Warning nd = sequence size - 1
      nd=double(seqlen)-1;
      double pt;

      //First segment
      for(t=_order;(unsigned long)(t)<=tab[1];t++)
	{ 
	  td= double(t);
	
	  pt=1;
	  for(j=0;j<size;j++)
	    {
	      sumM[0][j][0]+=pt;   
	      pt=pt*td/nd;
	    }
	      
	  for(v=0;v<nbc;v++)
	    {
	      //tell_code(t) gives code of t
	      if(v==seq.tell_int(t))
		{ 
		  pt=1;
		  for(j=0;j<size;j++)
		    {
		      sumB[0][v][j][0]+=pt;
		      pt=pt*td/nd;
		    }
		}
	    }	    	     
	}
	
      //Others segments
      for(k=1;k<_N;k++)
	{
	  for(t=tab[k]+1;(unsigned long)(t)<=tab[k+1];t++)
	    {  
	      td= double(t);       
	      pt=1;
	      for(j=0;j<size;j++)
		{
		  sumM[0][j][k]+=pt;   
		  pt=pt*td/nd;
		}

	      for(v=0;v<nbc;v++)
		{ 
		  //tell_code(t) gives code of t
		  if(v==seq.tell_int(t))
		    { 
		      pt=1;
		      for(j=0;j<size;j++)
			{
			  sumB[0][v][j][k]+=pt;
			  pt=pt*td/nd;
			}
		    }
		}	    	     
	    }
	}
    }
  else
    {
      //Initialisation of sumM and sumB
      int ud=0;
      i=1;
      for(i=1;i<_order;i++)
	{
	  ud+=int(pow(double(_alphabet_size),i));
	}  
      int uf=ud+int(pow(double(_alphabet_size),i));

      double td, nd; 
      //Warning nd = sequence size - 1
      nd=double(seqlen)-1;
      double pt;

      //First segment
      for(t=_order;(unsigned long)(t)<=tab[1];t++)
	{ 
	  td= double(t);
	  for(u=ud;u<uf;u++)
	    {
	      //Code of markovian past: 
	      //Example A={a,b} 
	      //Order 1: a=0, b=1; order 2: aa=2, ab=3, ba=4, bb=5; order 3:  aaa=6, etc...
	      //tell_code(t) gives code of t-order...t
	      if(u == seq.tell_code(t)/_alphabet_size-1)
		{
		  pt=1;
		  for(j=0;j<size;j++)
		    {
		      sumM[u-ud][j][0]+=pt;   
		      pt=pt*td/nd;
		    }
	      
		  for(v=0;v<nbc;v++)
		    {
		      //tell_code(t) gives code of t
		      if(v==seq.tell_int(t))
			{ 
			  pt=1;
			  for(j=0;j<size;j++)
			    {
			      sumB[u-ud][v][j][0]+=pt;
			      pt=pt*td/nd;
			    }
			}
		    }	    	     
		}
	    }
	}

      //Others segments
      for(k=1;k<_N;k++)
	{
	  for(t=tab[k]+1;(unsigned long)(t)<=tab[k+1];t++)
	    {  
	      td= double(t);       
	      for(u=ud;u<uf;u++)
		{
		  //Code of markovian past: 
		  //Example A={a,b} 
		  //Order 1: a=0, b=1; order 2: aa=2, ab=3, ba=4, bb=5; order 3:  aaa=6, etc...
		  //tell_code(t) gives code of t-order...t
		  if(u == seq.tell_code(t)/_alphabet_size-1)
		    {
		      pt=1;
		      for(j=0;j<size;j++)
			{
			  sumM[u-ud][j][k]+=pt;   
			  pt=pt*td/nd;
			}

		      for(v=0;v<nbc;v++)
			{ 
			  //tell_code(t) gives code of t
			  if(v==seq.tell_int(t))
			    { 
			      pt=1;
			      for(j=0;j<size;j++)
				{
				  sumB[u-ud][v][j][k]+=pt;
				  pt=pt*td/nd;
				}
			    }
			}	    	     
		    }
		}
	    }
	}
    }
  delete[] tab;
}


//Initialization of system matrices

void Splines_M::init_table(double *** sumM, double **** sumB, double *** M, double *** B) 
{
  int d = _degree;
  int nbM=_degree+1;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int i,j,k,c,m,t,u,v;
  int size=2*_degree+1;

  //Table of power of tab[i]
  double nd; 
  //Warning nd = sequence size - 1
  nd=double(_n)-1;
  //alpha[i][j]= (tab[j])^i/n^i
  double ** alpha = new double * [nbM];
  alpha[0]= new double [nbM*(_N+1)];
  for(i=1;i<nbM;i++)
    {
      alpha[i] = alpha[i-1]+(_N+1);
    }
  for(j=0;j<_N+1;j++)
    {  
      alpha[0][j]=1;
      for(i=1;i<nbM;i++)
	{
	  alpha[i][j]=alpha[i-1][j]*_tab[j]/nd;
	}
    }

  //Initialisation of M and B
  //======M=======
  for (c=0;c<nbl;c++)
    {	 
      //Sigma_1
      for(i=0;i<_N;i++)
	{
	  for(j=0;j<d+1;j++)
	    {
	      for(k=0;k<d+1;k++)
		{
		  M[c][i*(d+1)+k][i*(d+1)+j]= sumM[c][k+j][i];
		}
	    }
	}
      //Sigma_2
      int s1=_N*(d+1);
      for(i=0;i<_N-1;i++)
	{
	  for(j=0;j<d;j++)
	    {
	      for(k=j;k<d+1;k++)
		{
		  M[c][s1+i*d+j][i*(d+1)+k] = 
		    (factorielle(k)/factorielle(k-j))*alpha[k-j][i+1] ;
		}
	      for(k=d+1+j;k<2*(d+1);k++)
		{
		  M[c][s1+i*d+j][i*(d+1)+k]= 
		    -(factorielle(k-(d+1))/factorielle(k-(d+1+j)))*alpha[k-(d+1+j)][i+1] ;
		}
	    }
	}
      //Sigma_3
      for(i=0;i<_N-1;i++)
	{
	  for(j=0;j<d;j++)
	    {
	      for(k=j;k<d+1;k++)
		{
		  M[c][i*(d+1)+k][s1+i*d+j] = 
		    (factorielle(k)/factorielle(k-j))*alpha[k-j][i+1] ;
		}
	      for(k=d+1+j;k<2*(d+1);k++)
		{
		  M[c][i*(d+1)+k][s1+i*d+j]= 
		    -(factorielle(k-(d+1))/factorielle(k-(d+1+j)))*alpha[k-(d+1+j)][i+1] ;
		}
	    }
	}

    }

  //=======B=======

  for (c=0;c<nbl;c++)
    {	 
      for(j=0;j<nbc;j++)
	{
	  for(i=0;i<_N;i++)
	    {
	      for(k=0;k<d+1;k++)
		{
		  B[c][j][i*(d+1)+k]= sumB[c][j][k][i];
		}
	    }
	}
    }

  delete[] alpha[0];
  delete[] alpha;
}

//###################################################################################

//Solve of the system MX=B

void Splines_M::solve_systeme(double *** M, double *** B)
{ 
  int nbM=_degree+1;
  int dim= _N*nbM+_degree*(_N-1);
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int c;
  int NN=dim, nrhs=nbc, lda=dim, ldb=dim, info;
  int * ipiv = new int[NN];
  for(c=0;c<nbl;c++)
    {
      dgesv_( NN, nrhs, M[c][0], lda, ipiv, B[c][0], ldb, info );
    }
  delete[] ipiv;
}

//###################################################################################

//Allocation of matrices table
void Splines_M::alloc_TM(double **** TM) 
{
  int nbM=_degree+1;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int i,c,m;

  for(m=0;m<_N;m++)
    {
      TM[m] = new double ** [nbM];
      for(c=0;c<nbM;c++)
	{
	  TM[m][c] = new double * [nbl];
	  TM[m][c][0]= new double [nbl*nbc];
	  for(i=1;i<nbl;i++)
	    {
	      TM[m][c][i] = TM[m][c][i-1]+ nbc;
	    }
	}
    }
}

//###################################################################################

//Initialization of matrices table _TM to zero
void Splines_M::init_TM(double **** TM) 
{
  int nbM=_degree+1;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int i,j,c,m;
  for(m=0;m<_N;m++)
    {
      for(c=0; c<nbM; c++)
	{  
	  for(i=0;i<nbl;i++)
	    {
	      for(j=0;j<nbc;j++)
		{
		  TM[m][c][i][j]=0;
		}
	    }
	}
    }
}
 
//###################################################################################

// Constructor 1 : 1 sequence.

Splines_M::Splines_M (const Sequence & seq, int degree, int N)
  : Drifting_Markov(seq)
{
  //Initialization of private members
  _degree=degree; 
  _N=N;
  int d= degree;
  int nbM= _degree+1;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int i,j,k,c,m,t,u,v;
  _K=nbl*(nbc-1)*_N*(d+1); 
  
  _tab= new unsigned long [_N+1];
  init_tab(_tab,_n);
 
  //Allocation of matrices table
  _TM= new double *** [_N];
  alloc_TM(_TM);

  // M and B allocation
  double *** M = new double ** [nbl];
  double ***B = new double ** [nbl];
  alloc_table(M,B);
  
  // sumM and sumB allocation
  double *** sumM= new double ** [nbl]; 
  double **** sumB= new double *** [nbl]; 
  alloc_sum(sumM,sumB);

  // sumM and sumB initialization
  init_sum(seq,sumM,sumB);

  // M and B initialization
  init_table(sumM,sumB,M,B);

  //System
  solve_systeme(M,B);

  //Solution of the system  
  for(m=0;m<_N;m++)
    {
      for(c=0;c<nbM;c++)
	{
	  for(i=0;i<nbl;i++)
	    {
	      for(j=0;j<nbc;j++)
		{
		  _TM[m][c][i][j]=B[i][j][nbM*m+c];
		}
	    }
	}
    }

  //free 
  for(u=0;u<nbl;u++)
    {
      delete[] sumM[u][0];
      delete[] sumM[u];
      for(v=0;v<nbc;v++)
	{
	  delete[] sumB[u][v][0];
	  delete[] sumB[u][v];
	}
      delete[] sumB[u];
    }
  delete[] sumM;
  delete[] sumB;

  for(c=0;c<nbl;c++)
    {    
      delete[] M[c][0];
      delete[] M[c];
      delete[] B[c][0];
      delete[] B[c];
    }
  delete[] M;
  delete[] B;    
}


//###################################################################################

// Constructor 2 : mean of many sequences (mean Pi_t).

Splines_M::Splines_M (const SequenceSet & seqset, int degree, int N)  : Drifting_Markov(seqset)
{  
  //Initialization of private members
  _N=N;
  _degree=degree; 
  int d= degree;
  int nbM=_degree+1;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  _K=nbl*(nbc-1)*_N*(d+1); 
  
  int i,j,c,m,u,v,t,k;

  _tab= new unsigned long [_N+1];
  init_tab(_tab,_n);

  //Allocation of matrices table: mean of  Pi_t
  _TM= new double *** [_N];
  alloc_TM(_TM);
  init_TM(_TM);

  //Allocation of matrices table: Pi_t for one sequence
  double **** TM= new double *** [_N];
  alloc_TM(TM);

  int ns;
  int nbs=seqset.tell_nb_sequence(); 

  for(ns=0;ns<nbs;ns++)
    { 
      //Allocation of M and B and sumM and sumB
      double *** M = new double ** [nbl];
      double *** B = new double ** [nbl];
      alloc_table(M,B);
      double *** sumM= new double ** [nbl];
      double **** sumB= new double *** [nbl]; 
      alloc_sum(sumM,sumB);

      //Initialization of M and B and sumM and sumB
      init_sum(seqset(ns),sumM,sumB);
      init_table(sumM,sumB,M,B);

      //Solve the system
      solve_systeme(M,B);

      for(m=0;m<_N;m++)
	{
	  for(c=0;c<nbM;c++)
	    {
	      for(i=0;i<nbl;i++)
		{
		  for(j=0;j<nbc;j++)
		    {
		      TM[m][c][i][j]=B[i][j][nbM*m+c];
		    }
		}
	    }
	}
      //Add TM in _TM 
      for(m=0;m<_N;m++)
	{
	  for(c=0; c<nbM; c++)
	    {  
	      for(i=0;i<nbl;i++)
		{
		  for(j=0;j<nbc;j++)
		    {
		      _TM[m][c][i][j]+= TM[m][c][i][j];
		    }
		}
	    }
	}
      for(u=0;u<nbl;u++)
	{
	  delete[] sumM[u][0];
	  delete[] sumM[u];
	  for(v=0;v<nbc;v++)
	    {
	      delete[] sumB[u][v][0];
	      delete[] sumB[u][v];
	    }
	  delete[] sumB[u];
	}
      delete[] sumM;
      delete[] sumB;

      for(c=0;c<nbl;c++)
	{    
	  delete[] M[c][0];
	  delete[] M[c];
	  delete[] B[c][0];
	  delete[] B[c];
	}
      delete[] M;
      delete[] B;
    }

  //Divide _TM by the number of sequences
  for(m=0;m<_N;m++)
    {
      for(c=0; c<nbM; c++)
	{  
	  for(i=0;i<nbl;i++)
	    {
	      for(j=0;j<nbc;j++)
		{
		  _TM[m][c][i][j]=_TM[m][c][i][j]/double(nbs) ;
		}
	    }
	}
    }

  //free 
  for(m=0;m<_N;m++)
    {
      for(c=0;c<nbM;c++)
	{
	  delete[] TM[m][c][0];
	  delete[] TM[m][c];
	}
      delete[] TM[m];      
    }
  delete[] TM;
}

//###################################################################################

//Constructor 3 : sum of many sequences (Sum over sequences)

Splines_M::Splines_M (int degree, const SequenceSet & seqset, int N)  : Drifting_Markov(seqset)
{  
  //Initialization of private members
  _N=N;
  _degree=degree; 
  int d= _degree;
  int nbM=_degree+1;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  _K=nbl*(nbc-1)*_N*(d+1); 
 
  int i,j,c,m,u,v,t,k;

  _tab= new unsigned long [_N+1];
  init_tab(_tab,_n);

  //Allocation of matrices table: mean of  Pi_t
  _TM= new double *** [_N];
  alloc_TM(_TM);
  init_TM(_TM);

  //Allocation of M and B and sumM and sumB
  double *** M = new double ** [nbl];
  double *** B = new double ** [nbl];
  alloc_table(M,B);
  double *** sumM= new double ** [nbl];
  double **** sumB= new double *** [nbl]; 
  alloc_sum(sumM,sumB);

  int ns;
  int nbs=seqset.tell_nb_sequence(); 

  for(ns=0;ns<nbs;ns++)
    { 
      //Initialization of sumM and sumB
      init_sum(seqset(ns),sumM,sumB);
    }
  //Initialization of M and B
  init_table(sumM,sumB,M,B);
 
  //Solve the system
  solve_systeme(M,B);

  //Solution of the system  
  for(m=0;m<_N;m++)
    {
      for(c=0;c<nbM;c++)
	{
	  for(i=0;i<nbl;i++)
	    {
	      for(j=0;j<nbc;j++)
		{
		  _TM[m][c][i][j]=B[i][j][nbM*m+c];
		}
	    }
	}
    }

  //free 
  for(u=0;u<nbl;u++)
    {
      delete[] sumM[u][0];
      delete[] sumM[u];
      for(v=0;v<nbc;v++)
	{
	  delete[] sumB[u][v][0];
	  delete[] sumB[u][v];
	}
      delete[] sumB[u];
    }
  delete[] sumM;
  delete[] sumB;

  for(c=0;c<nbl;c++)
    {    
      delete[] M[c][0];
      delete[] M[c];
      delete[] B[c][0];
      delete[] B[c];
    }
  delete[] M;
  delete[] B;
}



//###################################################################################

// Constructor 4 : configuration file


Splines_M::Splines_M(char * config_file)
{
  ifstream cf(config_file);
  int method;
  //method = 1
  cf>>method;
  cf>>_alphabet_size;
  cf>>_n;
  cf>>_order;
  cf>>_degree;
  cf>>_N;

  int nbM=_degree+1;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));

  _tPi=_alphabet_size * int(pow(double(_alphabet_size),_order));
  _K=nbl*(nbc-1)*_N*nbM; 
 

  int c,i,j,k,m;

  _tab= new unsigned long [_N+1];
  for (i=0;i<_N+1;i++)
    {
      cf>>_tab[i];
    }

  //Allocation of matrices table
  _TM= new double *** [_N];
  alloc_TM(_TM);

  //Reading of TM
  for(m=0;m<_N;m++)
    {
      for(k=0;k<nbM;k++)
	{
	  for (i=0; i<nbl;i++)
	    {
	      for (j=0;j<nbc;j++)
		{
		  cf>>_TM[m][k][i][j];
		}	  
	    }
	}
    }
  cf.close();
}


//====================================================================================
//====================================================================================


Splines_M::Splines_M(Splines_M & C) : Drifting_Markov(C)
{
  int c,i,j,m;

  _degree = C._degree;
  _K=C._K; 
  _N=C._N;

  int nbM=_degree+1;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order)); 
  
  _tab= new unsigned long [_N+1];
  for (i=0;i<_N+1;i++)
    {
      _tab[i]=C._tab[i];
    }

  //Allocation of matrices table
  _TM= new double *** [_N];
  alloc_TM(_TM);

  //Copy  
  for(m=0;m<_N;m++)
    {
      for(c=0; c<nbM; c++)
	{  
	  for(i=0;i<nbl;i++)
	    {
	      for(j=0;j<nbc;j++)
		{
		  _TM[m][c][i][j] = C._TM[m][c][i][j];
		}
	    }
	}
    }
}



//===========================================================================================
//===========================================================================================

////////////////
// DESTRUCTOR //
////////////////

//###########################################################################################

//Splines_M::~Splines_M(){}
Splines_M::~Splines_M()
{
  int c,m;
  int nbM=_degree+1;
  for(m=0;m<_N;m++)
    {
      for (c=0;c<nbM;c++)
	{
	  delete[] _TM[m][c][0];
	  delete[] _TM[m][c];
	}
      delete[] _TM[m]; 
    }
  delete[] _TM;
  delete[] _tab;
}


//###########################################################################################
//Number of parameters
int Splines_M::nb_param()
{
  return _K;
}

//###########################################################################################
 
//Quadratic distance between this._TM and C._TM
void Splines_M::mean_distance(Splines_M & C)
{
  //No comparaisons if not the same order
  if((_tPi!=C._tPi)||(_degree!=C._degree))
    { 
      cout<<"==============================================================="<<endl;
      cout<<"                         mean_distance                         "<<endl;
      cout<<"                                                               "<<endl;
      cout<<"          Error : Not the same order, not comparable           "<<endl;
      cout<<"==============================================================="<<endl;
    }
  else
    {
      int c,i,j,m;
      double D1=0, D2=0; 
      int nbM=_degree+1;
      int nbc=_alphabet_size;
      int nbl= int(pow(double(_alphabet_size),_order));
      for(m=0;m<_N;m++)
	{  
	  for(c=0;c<nbM;c++)
	    {
	      for(i=0;i<nbl;i++)
		{
		  for(j=0;j<nbc;j++)
		    {
		      D1+=abs(_TM[m][c][i][j]-C._TM[m][c][i][j]);
		      D2+=pow(_TM[m][c][i][j]-C._TM[m][c][i][j],double(2));
		    }
		}
	    }
	}
      double ne = double(_N*nbM*nbl*nbc);
      cout<<"   Sum             = "<<D1<<endl;
      cout<<"   Mean            = "<<D1/ne<<endl;
      cout<<"   Sum of squares  = "<<D2<<endl;
      cout<<"   Mean of squares = "<<D2/ne<<endl;
    }
}

//###########################################################################################

//Pi[code] at position t 
//Begin at 0 and end at _tPi-1

double Splines_M::Pi(int t, int code)
{
  int nbc=_alphabet_size;
  double Pi_t = Pi(t,code/nbc,code%nbc);
  return Pi_t;
}

//###########################################################################################

//Pi(i,j) at position t
//i is the line number
//j is the column number
//0 <= i < nbl
//0 <= j < nbc

double Splines_M::Pi(int t, int i, int j)
{
  double Pi_t;
  int c,l,m;
  int nbM=_degree+1;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));

  //Warning: we divide by sequence size - 1
  //Warning convert double to int...
  double k;
  int nseg=t/(_n/_N);
  if(nseg==_N)
    {
      nseg=nseg-1;
    }
  if(_n%2==1)
    {
      k=double(t)/double(_n-1);
    }
  else
    {
      k=double(t)/double(_n);
    }

  if ((i<0)||(j<0)||(i>=nbl)||(j>=nbc)||(k>1))
    {
      cout<<"Error: code or (i,j) not correct or t not correct"<<endl;
      return 0;
    }
  else
    { 
      double * vect = new double [nbc];
      double sum;
      sum=0;
      for(l=0;l<nbc;l++)
	{ 
	  vect[l]=0; 
	  for(c=0;c<nbM;c++)
	    {
	      vect[l]+=_TM[nseg][c][i][l]*pow(k,double(c));
	    }	  
  	  //Normalization
// 	  if (vect[j]<0)
// 	    {
// 	      cout<<"Error: Pi("<<t<<","<<code<<")<0"<<endl;
// 	      return 0;
// 	    }
  	  vect[l]=abs(vect[l]);
  	  sum+=vect[l];
	}
      for(l=0;l<nbc;l++)
    	{
    	  vect[l]=vect[l]/sum;
    	}
      Pi_t=vect[j];  

      delete[] vect;
      return Pi_t;
    }
}


//###########################################################################################

//Degree
int Splines_M::degree()
{
  return _degree;
}

//===========================================================================================

int Splines_M::config_file(ofstream& cf)
{
  int c,i,j,m;
  int nbM=_degree+1;
  int nbc=_alphabet_size;
  int nbl= int(pow(double(_alphabet_size),_order));
  int method=1;

  cf<<method<<endl;
  cf<<_alphabet_size<<endl;
  cf<<_n<<endl;
  cf<<_order<<endl;
  cf<<_degree<<endl;
  cf<<_N<<endl;
  for (i=0;i<_N+1;i++)
    {
      cf<<_tab[i]<<" ";
    }
  cf<<endl<<endl;;
 
  for(m=0;m<_N;m++)
    {
      for(c=0;c<nbM;c++)
	{
	  for(i=0;i<nbl;i++)
	    {
	      for(j=0;j<nbc;j++)
		{
		  cf<<_TM[m][c][i][j]<<" ";
		}
	      cf<<endl;
	    }
	  cf<<endl;
	}
      cf<<endl;
    }
  return 1;
}

//###########################################################################################

