/* Splines_M.h
 *
 * Copyright (C) 2008 Laboratoire Statistique & Genome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef SPLINES_M_H
#define SPLINES_M_H

#include <Drifting_Markov.h>

using namespace std;

//===============================

class Splines_M : public Drifting_Markov
{

private:
  int _degree;           //Degree d of splines
  int _N;                //Number N of regions  
  int _K;                //Number of parameters of the model                  
  double **** _TM;       //Drifting Markov : table of all matrices Mi_j (d+1) matrices for each region
  unsigned long * _tab;  //Position of knots

  void alloc_table(double *** M, double *** B);  
  void alloc_sum(double *** sumM, double **** sumB);
  void init_tab(unsigned long * tab, unsigned long seqlen);
  void init_sum(const Sequence & seq, double *** sumM, double **** sumB);
  void init_table(double *** sumM, double **** sumB, double *** M, double *** B);
  void solve_systeme(double *** M, double *** B);
  void alloc_TM(double **** TM);
  void init_TM(double **** TM);

public:

  //---------------//
  // Constructeurs //
  //---------------//
  
  Splines_M(const Sequence & seq, int degree, int N);
  Splines_M(const SequenceSet & seqset, int degree, int N);
  Splines_M(int degree, const SequenceSet & seqset, int N);
  Splines_M(char * config_file);

  //Copy constructor
  Splines_M(Splines_M & C);

  //-------------//
  // Destructeur //
  //-------------// 

  virtual ~Splines_M();

  //----------//
  // Mthodes //
  //----------//

  //Number of parameters = k
  virtual int nb_param();

  //+++++++++++++
  //Quadratic distance between this._TM and C._TM
  void mean_distance(Splines_M & C);  
  //+++++++++++++  

  //Pi_t(u,v) where code=code(uv)
  //Warning: Code of aaa = 0.
  virtual double Pi(int t, int code);

  //Pi_t(i,j)
  //0<=i<=nbl-1 and 0<=j<=nbc-1
  virtual double Pi(int t, int i, int j); 

  //Degree
  virtual int degree();

  //Edit a configuration file of the DMM
  virtual int config_file(ofstream& cf);

};
//===========================

#endif //SPLINES_M_H
