/* Tools.cc
 *
 * Copyright (C) 2008 Laboratoire Statistique & Genome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "Tools.h"

//##########################################################################################

//ln n!

double lnfactorielle(int n)
{
  double f,c;
  f=0;
  c=1.0;
  while(c<=double(n))
    {

      f=f+log(c);
      c=c+1;
    }
  return f;
}

//##########################################################################################

//Calcul n!

double factorielle(int n)
{
  double f,c;
  f=1.0;
  c=1.0;
  while(c<=double(n))
    {
      f=f*c;
      c=c+1;
    }
  return f;
}

//##########################################################################################

//Calcul C_n^p

double comb(int n, int p)
{
  double f,c;
  if (p<n-p)
    {
      f=1.0;
      c=double(p+1);
      while(c<=double(n))
 	{
 	  f=f*c;
 	  c=c+1;
 	}
      f=f/factorielle(n-p);
    }
  else
    {
      f=1.0;
      c=double(n-p+1);
      while(c<=double(n))
 	{
 	  f=f*c;
 	  c=c+1;
 	} 
      f=f/factorielle(p);
    }
  return f;
}

//##########################################################################################

//Calcul ln C_n^p

double lncomb(int n, int p)
{
  double f,c;
  if (p<n-p)
    {
      f=0;
      c=double(p+1);
      while(c<=double(n))
 	{
 	  f=f+log(c);
 	  c=c+1;
 	}
      f=f-lnfactorielle(n-p);
      //f=exp(f);
    }
  else
    {
      f=0;
      c=double(n-p+1);
      while(c<=double(n))
 	{
 	  f=f+log(c);
 	  c=c+1;
 	} 
      f=f-lnfactorielle(p);
      //f=exp(f);
    }
  return f;
}

//##########################################################################################

int compare(char * a, char * b)
{
  int i;
  int s=0;
  for(i=0;i<18;i++)
    {
      s=s+(a[i]==b[i]);
    }
  if (s==18)
    {
      return 1;
    }
  return 0;
}


//##########################################################################################

int Multinomial_1_Draw ( int dim, double * param )
{
  double sum = 0.;
  double random = double(rand())/double(RAND_MAX);
  for ( int i=0; i<dim; i++)
    {
      sum += param[i];
      if ( random<sum ) return i;
      if ((sum<1)&&(i==dim-1)) 
	{
	  cerr<<"Sum of Param of Multinom.<1";
	  return 0;
	} 
    }
  return(dim-1);
}

//##########################################################################################

double amat(double t)
{
  double r;
  r= 1-3*t*t+2*t*t*t;
  return r;
}

//##########################################################################################

double bmat(double t)
{
  double r;
  r= t-2*t*t+t*t*t;
  return r;
}

//##########################################################################################

void normalA(double *** A, int nbl, int nbc, double AMIN)
{
  int i,j,u,v;
  int ** amin = new int * [nbl];
  amin[0]= new int [nbl*nbc];
  for(i=1;i<nbl;i++)
    {
      amin[i]=amin[i-1]+nbc;
    }

  for(i=0;i<nbl;i++)
    {
      for(j=0;j<nbc;j++)
	{
	  amin[i][j]=0;
	}
    }


  double sumamin=0;
  double sumla=0;

  for(u=0;u<nbl;u++)
    {
      sumamin=0;
      sumla=0;
      for(v=0;v<nbc;v++)
	{	
	  //AMIN 
	  if(A[u][v][0]<AMIN)
	    {
	      amin[u][v]=1;
	      sumamin+=1;
	      A[u][v][0]=AMIN;
	    } 
	  //Sum of good terms
	  if(amin[u][v]==0)
	    {    
	      sumla+= A[u][v][0];
	    }	   
	} 
      for(v=0;v<nbc;v++)
	{	
	  //AMIN  
	  if(amin[u][v]==0)
	    {    
	      A[u][v][0]=A[u][v][0]/sumla - sumamin*AMIN/(nbc-sumamin);
	    }
	}
    }

  delete[] amin[0];
  delete[] amin;

}
  





void normalB(double *** B, int nbl, int nbc, double BMIN, double BMAX)
{
  int i,j,u,v;
  int ** bmax = new int * [nbl];
  bmax[0]= new int [nbl*nbc];
  for(i=1;i<nbl;i++)
    {
      bmax[i]=bmax[i-1]+nbc;
    }
  
  int ** bmin = new int * [nbl];
  bmin[0]= new int [nbl*nbc];
  for(i=1;i<nbl;i++)
    {
      bmin[i]=bmin[i-1]+nbc;
    }

  for(i=0;i<nbl;i++)
    {
      for(j=0;j<nbc;j++)
	{
	  bmin[i][j]=0;
	  bmax[i][j]=0;
	}
    }

  double sumbmax=0;
  double sumbmin=0;
  double suml=0;
  double Totalb=0;
  for(u=0;u<nbl;u++)
    {
      sumbmax=0;
      sumbmin=0;
      suml=0;
      for(v=0;v<nbc;v++)
	{	
	  //BMAX BMIN
	  if(B[u][v][1]>BMAX)
	    {
	      bmax[u][v]=1;
	      sumbmax+=1;
	      B[u][v][1]=BMAX;
	    }	
	  if(B[u][v][1]<BMIN)
	    {
	      bmin[u][v]=1;
	      sumbmin+=1;
	      B[u][v][1]=BMIN;
	    }	 
	  //Sum of good terms
	  if((bmax[u][v]==0)&&(bmin[u][v]==0))
	    {
	      suml+= B[u][v][1];
	    } 
	} 
      Totalb= suml + BMIN*sumbmin + BMAX*sumbmax;
      for(v=0;v<nbc;v++)
	{	
	  //BMAX BMIN	  
	  if((bmax[u][v]==0)&&(bmin[u][v]==0))
	    {
	      B[u][v][1]=B[u][v][1]-(Totalb/double(nbc-(sumbmin+sumbmax)));
	    } 	 
	}
    }
  delete[] bmin[0];
  delete[] bmin;
  delete[] bmax[0];
  delete[] bmax;
}

void decompose(int u, int order, int size, int * tab)
{
  int i,j,k;
  j=u;
  for(i=0;i<order;i++)
    {
      k=int(pow(double(size),double(order-(i+1))));
      tab[i]=j/k;
      j=j%k;
    }
}

//##########################################################################################
