/* Word.cc
 *
 * Copyright (C) 2008 Laboratoire Statistique & Genome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "Word.h" 

Word::Word(const char * word, char* fa)
{
  int i;
  _length= strlen(word);
  _word = new char [_length];
  for(i=0;i<_length;i++)
    {
      _word[i]=word[i];
    }
  _alphabet = new Translator(fa);
}

Word::~Word()
{
  delete[] _word;
  delete _alphabet;
}


//===========================================

double Word::proba(unsigned long t, Drifting_Markov & DMM)
{
  int order= DMM.order();
  int alphabet_size=DMM.alphabet_size();
  if (alphabet_size!=_alphabet->tell_alphabet_size())
    {
      cout<<"Erreur: alphabet_size"<<endl;
      return 0;
    }

  int i,j,c; 
  int nbc=alphabet_size;
  int nbl= int(pow(double(alphabet_size),order));
  double p1,p2,p;
  char* B=new char[order+1+1];
  char* D=new char[order+1];
  B[order+1]='\0';
  D[order]='\0';
  p1=1;
  p2=1;
  int len=0;

 
  if(_length>=order)
    {
      //Exemple: gctggtgg ordre 2
      // mu(t,gc)*Pi(t+2,gct)*Pi(t+3,ctg)...

      Markov Pi_t(alphabet_size,order);
      for(i=0;i<nbl;i++)
	{
	  for(j=0;j<nbc;j++)
	    {
	      Pi_t(j+nbc*i)=DMM.Pi(t,i,j);
	    }
	}
      Pi_t.compute_stat_law(true);

      //First order letters of word in D
      for (c=0;c<order;c++)
	{
	  D[c]=_word[c];
	} 
      p1=p1*Pi_t.Mu(_alphabet->word_to_coded_int(D,len));

      //Rest of the word
      //word of order+1 letters in B
      t=t+order;
      for(j=0;j<_length-order;j++)
	{
	  for (c=0;c<=order;c++)
	    {
	      B[c]=_word[c+j];
	    } 
	  
	  //B is a word of size order+1 and we want calculate 
	  //the probability of the last letter of B knowing the order first letters.
	  //Index i such that _PI[i]=proba(A[a]|A[d]A[d+1]...A[a-1]) 
	  //is  _alphabet->word_to_coded_int(B,len)
	  p2=p2*DMM.Pi(t,_alphabet->word_to_coded_int(B,len));	
	  t++;
	}    
    }
  else
    {
      //Exemple gct ordre 4
      //mu(t,g)*mu_(t+1,c)*mu(t+2,t)
      for(i=0;i<_length;i++)
	{
	  p1=p1*DMM.proba_stat(t,_word[i]);
	  t++;
	}
    }
  p=p1*p2;
  delete[] B;
  delete[] D; 

  return p;

}



//=========== EXPECTATION ============

double Word::expect(unsigned long n, Drifting_Markov & DMM)
{
  if(VERBOSE)
    {
      cout<<"Compute Expectation..."<<endl;
    }
  double Nexp=0;
  unsigned long i;
  for (i=0;i<n-_length+1;i++)
    {
      Nexp+=proba(i,DMM);
    }

  return Nexp;
}

//============== Length =================
int Word::length()
{
  return _length;
}
