/* drimm.cc
 *
 * Copyright (C) 2008 Laboratoire Statistique & Genome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <unistd.h>

#include "Drifting_Markov.h"
#include "Polynomial.h"
#include "Polynomial_M.h"
#include "Splines.h"
#include "Splines_M.h"
#include "Splines_H.h"
#include "Word.h"

// Commentary in the shell
int VERBOSE = 1;

//===========================================================================================
//===========================================================================================

//////////////////////
// LA FONCTION MAIN //
//////////////////////

//===========================================================================================

int main(int argc, char *argv[]) 
{     
  //========================================//
  // Back up of the parameters of execution //
  //========================================//
  
  //Number of arguments in the command line
  int     num_arg=1 ; 
  //File describing the alphabet (a).
  char  * alphabet_file=NULL  ;
  //Base name for all output files.
  char  * output_file=NULL  ;
  //File containing sequences (s).
  char  * sequence_file=NULL  ; 
  //File containing sequences (sm).
  char  * sequence_mean_file=NULL  ;
  //File containing sequences (ss).
  char  * sequence_sum_file=NULL  ;
  //File containing sequences for the computation of the log-likelihood
  char  * sequence_file_l=NULL  ;
  //File containing sequences for the computation of AIC
  char  * sequence_file_aic=NULL  ;
  //File containing sequences for the computation of BIC
  char  * sequence_file_bic=NULL  ;
  //Configuration file
  char  * model_file=NULL  ;
  //Order and degree
  short order=-1;
  short degree=-1; 
  //Pi_t/n
  int SpM=0;
  int SpH=0;
  int Spl=0;
  int Poly=0;
  int PolyM=0;
  //Splines number of segments and methods
  int N=0;
  //Splines number of aller-retour
  int Nar=0;
  //Computation of log-likelihood, AIC, BIC, stationary laws?
  int L=0;
  int AIC=0; 
  int BIC=0;
  int l=0;
  int aic=0; 
  int bic=0;
  int law=0;
  int slaw=0; 
  int dist=0;
  int sdist=0;
  int model=0;
  unsigned long in =0;
  unsigned long out=0;

  //Order?
  int ord=0;
  //Degree?
  int deg=0;
  //Number of segments?
  int Ns=0;
  //Number of aller-retour?
  int Nars=0;
  //Pi_t?
  int PI=0;
  //Simulation?
  int simu=0;
  //Simple model?
  int fs=0;
  //Mean model?
  int fsm=0;
  //Sum model?
  int fss=0;

  //Help
  int h=0;

  srand ( time(NULL) );

  //Parameters checking
  while (num_arg < argc)
    {
      if ((strcmp (argv[num_arg], "-SpM") == 0) && (num_arg + 1 < argc))
	{
	  SpM=1;
	  num_arg++ ;
	} 
      else
	if (strcmp (argv[num_arg], "-SpH") == 0)
	  {
	    SpH=1;
	    num_arg++ ;
	  }  
	else
	  if (strcmp (argv[num_arg], "-Spl") == 0)
	    {
	      Spl=1;
	      num_arg++ ;
	    } 
	  else
	    if (strcmp (argv[num_arg], "-Poly") == 0)
	      {
		Poly=1;
		num_arg++ ;
	      } 
	    else
	      if (strcmp (argv[num_arg], "-PolyM") == 0)
		{
		  PolyM=1;
		  num_arg++ ;
		} 
	      else 
		if (strcmp (argv[num_arg], "-out") == 0)
		  {
		    num_arg++ ;
		    output_file= argv[num_arg] ;
		    ofstream testfile(output_file);
		    if (!testfile.good()) {
		      cerr <<  "Unable to open file " << output_file << endl;
		      exit(-1);
		    }
		    testfile.close();
		    unlink(output_file);
		    num_arg++ ;		    
		  }
		else
		  if (strcmp (argv[num_arg], "-a") == 0)
		    {
		      num_arg++ ;
		      alphabet_file= argv[num_arg] ;
		      ifstream testfile(alphabet_file);
		      if (!testfile.good()) {
			cerr <<  "Unable to open file " << alphabet_file << endl;
			exit(-1);
		      }
		      testfile.close();
		      num_arg++ ;
		    } 
		  else 
		    if (strcmp (argv[num_arg], "-s") == 0)
		      {
			num_arg++ ;
			fs=1;
			sequence_file= argv[num_arg] ;
			ifstream testfile(sequence_file);
			if (!testfile.good()) {
			  cerr <<  "Unable to open file " << sequence_file << endl;
			  exit(-1);
			}
			testfile.close();
			num_arg++ ;
		      } 
		    else 
		      if (strcmp (argv[num_arg], "-sm") == 0)
			{
			  num_arg++ ;
			  fsm=1;
			  sequence_mean_file= argv[num_arg] ;
			  ifstream testfile(sequence_mean_file);
			  if (!testfile.good()) {
			    cerr <<  "Unable to open file " << sequence_mean_file << endl;
			    exit(-1);
			  }
			  testfile.close();
			  num_arg++ ;
			} 
		      else 
			if (strcmp (argv[num_arg], "-ss") == 0)
			  {
			    num_arg++ ;
			    fss=1;
			    sequence_sum_file= argv[num_arg] ;
			    ifstream testfile(sequence_sum_file);
			    if (!testfile.good()) {
			      cerr <<  "Unable to open file " << sequence_sum_file << endl;
			      exit(-1);
			    }
			    testfile.close();
			    num_arg++ ;
			  } 
			else 
			  if (strcmp (argv[num_arg], "-order") == 0)
			    {
			      num_arg++ ;
			      ord=1;
			      order= atoi(argv[num_arg]); 
			      num_arg++ ;
			    } 
			  else 
			    if (strcmp (argv[num_arg], "-deg") == 0)
			      {
				num_arg++ ;
				deg=1;
				degree= atoi(argv[num_arg]); 
				num_arg++ ;
			      }  
			    else 
			      if (strcmp (argv[num_arg], "-N") == 0)
				{
				  num_arg++ ;
				  Ns=1;
				  N= atoi(argv[num_arg]); 
				  num_arg++ ;
				} 
			      else 
				if (strcmp (argv[num_arg], "-Nar") == 0)
				  {
				    num_arg++ ;
				    Nars=1;
				    Nar= atoi(argv[num_arg]); 
				    num_arg++ ;
				  } 	
				else 
				  if (strcmp (argv[num_arg], "-L") == 0)
				    {		  
				      L=1;
				      num_arg++ ;
				    } 
				  else 
				    if (strcmp (argv[num_arg], "-AIC") == 0)
				      {
					AIC=1;
					num_arg++ ;
				      } 
				    else 
				      if (strcmp (argv[num_arg], "-BIC") == 0)
					{
					  BIC=1;
					  num_arg++ ;
					} 	 
				      else 
					if (strcmp (argv[num_arg], "-l") == 0)
					  {		  
					    l=1;
					    num_arg++ ;
					    sequence_file_l= argv[num_arg] ;
					    ifstream testfile(sequence_file_l);
					    if (!testfile.good()) {
					      cerr <<  "Unable to open file " << sequence_file_l << endl;
					      exit(-1);
					    }
					    testfile.close();
					    num_arg++ ;
					  } 
					else 
					  if (strcmp (argv[num_arg], "-aic") == 0)
					    {
					      aic=1;
					      num_arg++ ;
					      sequence_file_aic= argv[num_arg] ;
					      ifstream testfile(sequence_file_aic);
					      if (!testfile.good()) {
						cerr <<  "Unable to open file " << sequence_file_aic << endl;
						exit(-1);
					      }
					      testfile.close();
					      num_arg++ ;
					    } 
					  else 
					    if (strcmp (argv[num_arg], "-bic") == 0)
					      {
						bic=1;
						num_arg++ ;
						sequence_file_bic= argv[num_arg] ;
						ifstream testfile(sequence_file_bic);
						if (!testfile.good()) {
						  cerr <<  "Unable to open file " << sequence_file_bic << endl;
						  exit(-1);
						}
						testfile.close();
						num_arg++ ;
					      }  
					    else 
					      if (strcmp (argv[num_arg], "-law") == 0)
						{
						  law=1;
						  num_arg++;
						} 
					      else 
						if (strcmp (argv[num_arg], "-slaw") == 0)
						  {
						    slaw=1;
						    num_arg++;
						    in = atoi(argv[num_arg]);
						    num_arg++ ;
						    out = atoi(argv[num_arg]);
						    num_arg++;
						  } 
						else 
						  if (strcmp (argv[num_arg], "-dist") == 0)
						    {
						      dist=1;
						      num_arg++;
						    } 
						  else 
						    if (strcmp (argv[num_arg], "-sdist") == 0)
						      {
							sdist=1;
							num_arg++;
							in = atoi(argv[num_arg]);
							num_arg++ ;
							out = atoi(argv[num_arg]);
							num_arg++;
						      } 
						    else 
						      if (strcmp (argv[num_arg], "-nv") == 0)
							{
							  VERBOSE=0;
							  num_arg++ ;
							} 
						      else 
							if (strcmp (argv[num_arg], "-pi") == 0)
							  {
							    PI=1;
							    num_arg++ ;
							  }  
							else 
							  if (strcmp (argv[num_arg], "-simu") == 0)
							    {
							      simu=1;
							      num_arg++ ;
							    }  
							  else 
							    if (strcmp (argv[num_arg], "-cf") == 0)
							      {
								model=1;
								num_arg++ ;
								model_file= argv[num_arg] ;
								ifstream testfile(model_file);
								if (!testfile.good()) {
								  cerr <<  "Unable to open file " << model_file << endl;
								  exit(-1);
								}
								testfile.close();
								num_arg++ ;
							      }  
							    else 
							      if (strcmp (argv[num_arg], "-h") == 0)
								{
								  num_arg++ ;
								  h=1;
								} 
							      else 
								{
								  cerr << "WAY TO DO : "<< argv[0]<<endl
								       << "   {"<<endl
								       << "    -SpM -N <Number of segments>0> OR"<<endl
								       << "    -SpH -N <Number of segments>0> -Nar <Number of ar>0> OR"<<endl
								       << "    -Spl -N <Number of segments>0> -Nar <Number of ar>0> OR"<<endl
								       << "    -Poly OR"<<endl
								       << "    -PolyM"<<endl
								       << "   }"<<endl
								       << "   {"<<endl
								       << "    -s <sequence_file> OR"<<endl
								       << "    -sm <sequence_mean_file> OR"<<endl
								       << "    -ss <sequence_sum_file>"<<endl
								       << "   }"<<endl
								       << "   {"<<endl
								       << "    -deg <degree>=0>"<<endl
								       << "   }"<<endl
								       << "   -order <order>=0>"<<endl
								       << "   -out <basename>"<<endl
								       << "   -a <alphabet_file>"<<endl
								       << "   [-L]"
								       << " [-AIC]"
								       << " [-BIC]"<<endl
								       << "   [-l <sequence_file_l>]"
								       << " [-aic <sequence_file_aic>]"
								       << " [-bic <sequence_file_bic>]"<<endl
								       << "   [-law]"
								       << " [-slaw <in>0> <out>0>]"<<endl
								       << "   [-dist]"
								       << " [-sdist <in>0> <out>0>]"<<endl
								       << "   [-pi]"
								       << " [-simu]"
								       << " [-nv]"
								       << " [-h]"<<endl
								       << endl 
								       << "OR : "<< argv[0]<<endl
								       << "   -cf <model_file>"
								       << " -a <alphabet_file>"
								       << " -out <basename>"<<endl
								       << "   [-l <sequence_file_l>]"
								       << " [-aic <sequence_file_aic>]"
								       << " [-bic <sequence_file_bic>]"<<endl
								       << "   [-law]"
								       << " [-slaw <in>0> <out>0>]"<<endl
								       << "   [-dist]"
								       << " [-sdist <in>0> <out>0>]"<<endl
								       << "   [-pi]"
								       << " [-simu]"
								       << " [-nv]"
								       << " [-h]"<<endl							  
								       <<endl;
								  return (1) ;
								}
    }
  

  if((h==1)||(num_arg==1))
    {
      cout<<endl;
      cout<<"##############################################################"
	  <<endl
	  <<"##########################          ##########################"
	  <<endl
	  <<"##########################   HELP   ##########################"
	  <<endl
	  <<"##########################          ##########################"
	  <<endl
	  <<"##############################################################"
	  <<endl
	  <<endl;

      cout<<"Use 0: DRIMM -h"
	  <<endl<<endl;

      cout<<"Use 1: DRIMM"
	  << "   {"<<endl
	  << "    -SpM -N <Number of segments>0> OR"<<endl
	  << "    -SpH -N <Number of segments>0> -Nar <Number of ar>0> OR"<<endl
	  << "    -Spl -N <Number of segments>0> -Nar <Number of ar>0> OR"<<endl
	  << "    -Poly OR"<<endl
	  << "    -PolyM"<<endl
	  << "   }"<<endl
	  << "   {"<<endl
	  << "    -s <sequence_file> OR"<<endl
	  << "    -sm <sequence_mean_file> OR"<<endl
	  << "    -ss <sequence_sum_file>"<<endl
	  << "   }"<<endl
	  << "   {"<<endl
	  << "    -deg <degree>=0>"<<endl
	  << "   }"<<endl
	  << "   -order <order>=0>"<<endl
	  << "   -out <basename>"<<endl
	  << "   -a <alphabet_file>"<<endl
	  << "   [-L]"
	  << " [-AIC]"
	  << " [-BIC]"<<endl
	  << "   [-l <sequence_file_l>]"
	  << " [-aic <sequence_file_aic>]"
	  << " [-bic <sequence_file_bic>]"<<endl
	  << "   [-law]"
	  << " [-slaw <in>0> <out>0>]"<<endl
	  << "   [-dist]"
	  << " [-sdist <in>0> <out>0>]"<<endl
	  << "   [-pi]"
	  << " [-simu]"
	  << " [-nv]"
	  << " [-h]"
	  <<endl<<endl;
      
      cout<<"Use 2: DRIMM"
	  << "   -cf <model_file>"
	  << " -a <alphabet_file>"
	  << " -out <basename>"<<endl
	  << "   [-l <sequence_file_l>]"
	  << " [-aic <sequence_file_aic>]"
	  << " [-bic <sequence_file_bic>]"<<endl
	  << "   [-law]"
	  << " [-slaw <in>0> <out>0>]"<<endl
	  << "   [-dist]"
	  << " [-sdist <in>0> <out>0>]"<<endl
	  << "   [-pi]"
	  << " [-simu]"
	  << " [-nv]"
	  << " [-h]"
	  <<endl<<endl;


      cout<<"=============================================================================="<<endl;
      cout<<"                        DRIMM (Drifting Markov Models)                        "<<endl;
      cout<<"=============================================================================="<<endl;
      cout<<endl;
      cout<<"This program is dedicaced to the estimation and the use of";
      cout<<" Drifting Markov Models (DMM)"<<endl; 
      cout<<endl<<endl;
      
      cout<<"There is two ways to use the program :"<<endl;
      cout<<"Use 1: DMM is estimated on a sequence"<<endl;
      cout<<"Use 2: DMM is given in a model file"<<endl<<endl;

      cout<< " -SpM                                         Estimation by splines M."<<endl;
      cout<< "                                              -N must be used."<<endl;
      cout<< " -SpH                                         Estimation by splines H."<<endl;
      cout<< "                                              -N and -Nar must be used."<<endl;
      cout<< " -Spl                                         Estimation by splines."<<endl;
      cout<< "                                              -N and -Nar must be used."<<endl;
      cout<< " -Poly                                        Estimation by polynom (stochastic matrices)."<<endl;
      cout<< " -PolyM                                       Estimation by polynom (no stochastic matrices)."<<endl;
      cout<< " -out <basename>                              Basename for all output files"<<endl;
      cout<< "                                              Output 'basename.config_<num>.out' with -fs."<<endl;
      cout<< "                                              Output 'basename.config_sm.out' with -fsm."<<endl;
      cout<< "                                              Output 'basename.config_ss.out' with -fss."<<endl;
      cout<< "                                              Output 'basename.config.out' with -cf."<<endl;
      cout<< "                                              <num> is an integer designing the number of the sequence."<<endl;
      cout<< "                                              These configuration files could be use as <model_file>." <<endl;
      cout<< " -a <alphabet_file>                           A file describing the alphabet to use." <<endl;
      cout<< " -s <sequence_file>                           A file containing file of sequences."<<endl;
      cout<< "                                              DMM is estimated on each sequence (Use 1)."<<endl;
      cout<< " -sm <sequence_mean_file>                     A file containing file of sequences."<<endl;
      cout<< "                                              DMM is estimated as a mean of DDM on each sequence (Use 1)."<<endl;
      cout<< "                                              At least one of -fs or -fsm or -fss must be used"<<endl;
      cout<< " -ss <sequence_sum_file>                      A file containing file of sequences."<<endl;
      cout<< "                                              DMM is estimated over all the sequences (Use 1)."<<endl;
      cout<< "                                              At least one of -fs or -fsm or -fss must be used"<<endl;
      cout<< " -order <order>=0>                            Order of the DMM (Use 1)."<<endl;
      cout<< "                                              Order must be > 0 with -SpM or -SpH."<<endl;
      cout<< " -deg <degree>=0>                             Degree of the DMM (Use 1)."<<endl;
      cout<< "                                              Degree is useless with -SpH and -Spl. Degree is 3."<<endl;
      cout<< " -N <Number of segments>0>                    Number of segments in the case of estimation by splines."<<endl; 
      cout<< " -Nar <Number of segments>0>                  Number of aller retour with -Spl and -SpH."<<endl;     
      cout<< " -cf <model_file>                             DMM is given by the <model_file> (Use 2)."<<endl;
      cout<< "                                              Must be used with -Poly, -PolyM, -Spl, -SpM, or -SpH."<<endl;
      cout<< " -L                                           Compute the log-likelihood on the estimated sequences (Use 1)."<<endl;
      cout<< "                                              Can not be used with -fsm or -fss"<<endl;
      cout<< " -AIC                                         Compute the AIC on the estimated sequences (Use 1)."<<endl;
      cout<< "                                              Can not be used with -fsm or -fss"<<endl;
      cout<< " -BIC                                         Compute the BIC on the estimated sequences (Use 1)."<<endl;
      cout<< "                                              Can not be used with -fsm or -fss"<<endl;
      cout<< " -l <sequence_file_l>                         Compute the log-likelihood on <sequence_file_l>."<<endl;
      cout<< " -aic <sequence_file_aic>                     Compute the aic on <sequence_file_aic>."<<endl;
      cout<< " -bic <sequence_file_bic>                     Compute the bic on <sequence_file_bic>."<<endl;
      cout<< " -law                                         Compute the stationary law of the DMM."<<endl;
      cout<< "                                              basename.trace_stat_<num>.out is an output containing the stationary law."<<endl;
      cout<< "                                              <num> is an integer designing the number of the sequence."<<endl;
      cout<< " -slaw <in>0> <out>0>                         Compute the stationary law of the DMM between <in> and <out>."<<endl;
      cout<< "                                              basename.trace_stat_segment_<num>.out is an output containing the stationary law."<<endl;
      cout<< "                                              <num> is an integer designing the number of the sequence."<<endl;
      cout<< " -dist                                        Compute distributions of the DMM."<<endl;
      cout<< "                                              basename.trace_dist_<num>.out is an output containing the distributions."<<endl;
      cout<< "                                              <num> is an integer designing the number of the sequence."<<endl;
      cout<< " -sdist <in>0> <out>0>                        Compute the distribution of the DMM between <in> and <out>."<<endl;
      cout<< "                                              basename.trace_dist_segment_<num>.out is an output containing the distributions."<<endl;
      cout<< "                                              <num> is an integer designing the number of the sequence."<<endl;
      cout<< " -nv                                          Mode 'not verbose'."<<endl;
      cout<< " -pi                                          Return files 'basename.Pit_<num>.out' or 'basename.Pit_sm.out'"<<endl;
      cout<< "                                              or 'basename.Pit.out' or 'basename.Pit_ss.out'"<<endl; 
      cout<< "                                              containing the matrix Pi_t for all t."<<endl;
      cout<< "                                              <num> is an integer designing the number of the sequence."<<endl; 
      cout<< " -simu                                        Return files 'basename.simulation_<num>.out' or 'basename.simulation_sm.out'"<<endl;
      cout<< "                                              or 'basename.simulation_ss.out' or 'basename.simulation.out'"<<endl; 
      cout<< "                                              containing a sequence simulated by the DMM "<<endl;
      cout<< "                                              <num> is an integer designing the number of the sequence."<<endl; 
      cout<< " -h                                           Print this help." <<endl;
      cout<<endl<<endl;
      return 1;
    }

  // Parameters checking
  int error =0;

  //Use 2 (Configuration file)
  if (model==1)
    {
      if (model_file==NULL)
	{
	  cerr<<" Argument is missing : -cf <model_file> ?"<<endl;
	  error=1;
	}
      if (output_file==NULL)
	{
	  cerr<<" Argument is missing : -out <basename> ?"<<endl;
	  error=1;
	}
      if (alphabet_file==NULL)
	{
	  cerr<<" Argument is missing : -a <alphabet_file> ?"<<endl;
	  error=1;
	}
      if((SpM==0)&&(SpH==0)&&(Spl==0)&&(Poly==0)&&(PolyM==0))
	{
	  cerr<<" Argument is missing : -SpM? -SpH? -Spl? -Poly? -PolyM?"<<endl;
	  error=1;
	}
      if (SpM+SpH+Spl+Poly+PolyM>1)
	{
	  cerr<<" You have to use ONE and only ONE of these: -SpM or -SpH or -Spl or -Poly or -PolyM."<<endl;
	  error=1;
	}
      if((Ns==1))
	{ 
	  cerr<<" You use a model. Do not use -N. That is given in model file."<<endl;
	  error=1;
	}
      if((ord==1))
	{ 
	  cerr<<" You use a model. Do not use -order. That is given in model file."<<endl;
	  error=1;
	}
      if((deg==1))
	{ 
	  cerr<<" You use a model. Do not use -deg. That is given in model file."<<endl;
	  error=1;
	}
     if ((slaw==1)&&((in==0)||(out==0)))
	{
	  cerr<<" Argument is bad defined : -slaw <in>0> <out>0> ?"<<endl;
	  error=1;
	}
      if ((sdist==1)&&((in==0)||(out==0)))
	{
	  cerr<<" Argument is bad defined : -sdist <in>0> <out>0> ?"<<endl;
	  error=1;
	}
      if((sequence_file_l==NULL)&&(l==1))
	{
	  cerr<<" Argument is missing : -l <sequence_file_l> ?"<<endl;
	  error=1;
	}
      if((sequence_file_aic==NULL)&&(aic==1))
	{
	  cerr<<" Argument is missing : -aic <sequence_file_aic> ?"<<endl;
	  error=1;
	}
      if((sequence_file_bic==NULL)&&(bic==1))
	{
	  cerr<<" Argument is missing : -bic <sequence_file_bic> ?"<<endl;
	  error=1;
	}
      if (L==1)
	{
	  cerr<<" You use a model and not a sequence. Use -l <sequence_file_l>. "<<endl;
	  error=1;
	}
      if (AIC==1)
	{
	  cerr<<" You use a model and not a sequence. Use -aic <sequence_file_aic>. "<<endl;
	  error=1;
	}
      if (BIC==1)
	{
	  cerr<<" You use a model and not a sequence. Use -bic <sequence_file_bic>. "<<endl;
	  error=1;
	}
     if (sequence_file!=NULL)
	{
	  cerr<<" You use a model and not a sequence. Do not use -s <sequence_file>."<<endl; 
	  error=1;
	}
      if (sequence_mean_file!=NULL)
	{
	  cerr<<" You use a model and not a sequence. Do not use -sm <sequence_mean_file>."<<endl; 
	  error=1;
	}
      if (sequence_sum_file!=NULL)
	{
	  cerr<<" You use a model and not a sequence. Do not use -ss <sequence_sum_file>."<<endl; 
	  error=1;
	}
    }

  //Use 1 (Estimation of a DMM)
  if (model==0)
    {     
      if((SpM==0)&&(SpH==0)&&(Spl==0)&&(Poly==0)&&(PolyM==0))
	{
	  cerr<<" Argument is missing : -SpM? -SpH? -Spl? -Poly? -PolyM?"<<endl;
	  error=1;
	}
      if (SpM+SpH+Spl+Poly+PolyM>1)
	{
	  cerr<<" You have to use ONE and only ONE of these: -SpM or -SpH or -Spl or -Poly or -PolyM."<<endl;
	  error=1;
	}
      if(((SpM==1)||(SpH==1)||(Spl==1))&&(N<=1))
	{
	  cerr<<" Argument is missing : -N <Number of segments>1>?"<<endl;
	  error=1;
	} 
      if(((Spl==1)||(SpH==1))&&(Nars<1))
	{
	  cerr<<" Argument is missing : -Nar <Number of aller retour>0>?"<<endl;
	  error=1;
	}
      if(((Poly==1)||(PolyM==1)||(SpM==1))&&(Nars==1))
	{
	  cerr<<" You use Polynomial Polynomial_M or Splines_M. -Nar is useless."<<endl;
	  error=1;
	} 
      if(((Poly==1)||(PolyM==1))&&(Ns==1))
	{
	  cerr<<" You use Polynomial or Polynomial_M. -N is useless."<<endl;
	  error=1;
	}
      if (output_file==NULL)
	{
	  cerr<<" Argument is missing : -out <basename> ?"<<endl;
	  error=1;
	}
      if (alphabet_file==NULL)
	{
	  cerr<<" Argument is missing : -a <alphabet_file> ?"<<endl;
	  error=1;
	}
      if ((sequence_file==NULL)&&(sequence_mean_file==NULL)&&(sequence_sum_file==NULL))
	{
	  cerr<<" Argument is missing : -s <sequence_file> ? -sm <sequence_mean_file> ? -ss <sequence_sum_file> ?"<<endl;
	  error=1;
	}    
      if (order<0)
	{
	  cerr<<" Argument is bad defined : -order <order>=0> ?"<<endl;	
	  error=1;
	}
      if (((SpM==1)||(Poly==1)||(PolyM==1))&&(degree<0))
	{
	  cerr<<" Argument is bad defined : -deg <degree>=0> ?"<<endl;
	  error=1;
	}        
     if ((Spl==1)&&(deg==1))
	{
	  cerr<<" You use Splines. -deg is useless. By default, degree is 3."<<endl;
	  error=1;
	}   
      if ((SpH==1)&&(deg==1))
	{
	  cerr<<" You use Splines_H. -deg is useless. By default, degree is 3."<<endl;
	  error=1;
	}   
      if((degree==0)&&(SpM==1))
	{
	  cerr<<" Argument is bad defined : -deg <degree>0> ?"<<endl;
	  error=1;
	}   
      if ((slaw==1)&&((in<0)||(out<=0)))
	{
	  cerr<<" Argument is bad defined : -slaw <in>0> <out>0> ?"<<endl;
	  error=1;
	}
      if ((sdist==1)&&((in<0)||(out<=0)))
	{
	  cerr<<"  Argument is bad defined :-sdist <in>0> <out>0> ?"<<endl;
	  error=1;
	}
      if((sequence_file_l==NULL)&&(l==1))
	{
	  cerr<<" Argument is missing : -l <sequence_file_l> ?"<<endl;
	  error=1;
	}
      if((sequence_file_aic==NULL)&&(aic==1))
	{
	  cerr<<" Argument is missing : -aic <sequence_file_aic> ?"<<endl;
	  error=1;
	}
      if((sequence_file_bic==NULL)&&(bic==1))
	{
	  cerr<<" Argument is missing : -bic <sequence_file_bic> ?"<<endl;
	  error=1;
	}      
      if ((L==1)&&((fsm==1)||(fss==1)))
	{
	  cerr<<" You use a mean or a sum model and not a sequence. Use -l <sequence_file_l>. "<<endl;
	  error=1;
	}
      if ((AIC==1)&&((fsm==1)||(fss==1)))
	{
	  cerr<<" You use a mean or a sum model and not a sequence. Use -aic <sequence_file_aic>. "<<endl;
	  error=1;
	}
      if ((BIC==1)&&((fsm==1)||(fss==1)))
	{
	  cerr<<" You use a mean or a sum model and not a sequence. Use -bic <sequence_file_bic>. "<<endl;
	  error=1;
	}
    }
  if (error==1)
    {
      cerr<<" DRIMM -h for help."<<endl;
      return 1;
    }

  //=========//
  // Program //
  //=========//
  
  Translator alpha(alphabet_file);
 
  //=========================
  // Sans model_file
  //=========================
  if(model==0)
    {
      //=======================================
      // No mean in sequences of sequence_file
      //=======================================
      if(fs==1)
	{
	  const SequenceSet seq(sequence_file, alphabet_file, order);  
	  int i,j,k;
	  int nbs=seq.tell_nb_sequence();
	  int nbs2;
	  for(j=0;j<nbs;j++)
	    {
	      if(VERBOSE)
		{
		  cout<<endl;
		  cout<<"==============="<<endl;
		  cout<<" Sequence "<<j+1<<endl;
		  cout<<"==============="<<endl;
		}
	      if(VERBOSE)
		{
		  cout<<endl;
		  cout<<"***************"<<endl;
		  cout<<"*    Data     *"<<endl;
		  cout<<"***************"<<endl<<endl;
		  cout<<"Alphabet = {"<<alpha.int_to_str(0);
		  for(i=1;i<alpha.tell_alphabet_size();i++)
		    {
		      cout<<","<<alpha.int_to_str(i);
		    }
		  cout<<"}"<<endl;
		  cout<<"Sequence = "<<seq(j).tell_seq_name()<<endl;
		  cout<<"Sequence length = "<<seq(j).tell_length()<<endl;
		  cout<<"Order = "<<order<<endl;
		  if ((SpM==1)||(Poly==1)||(PolyM==1))
		    {
		      cout<<"Degree = "<<degree<<endl;
		    }
		  if ((SpH==1)||(Spl==1))
		    {
		      cout<<"Degree = 3"<<endl;
		    }
		  cout<<endl;
		} 

	      //========================================
	      //Estimation of a DMM on sequence j
	      //different methods
 	      Drifting_Markov * DMM;
  	      if (Poly==1)
  		{
  		  DMM = new Polynomial(seq(j), degree); 	
  		}
	      if (PolyM==1)
  		{
  		  DMM = new Polynomial_M(seq(j), degree); 	
  		}
  	      if (SpM==1)
  		{
  		  DMM = new Splines_M(seq(j),degree,N);	
  		}
  	      if (SpH==1)
  		{
		  DMM = new Splines_H(seq(j),N,Nar);
  		} 
  	      if (Spl==1)
  		{
 		  DMM = new Splines(seq(j),N,Nar);
  		}
	      //========================================

	      if (L==1)  
		{	      
		  cout<<"L ="<<setprecision(10)<<DMM->log_likelihood(seq(j))<<endl;
		}
	      if (AIC==1) 
		{
		  cout<<"AIC ="<<setprecision(10)<<DMM->AIC(seq(j))<<endl;
		}
	      if (BIC==1) 
		{
		  cout<<"BIC ="<<setprecision(10)<<DMM->BIC(seq(j))<<endl;
		}
	      if (l==1)   
		{
		  const SequenceSet seq_l(sequence_file_l, alphabet_file, order);  
		  nbs2=seq_l.tell_nb_sequence();
		  for(k=0;k<nbs2;k++)
		    {
		      cout<<seq_l.tell_seq_name(k)<<" : ";
		      cout<<"l ="<<setprecision(10)<<DMM->log_likelihood(seq_l(k))<<endl;
		    }
		}
	      if (aic==1) 
		{
		  const SequenceSet seq_aic(sequence_file_aic, alphabet_file, order);  
		  nbs2=seq_aic.tell_nb_sequence();
		  for(k=0;k<nbs2;k++)
		    {
		      cout<<seq_aic.tell_seq_name(k)<<" : ";
		      cout<<"aic ="<<setprecision(10)<<DMM->AIC(seq_aic(k))<<endl;
		    }
		}
	      if (bic==1) 
		{
		  const SequenceSet seq_bic(sequence_file_bic, alphabet_file, order);  
		  nbs2=seq_bic.tell_nb_sequence();
		  for(k=0;k<nbs2;k++)
		    {
		      cout<<seq_bic.tell_seq_name(k)<<" : ";
		      cout<<"bic ="<<setprecision(10)<<DMM->BIC(seq_bic(k))<<endl;
		    }
		}
	      if (law==1) 
		{
		  DMM->proba_stat_evol(j,output_file);
		} 
	      if (dist==1) 
		{
		  DMM->distrib_evol(j,output_file);
		} 
	      if (slaw==1) 
		{
		  if ((0<=in)&&(in<out)&&(out<seq(j).tell_length()))
		    {
		      DMM->proba_stat_evol(j,in,out,output_file);
		    } 
		  else
		    {
		      cout<<"No stationary laws : in and/or out bad defined"<<endl;
		    }
		}
	      if (sdist==1) 
		{
		  if ((0<=in)&&(in<out)&&(out<seq(j).tell_length()))
		    {
		      DMM->distrib_evol(j,in,out,output_file);
		    } 
		  else
		    {
		      cout<<"No distributions : in and/or out bad defined"<<endl;
		    }
		}
	      //++++++++++++++++++++++++
	      //Configuration files
	      //Polynomial method = 0
	      //Splines_M method = 1
	      //Splines_H method = 2
	      //Splines method = 3
	      //Polynomial_M method = 4
	      
	      ostringstream oss;
	      oss << output_file <<".config_"<<j<<".out";
	      const char * config=oss.str().c_str();
	      
	      ofstream cf(config);
	      DMM->config_file(cf);
	      cf.close(); 
	      if(VERBOSE)
		{
		  cout<<"Output file: "<< output_file <<".config_"<<j<<".out"<<endl;
		}
	      //++++++++++++++++++++++++

	      //Transition matrix file
	      if (PI==1)
		{
		  ostringstream pits;
		  pits<< output_file <<".Pit_"<<j<<".out";
		  const char * pitrans=pits.str().c_str();

		  int compt;
		  int len=DMM->length();
		  ofstream pit(pitrans);
		  int pas= len/10000 +1; //"+1" for sequence size smaller than 10000
		  for(compt=0;compt<len;compt=compt+pas)
		    {
		      DMM->print_Pit((unsigned long)(compt), pit);
		    }
		  pit.close();
		  if(VERBOSE)
		    {
		      cout<<"Output file: "<< output_file <<".Pit_"<<j<<".out"<<endl;		 
		    }
		}

	      //Simulation files
	      if (simu==1)
		{ 
		  ostringstream oss3;
		  oss3<< output_file <<".simulation_"<<j<<".out";
		  const char * simul=oss3.str().c_str();
		  ofstream sim(simul);
		  DMM->simulation(alphabet_file,sim);
		  sim.close(); 
		  if(VERBOSE)
		    {
		      cout<<"Output file: "<< output_file <<".simulation_"<<j<<".out"<<endl;
		    }
		}

	      delete DMM;
	    }
	} 
 
      //=====================================
      // Mean in sequences of sequence_file
      //=====================================
      if (fsm==1)
	{
	  const SequenceSet seqset(sequence_mean_file, alphabet_file, order);  
	  int i,k;
	  int nbs=seqset.tell_nb_sequence();
	  int nbs2;

	  //Checking sequence sizes
	  int s=seqset(0).tell_length();
	  int same=1;	 
	  for(i=1;i<nbs;i++)
	    {
	      same+=(seqset(i).tell_length()== (unsigned long)(s));
	    }
	  if (same!=nbs)
	    {
	      cerr<<"Sequences of <sequence_mean_file> have not the same size."<<endl;
	      return 1;
	    }

	  if(VERBOSE)
	    {
	      cout<<endl;
	      cout<<"***************"<<endl;
	      cout<<"*    Data     *"<<endl;
	      cout<<"***************"<<endl<<endl;
	      cout<<"Alphabet = {"<<alpha.int_to_str(0);
	      for(i=1;i<alpha.tell_alphabet_size();i++)
		{
		  cout<<","<<alpha.int_to_str(i);
		}
	      cout<<"}"<<endl;
	      cout<<"Sequences = "<<sequence_mean_file<<endl;
	      cout<<"Sequences size = "<<seqset(0).tell_length()<<endl;
	      cout<<"Order = "<<order<<endl;
	      if ((SpM==1)||(Poly==1)||(PolyM==1))
		{
		  cout<<"Degree = "<<degree<<endl;
		}
	      if ((SpH==1)||(Spl==1))
		{
		  cout<<"Degree = 3"<<endl;
		}
	      cout<<endl;
	    }

	  //========================================
	  //Estimation of a DMM on all the sequence
	  //different methods
	  Drifting_Markov * DMM;
	  if (Poly==1)
	    {
	      DMM = new Polynomial(seqset, degree); 	
	    }
	  if (PolyM==1)
	    {
	      DMM = new Polynomial_M(seqset, degree); 	
	    }
	  if (SpM==1)
	    {
	      DMM = new Splines_M(seqset,degree,N);	
	    }
	  if (SpH==1)
	    {
	      DMM = new Splines_H(seqset,N,Nar);
	    } 
	  if (Spl==1)
	    {
	      DMM = new Splines(seqset,N,Nar);
	    }
	  //========================================

	  if (l==1)   
	    {
	      const SequenceSet seq_l(sequence_file_l, alphabet_file, order);  
	      nbs2=seq_l.tell_nb_sequence();
	      for(k=0;k<nbs2;k++)
		{
		  cout<<seq_l.tell_seq_name(k)<<" : ";
		  cout<<"l ="<<setprecision(10)<<DMM->log_likelihood(seq_l(k))<<endl;
		}
	    }
	  if (aic==1) 
	    {
	      const SequenceSet seq_aic(sequence_file_aic, alphabet_file, order);  
	      nbs2=seq_aic.tell_nb_sequence();
	      for(k=0;k<nbs2;k++)
		{
		  cout<<seq_aic.tell_seq_name(k)<<" : ";
		  cout<<"aic ="<<setprecision(10)<<DMM->AIC(seq_aic(k))<<endl;
		}
	    }
	  if (bic==1) 
	    {
	      const SequenceSet seq_bic(sequence_file_bic, alphabet_file, order);  
	      nbs2=seq_bic.tell_nb_sequence();
	      for(k=0;k<nbs2;k++)
		{
		  cout<<seq_bic.tell_seq_name(k)<<" : ";
		  cout<<"bic ="<<setprecision(10)<<DMM->BIC(seq_bic(k))<<endl;
		}
	    }
	  if (law==1) 
	    {
	      DMM->proba_stat_evol(0,output_file);
	    } 
	  if (slaw==1) 
	    {
	      if ((0<=in)&&(in<out)&&(out<seqset(0).tell_length()))
		{
		  DMM->proba_stat_evol(0,in,out,output_file);
		} 
	      else
		{
		  cout<<"No stationary laws : in and/or out bad defined"<<endl;
		}
	    }
	  if (dist==1) 
	    {
	      DMM->distrib_evol(0,output_file);
	    } 
	  if (sdist==1) 
	    {
	      if ((0<=in)&&(in<out)&&(out<seqset(0).tell_length()))
		{
		  DMM->distrib_evol(0,in,out,output_file);
		} 
	      else
		{
		  cout<<"No distributions : in and/or out bad defined"<<endl;
		}
	    }	  

	  //++++++++++++++++++++++++
	  //Configuration file
	  //Polynomial method = 0
	  //Splines_M method = 1
	  //Splines_H method = 2
	  //Splines method = 3
	  //Polynomial_M method = 4
	  ostringstream oss;
	  oss<< output_file <<".config_sm.out";
	  const char * config=oss.str().c_str();
	  ofstream cf(config);
	  DMM->config_file(cf);
	  cf.close(); 
	  if(VERBOSE)
	    {	   
	      cout<<"Output file: "<< output_file <<".config_sm.out"<<endl;
	    }
	  //++++++++++++++++++++++++

	  //Transition matrix file
	  if (PI==1)
	    {
	      ostringstream pits;
	      pits<< output_file <<".Pit_sm.out";
	      const char * pitrans=pits.str().c_str();

	      int compt;
	      int len=DMM->length();
	      ofstream pit(pitrans);
	      int pas= len/10000 +1; //"+1" for sequence size smaller than 10000
	      for(compt=0;compt<len;compt=compt+pas)
		{
		  DMM->print_Pit((unsigned long)(compt), pit);
		}
	      pit.close();  
	      if(VERBOSE)
		{	    
	      cout<<"Output file: "<< output_file <<".Pit_sm.out"<<endl;
		}
	    }
	  
	  //Simulation file
	  if (simu==1)
	    {
	      ostringstream oss3;
	      oss3<< output_file <<".simulation_sm.out";
	      const char * simul=oss3.str().c_str();
	      ofstream sim(simul);
	      DMM->simulation(alphabet_file,sim);
	      sim.close();  
	      if(VERBOSE)
		{
		  cout<<"Output file: "<< output_file <<".simulation_sm.out"<<endl;
		}
	    }
	}
      //=====================================
      // Mean in sequences of sequence_file
      //=====================================
      if (fss==1)
	{
	  const SequenceSet seqset(sequence_sum_file, alphabet_file, order);  
	  int i,k;
	  int nbs=seqset.tell_nb_sequence();
	  int nbs2;

	  if(VERBOSE)
	    {
	      cout<<endl;
	      cout<<"***************"<<endl;
	      cout<<"*    Data     *"<<endl;
	      cout<<"***************"<<endl<<endl;
	      cout<<"Alphabet = {"<<alpha.int_to_str(0);
	      for(i=1;i<alpha.tell_alphabet_size();i++)
		{
		  cout<<","<<alpha.int_to_str(i);
		}
	      cout<<"}"<<endl;
	      cout<<"Sequences = "<<sequence_sum_file<<endl;
	      cout<<"Order = "<<order<<endl;
	      if ((SpM==1)||(Poly==1)||(PolyM==1))
		{
		  cout<<"Degree = "<<degree<<endl;
		}
	      if ((SpH==1)||(Spl==1))
		{
		  cout<<"Degree = 3"<<endl;
		}
	      cout<<endl;
	    }

	  //========================================
	  //Estimation of a DMM on all the sequence
	  //different methods
	  Drifting_Markov * DMM;
	  if (Poly==1)
	    {
	      DMM = new Polynomial(degree, seqset); 	
	    }
	  if (PolyM==1)
	    {
	      DMM = new Polynomial_M(degree, seqset); 	
	    }
	  if (SpM==1)
	    {
	      DMM = new Splines_M(degree, seqset ,N);	
	    }
	  if (SpH==1)
	    {
	      DMM = new Splines_H(N, seqset,Nar);
	    } 
	  if (Spl==1)
	    {
	      DMM = new Splines(N, seqset ,Nar);
	    }
	  //========================================

	  if (l==1)   
	    {
	      const SequenceSet seq_l(sequence_file_l, alphabet_file, order);  
	      nbs2=seq_l.tell_nb_sequence();
	      for(k=0;k<nbs2;k++)
		{
		  cout<<seq_l.tell_seq_name(k)<<" : ";
		  cout<<"l ="<<setprecision(10)<<DMM->log_likelihood(seq_l(k))<<endl;
		}
	    }
	  if (aic==1) 
	    {
	      const SequenceSet seq_aic(sequence_file_aic, alphabet_file, order);  
	      nbs2=seq_aic.tell_nb_sequence();
	      for(k=0;k<nbs2;k++)
		{
		  cout<<seq_aic.tell_seq_name(k)<<" : ";
		  cout<<"aic ="<<setprecision(10)<<DMM->AIC(seq_aic(k))<<endl;
		}
	    }
	  if (bic==1) 
	    {
	      const SequenceSet seq_bic(sequence_file_bic, alphabet_file, order);  
	      nbs2=seq_bic.tell_nb_sequence();
	      for(k=0;k<nbs2;k++)
		{
		  cout<<seq_bic.tell_seq_name(k)<<" : ";
		  cout<<"bic ="<<setprecision(10)<<DMM->BIC(seq_bic(k))<<endl;
		}
	    }
	  if (law==1) 
	    {
	      DMM->proba_stat_evol(0,output_file);
	    } 
	  if (slaw==1) 
	    {
	      if ((0<=in)&&(in<out)&&(out<seqset(0).tell_length()))
		{
		  DMM->proba_stat_evol(0,in,out,output_file);
		} 
	      else
		{
		  cout<<"No stationary laws : in and/or out bad defined"<<endl;
		}
	    }
	  if (dist==1) 
	    {
	      DMM->distrib_evol(0,output_file);
	    } 
	  if (sdist==1) 
	    {
	      if ((0<=in)&&(in<out)&&(out<seqset(0).tell_length()))
		{
		  DMM->distrib_evol(0,in,out,output_file);
		} 
	      else
		{
		  cout<<"No distributions : in and/or out bad defined"<<endl;
		}
	    }	  

	  //++++++++++++++++++++++++
	  //Configuration file
	  //Polynomial method = 0
	  //Splines_M method = 1
	  //Splines_H method = 2
	  //Splines method = 3
	  //Polynomial_M method = 4
	  ostringstream oss;
	  oss<< output_file <<".config_ss.out";
	  const char * config=oss.str().c_str();
	  ofstream cf(config);
	  DMM->config_file(cf);
	  cf.close();  
	  if(VERBOSE)
	    {
	      cout<<"Output file: "<< output_file <<".config_ss.out"<<endl;
	    }
	  //++++++++++++++++++++++++

	  //Transition matrix file
	  if (PI==1)
	    {
	      ostringstream pits;
	      pits<< output_file <<".Pit_ss.out";
	      const char * pitrans=pits.str().c_str();

	      int compt;
	      int len=DMM->length();
	      ofstream pit(pitrans);
	      int pas= len/10000 +1; //"+1" for sequence size smaller than 10000
	      for(compt=0;compt<len;compt=compt+pas)
		{
		  DMM->print_Pit((unsigned long)(compt), pit);
		}
	      pit.close(); 
	      if(VERBOSE)
		{
		  cout<<"Output file: "<< output_file <<".Pit_ss.out"<<endl;
		}
	    }
	  
	  //Simulation file
	  if (simu==1)
	    {
	      ostringstream oss3;
	      oss3<< output_file <<".simulation_ss.out";
	      const char * simul=oss3.str().c_str();
	      ofstream sim(simul);
	      DMM->simulation(alphabet_file,sim);
	      sim.close();  
	      if(VERBOSE)
		{
		  cout<<"Output file: "<< output_file <<".simulation_ss.out"<<endl;
		}
	    }
	}

    }

  /*************************************/
  /*************************************/
  /*************************************/
  /*************************************/
  /***                               ***/
  /***          Model-file           ***/
  /***                               ***/
  /*************************************/
  /*************************************/
  /*************************************/
  /*************************************/
  

  else
    {
      int i,j,k;
      j=-1;  
      int nbs2;
   
      //========================================
      //Construction of a DMM by a model file
      Drifting_Markov * DMM;
      int bad_model=0;	 
      ifstream mf(model_file);
      int method;
      mf>>method;
      mf.close();

      if (Poly==1)
	{
	  if(method!=0)
	    {
	      cerr<<"Error: You do not use Polynomial method."<<endl;
	      bad_model=1;
	    }
	  else
	    {
	      DMM = new Polynomial(model_file); 
	    }	
	} 
      if (PolyM==1)
	{
	  if(method!=4)
	    {
	      cerr<<"Error: You do not use Polynomial_M method."<<endl;
	      bad_model=1;
	    }
	  else
	    {
	      DMM = new Polynomial_M(model_file); 
	    }	
	}
      if (SpM==1)
	{
	  if(method!=1)
	    {
	      cerr<<"Error: You do not use Splines_M method."<<endl;
	      bad_model=1;
	    }
	  else
	    {
	      DMM = new Splines_M(model_file);	
	    }
	}
      if (SpH==1)
	{
	  if(method!=2)
	    {
	      cerr<<"Error: You do not use Splines_H method."<<endl;
	      bad_model=1;
	    }
	  else
	    {
	      DMM = new Splines_H(model_file);
	    }
	} 
      if (Spl==1)
	{
	  if(method!=3)
	    {
	      cerr<<"Error: You do not use Splines method."<<endl;
	      bad_model=1;
	    }
	  else
	    {
	      DMM = new Splines(model_file);
	    }
	}

      if(bad_model==1)
	{
	  delete DMM;
	  return 0;
	}
      //========================================
 
      order=DMM->order();

      if(VERBOSE)
	{ 
	  cout<<endl;
	  cout<<"***************"<<endl;
	  cout<<"*    Data     *"<<endl;
	  cout<<"***************"<<endl<<endl;
	  cout<<"Alphabet = {"<<alpha.int_to_str(0);
	  for(i=1;i<alpha.tell_alphabet_size();i++)
	    {
	      cout<<","<<alpha.int_to_str(i);
	    }
	  cout<<"}"<<endl;
	  cout<<"Sequence size = "<<DMM->length()<<endl;
	  cout<<"Order = "<<order<<endl;
	  if ((SpM==1)||(Poly==1)||(PolyM==1))
	    {
	      cout<<"Degree = "<<DMM->degree()<<endl;
	    }
	  if ((SpH==1)||(Spl==1))
	    {
	      cout<<"Degree = 3"<<endl;
	    }
	  cout<<endl;
	}

      if (l==1)   
	{
	  const SequenceSet seq_l(sequence_file_l, alphabet_file, order);  
	  nbs2=seq_l.tell_nb_sequence();
	  for(k=0;k<nbs2;k++)
	    {
	      cout<<seq_l.tell_seq_name(k)<<" : ";
	      cout<<"l ="<<setprecision(10)<<DMM->log_likelihood(seq_l(k))<<endl;
	    }
	}
      if (aic==1) 
	{
	  const SequenceSet seq_aic(sequence_file_aic, alphabet_file, order);  
	  nbs2=seq_aic.tell_nb_sequence();
	  for(k=0;k<nbs2;k++)
	    {
	      cout<<seq_aic.tell_seq_name(k)<<" : ";
	      cout<<"aic ="<<setprecision(10)<<DMM->AIC(seq_aic(k))<<endl;
	    }
	}
      if (bic==1) 
	{
	  const SequenceSet seq_bic(sequence_file_bic, alphabet_file, order);  
	  nbs2=seq_bic.tell_nb_sequence();
	  for(k=0;k<nbs2;k++)
	    {
	      cout<<seq_bic.tell_seq_name(k)<<" : ";
	      cout<<"bic ="<<setprecision(10)<<DMM->BIC(seq_bic(k))<<endl;
	    }
	}
      if (law==1) 
	{
	  DMM->proba_stat_evol(j,output_file);
	} 
      if (slaw==1) 
	{
	  if ((0<=in)&&(in<out)&&(out<DMM->length()))
	    {
	      DMM->proba_stat_evol(j,in,out,output_file);
	    } 
	  else
	    {
	      cout<<"No stationary laws : in and/or out bad defined"<<endl;
	    }
	}
      if (dist==1) 
	{
	  DMM->distrib_evol(j,output_file);
	} 
      if (sdist==1) 
	{
	  if ((0<=in)&&(in<out)&&(out<DMM->length()))
	    {
	      DMM->distrib_evol(j,in,out,output_file);
	    } 
	  else
	    {
	      cout<<"No distributions : in and/or out bad defined"<<endl;
	    }
	}
      
      //++++++++++++++++++++++++
      //Configuration file
      ostringstream oss;
      oss<< output_file <<".config.out";
      const char * config=oss.str().c_str();
      ofstream cf(config);
      DMM->config_file(cf);
      cf.close();  
      if(VERBOSE)
	{
	  cout<<"Output file: "<< output_file <<".config.out"<<endl;
	}
      //++++++++++++++++++++++++
	  	
      //Transition matrix file
      if (PI==1)
	{
	  ostringstream pits;
	  pits<< output_file <<".Pit.out";
	  const char * pitrans=pits.str().c_str();

	  int compt;
	  int len=DMM->length();
	  ofstream pit(pitrans);
	  int pas= len/10000 +1; //"+1" for sequence size smaller than 10000
	  for(compt=0;compt<len;compt=compt+pas)
	    {
	      DMM->print_Pit((unsigned long)(compt), pit);
	    }
	  pit.close(); 
	  if(VERBOSE)
	    {
	      cout<<"Output file: "<< output_file <<".Pit.out"<<endl;
	    }
	}
	  
  
      //Simulation file
      if (simu==1)
	{
	  ostringstream oss3;
	  oss3<< output_file <<".simulation.out";
	  const char * simul=oss3.str().c_str();
	  ofstream sim(simul);
	  DMM->simulation(alphabet_file,sim);
	  sim.close(); 
	  if(VERBOSE)
	    {
	      cout<<"Output file: "<< output_file <<".simulation.out"<<endl;
	    }
	}
      delete DMM;
    }
  return 0;
}        


