/* proba_word.cc
 *
 * Copyright (C) 2008 Laboratoire Statistique & Genome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <unistd.h>

#include "Drifting_Markov.h"
#include "Polynomial.h"
#include "Polynomial_M.h"
#include "Splines.h"
#include "Splines_M.h"
#include "Splines_H.h"
#include "Word.h"

// Commentary in the shell
int VERBOSE = 1;


//===========================================================================================
//===========================================================================================

//////////////////////
// LA FONCTION MAIN //
//////////////////////

//===========================================================================================

int main(int argc, char *argv[]) 
{     
  //Number of parameters in the command line
  int     num_arg=1 ; 
  //Words file
  char  * word=NULL  ;
  //Alphabet file
  char  * alphabet_file=NULL  ;  
  //Config file (Drifting_Markov)
  char * config_file=NULL;
  //Config file?
  int cf=0;
  //Positions
  int pos=-1;
  unsigned long in;
  unsigned long out;
  short g_in=-1;
  short g_out=-1;

  int h=-1;
  int i;
  //Estimation by Splines or by polynomial method
  int SpM=0;
  int Poly=0;
  int PolyM=0;
  int Spl=0;
  int SpH=0; 

  while (num_arg < argc)
    {
      if ((strcmp (argv[num_arg], "-w") == 0) && (num_arg + 1 < argc))
	{
	  num_arg++ ;
	  word= argv[num_arg] ;
	  num_arg++ ;
	} 
      else
	if (strcmp (argv[num_arg], "-a") == 0)
	  {
	    num_arg++ ;
	    alphabet_file= argv[num_arg] ;
	    ifstream testfile(alphabet_file);
	    if (!testfile.good()) {
	      cerr <<  "Unable to open file " << alphabet_file << endl;
	      exit(-1);
	    }
	    testfile.close();
	    num_arg++ ;
	  } 
	else 
	  if (strcmp (argv[num_arg], "-cf") == 0)
	    {
	      num_arg++ ;
	      config_file= argv[num_arg] ;
	      ifstream testfile(config_file);
	      if (!testfile.good()) {
		cerr <<  "Unable to open file " << config_file << endl;
		exit(-1);
	      }
	      testfile.close();
	      cf=1;
	      num_arg++ ;
	    } 
	  else
	    if (strcmp (argv[num_arg], "-SpM") == 0)
	      {
		SpM=1;
		num_arg++ ;
	      } 
	    else
	      if (strcmp (argv[num_arg], "-Poly") == 0)
		{
		  Poly=1;
		  num_arg++ ;
		}     
	      else
		if (strcmp (argv[num_arg], "-PolyM") == 0)
		  {
		    PolyM=1;
		    num_arg++ ;
		  }     
		else
		  if (strcmp (argv[num_arg], "-Spl") == 0)
		    {
		      Spl=1;
		      num_arg++ ;
		    }     
		  else
		    if (strcmp (argv[num_arg], "-SpH") == 0)
		      {
			SpH=1;
			num_arg++ ;
		      }     
		    else 
		      if (strcmp (argv[num_arg], "-pos") == 0)
			{
			  pos=1;
			  num_arg++ ;
			  in= atoi(argv[num_arg]); 
			  g_in=1;
			  num_arg++ ;
			  out= atoi(argv[num_arg]); 
			  g_out=1;
			  num_arg++ ;
			} 
		      else 
			if (strcmp (argv[num_arg], "-h") == 0)
			  {
			    num_arg++ ;
			    h=1;
			  }  
			else 
			  {
			    cerr<<endl;
			    cerr<<"#######################"<<endl
				<<"##### BAD OPTIONS #####"<<endl
				<<"#######################"<<endl
				<<endl;
			    cerr << "WAY TO DO : "<< argv[0]<<endl
				 << " -w <word>"<<endl
				 << " -a <alphabet_file>"<<endl
				 << " -cf <config_file>"<<endl
				 << " {-Poly or -PolyM or -SpM or -Spl or -SpH}"<<endl
				 << " [-pos <in> <out>]"<<endl
				 << " [-h]"			  	
				 << endl<<endl;
			    return 1;
			  }
    }
 


  if((h==1)||(num_arg==1))
    {
      cout<<endl;
      cout<<"##############################################################"
	  <<endl
	  <<"##########################          ##########################"
	  <<endl
	  <<"##########################   HELP   ##########################"
	  <<endl
	  <<"##########################          ##########################"
	  <<endl
	  <<"##############################################################"
	  <<endl
	  <<endl;

      cout<<"Use 0: PROBAword -h"
	  <<endl<<endl;
      
      cout<<"Use 1: PROBAword"<<endl
	  << " -w <word>"<<endl
	  << " -a <alphabet_file>"<<endl
	  << " -cf <config_file>"<<endl
	  << " {-Poly or -PolyM or -SpM or -Spl or -SpH}"<<endl
	  << " [-pos <in> <out>]"<<endl
	  << " [-h]" 
	  <<endl<<endl;

      cout<<"=============================================================================="<<endl;
      cout<<"                                PROBA                                         "<<endl;
      cout<<"=============================================================================="<<endl;
      cout<<endl;
      cout<<"This program is dedicaced to the computation, at each position in the sequence,"<<endl;
      cout<<"of the probability of a word according to a Drifting Markov Model (DMM)"<<endl; 
      cout<<endl<<endl;
      
      cout<<"There is one way to use the program :"<<endl;
      cout<<"Use 1: DMM is given in a model file"<<endl<<endl;

      cout<< " -w <word>                        A word." <<endl;
      cout<< " -a <alphabet_file>               A file describing the alphabet to use." <<endl;
      cout<< " -cf <config_file>                DMM is given by the <config_file> (Use 2)."<<endl;
      cout<< " -SpM                             Estimation by splines (global)."<<endl;
      cout<< " -Spl                             Estimation by splines (basis functions)."<<endl;
      cout<< " -SpH                             Estimation by splines (no basis functions)."<<endl;
      cout<< " -Poly                            Estimation by polynom (stochastic matrices)."<<endl;
      cout<< " -PolyM                           Estimation by polynom (no stochastic matrices)."<<endl;
      cout<< " [-pos <in> <out>]                Compute the probabilities of the word for positions in to out"<<endl;
      cout<< " -h                               Print this help." <<endl;
      cout<<endl<<endl;
      return 1;
    }


  int error=0;
  if (word == NULL)
    {
      cerr<<" Argument is missing : -w <words> ?"<<endl;
      error=1;
    }
  if (alphabet_file == NULL)
    {
      cerr<<" Argument is missing : -a <alphabet_file> ? "<<endl;
      error=1;
    }
  if (config_file==NULL)
    {
      cerr<<" Argument is missing : -cf <config_file> ? "<<endl;
      error=1;
    }
  if((SpM==0)&&(Poly==0)&&(Spl==0)&&(SpH==0)&&(PolyM==0))
    {
      cerr<<" Argument is missing : -SpM? -SpH? -Spl? -Poly? -PolyM?"<<endl;
      error=1;
    }
  if (SpM+Poly+Spl+SpH+PolyM>1)
    {
      cerr<<" You have to use ONE and only ONE of these: -Poly or -PolyM or -SpM or -Spl or -SpH."<<endl;
      error=1;
    }
  if ((pos==1)&&((g_in<0)||(g_out<0)))
    {
      cerr<<" Argument is bad defined : -pos <in> <out> ? "<<endl;
      error=1;
    }
  if ((pos==1)&&(in>out))
    {
      cerr<<" Argument is bad defined : -pos <in> <out> ? (in > out)"<<endl;
      error=1;
    }
  if (error == 1)
    {
      cerr<<" PROBAword -h for help."<<endl;
      return 1;
    }

  //==============//
  // Le programme //
  //==============//

 
  
  Translator alpha(alphabet_file);
      
  //========================================
  //Construction of a DMM by a model file
  Drifting_Markov * DMM;
  int bad_model=0;	 
  ifstream mf(config_file);
  int method;
  mf>>method;
  mf.close();

  if (Poly==1)
    {
      if(method!=0)
	{
	  cerr<<"Error: You do not use Polynomial method."<<endl;
	  bad_model=1;
	}
      else
	{
	  DMM = new Polynomial(config_file); 
	}	
    } 
  if (PolyM==1)
    {
      if(method!=4)
	{
	  cerr<<"Error: You do not use Polynomial_M method."<<endl;
	  bad_model=1;
	}
      else
	{
	  DMM = new Polynomial_M(config_file); 
	}	
    }
  if (SpM==1)
    {
      if(method!=1)
	{
	  cerr<<"Error: You do not use Splines_M method."<<endl;
	  bad_model=1;
	}
      else
	{
	  DMM = new Splines_M(config_file);	
	}
    }
  if (SpH==1)
    {
      if(method!=2)
	{
	  cerr<<"Error: You do not use Splines_H method."<<endl;
	  bad_model=1;
	}
      else
	{
	  DMM = new Splines_H(config_file);
	}
    } 
  if (Spl==1)
    {
      if(method!=3)
	{
	  cerr<<"Error: You do not use Splines method."<<endl;
	  bad_model=1;
	}
      else
	{
	  DMM = new Splines(config_file);
	}
    }

  if(bad_model==1)
    {
      delete DMM;
      return 0;
    }
  //========================================
 
  int order=DMM->order();
  int sequence_size= DMM->length();

  Word mot(word,alphabet_file);
  int word_size=mot.length();
  unsigned long t = sequence_size-word_size;
  double proba;
  ofstream pr;
      
  char fout[word_size+4];
  strcpy(fout,word);
  strcat(fout,".out");
  if(VERBOSE)
    {
      cout<<"Output file: "<<fout<<endl;
    }
  pr.open(fout);     
  if (pos==1)
    {
      for (i=in;(unsigned long)(i)<out+1;i++)
	{
	  proba=mot.proba(i,*DMM);
	  pr<<i<<" "<<proba<<endl;
	}
    }
  else
    {
      for (i=0;(unsigned long)(i)<t;i++)
	{
	  proba=mot.proba(i,*DMM);
	  pr<<i<<" "<<proba<<endl;
	}
    }
  pr.close(); 
  
  delete DMM;
  
}
