/* pvalue_dmm.cc
 *
 * Copyright (C) 2008 Laboratoire Statistique & Genome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <unistd.h>

#include "Drifting_Markov.h"
#include "Polynomial.h"
#include "Polynomial_M.h"
#include "Splines.h"
#include "Splines_M.h"
#include "Splines_H.h"
#include "Word.h"

// Commentary in the shell
int VERBOSE = 1;


//===========================================================================================
//===========================================================================================

//////////////////////
// LA FONCTION MAIN //
//////////////////////

//===========================================================================================

int main(int argc, char *argv[]) 
{     
  //Number of parameters in the command line
  int     num_arg=1 ; 
  //Words file
  char  * word=NULL  ;
  //Alphabet file
  char  * alphabet_file=NULL;  
  //Markov file (file of G. Nuel by xspatt program)
  char * markov_file=NULL;
  //Config file (Polynomial)
  char * config_file=NULL;
  //Count words in the sequence
  int cf=0;
  int over=0;
  int under=0;
  int Nobs=-1;

  //help?
  int h=0;
  //Estimation by Splines or by polynomial method
  int SpM=0;
  int Poly=0;
  int PolyM=0;
  int Spl=0;
  int SpH=0; 

  while (num_arg < argc)
    {
      if ((strcmp (argv[num_arg], "-w") == 0) && (num_arg + 1 < argc))
	{
	  num_arg++ ;
	  word= argv[num_arg] ;
	  num_arg++ ;
	} 
      else
	if (strcmp (argv[num_arg], "-c") == 0)
	  {
	    num_arg++ ;
	    Nobs= atoi(argv[num_arg]); 	      
	    num_arg++ ;
	  } 
	else
	  if (strcmp (argv[num_arg], "-a") == 0)
	    {
	      num_arg++ ;
	      alphabet_file= argv[num_arg] ;
	      ifstream testfile(alphabet_file);
	      if (!testfile.good()) {
		cerr <<  "Unable to open file " << alphabet_file << endl;
		exit(-1);
	      }
	      testfile.close();
				
	      num_arg++ ;
	    } 
	  else 
	    if (strcmp (argv[num_arg], "-m") == 0)
	      {
		num_arg++ ;
		markov_file= argv[num_arg] ;
		ifstream testfile(markov_file);
		if (!testfile.good()) {
		  cerr <<  "Unable to open file " << markov_file << endl;
		  exit(-1);
		}
		testfile.close();
				
		num_arg++ ;
	      } 
	    else 
	      if (strcmp (argv[num_arg], "-SpM") == 0)
		{
		  SpM=1;
		  num_arg++ ;
		} 
	      else
		if (strcmp (argv[num_arg], "-Poly") == 0)
		  {
		    Poly=1;
		    num_arg++ ;
		  }     
		else
		  if (strcmp (argv[num_arg], "-PolyM") == 0)
		    {
		      PolyM=1;
		      num_arg++ ;
		    }     
		  else
		    if (strcmp (argv[num_arg], "-Spl") == 0)
		      {
			Spl=1;
			num_arg++ ;
		      }     
		    else
		      if (strcmp (argv[num_arg], "-SpH") == 0)
			{
			  SpH=1;
			  num_arg++ ;
			}     
		      else 
			if (strcmp (argv[num_arg], "-cf") == 0)
			  {
			    num_arg++ ;
			    config_file= argv[num_arg] ;
			    ifstream testfile(config_file);
			    if (!testfile.good()) {
			      cerr <<  "Unable to open file " << config_file << endl;
			      exit(-1);
			    }
			    testfile.close();
				
			    cf=1;
			    num_arg++ ;
			  } 
			else 
			  if (strcmp (argv[num_arg], "-h") == 0)
			    {
			      num_arg++ ;
			      h=1;
			    } 
			  else 
			    {
			      cerr<<endl;
			      cerr<<"#######################"<<endl
				  <<"##### BAD OPTIONS #####"<<endl
				  <<"#######################"<<endl
				  <<endl;
			      cerr << "WAY TO DO : "<<argv[0]<<endl
				   << " -w <word>"<<endl
				   << " -c <Nobs>=0>"<<endl
				   << " -a <alphabet_file>"<<endl
				   << " -m <markov_file>"<<endl
				   << " -cf <config_file>"<<endl
				   << " {-Poly or -PolyM or -SpM or -Spl or -SpH}"<<endl
				   << " [-h]"
				   << endl<<endl;
			      return 1;
			    }
    }
 


  if((h==1)||(num_arg==1))
    {
      cout<<endl;
      cout<<"##############################################################"
	  <<endl
	  <<"##########################          ##########################"
	  <<endl
	  <<"##########################   HELP   ##########################"
	  <<endl
	  <<"##########################          ##########################"
	  <<endl
	  <<"##############################################################"
	  <<endl
	  <<endl;

      cout<<"Use 0: PVALUEdmm -h"
	  <<endl<<endl;

      cout<<"Use 1: PVALUEdmm "<<endl
	  << " -w <word>"<<endl
	  << " -c <Nobs>=0>"<<endl
	  << " -a <alphabet_file>"<<endl
	  << " -m <markov_file>"<<endl
	  << " -cf <config_file>"<<endl
	  << " {-Poly or -PolyM or -SpM or -Spl or -SpH}"<<endl
	  << " [-h]" 
	  <<endl<<endl;


      cout<<"=============================================================================="<<endl;
      cout<<"                                   PVALUEdmm                                  "<<endl;
      cout<<"=============================================================================="<<endl;
      cout<<endl;
      cout<<"This program is dedicaced to the computation of pvalue of words";
      cout<<"Results are given at this order:"<<endl; 
      cout<<" Word Nobs Expectation -log(P(N>Nobs) P(N>Nobs)      if  Nobs>=E(N)"<<endl;
      cout<<"                                 OR                                "<<endl;
      cout<<" Word Nobs Expectation +log(P(N<Nobs) P(N<Nobs)      if  Nobs<E(N) "<<endl;
      cout<<endl<<endl;
      
      cout<<"There is one way to use the program :"<<endl;
      cout<<"Use 1: DMM is given in a model file"<<endl<<endl;
      cout<< " -w <word>                                    Word to studied."<<endl; 
      cout<< " -c <Nobs>=0>                                 Number of occurrences of word."<<endl;
      cout<< " -a <alphabet_file>                           A file describing the alphabet to use." <<endl;
      cout<< " -m <markov_file>                             Markov_file of xspatt program."<<endl;
      cout<< " -SpM                                         Estimation by splines (global)."<<endl;
      cout<< " -Spl                                         Estimation by splines (basis functions)."<<endl;
      cout<< " -SpH                                         Estimation by splines (no basis functions)."<<endl;
      cout<< " -Poly                                        Estimation by polynom (stochastic matrices)."<<endl;
      cout<< " -PolyM                                       Estimation by polynom (no stochastic matrices)."<<endl;
      cout<< " -cf <config_file>                            DMM is given by the config_file"<<endl;
      cout<< " -h                                           Print this help." <<endl;
      cout<<endl<<endl;
      return 1;
    }

  int error=0;
  if (word == NULL)
    {
      cerr<<" Argument is missing : -w <word> ?"<<endl;
      error=1;
    }
  if (Nobs < 0)
    {
      cerr<<" Argument is bad defined : -c <Nobs>=0> ?"<<endl;
      error=1;
    }
  if (alphabet_file == NULL)
    {
      cerr<<" Argument is missing : -a <alphabet_file> ? "<<endl;
      error=1;
    }
  if (config_file==NULL)
    {
      cerr<<" Argument is missing : -cf <config_file> ? "<<endl;
      error=1;
    } 
  if (markov_file == NULL)
    {
      cerr<<" Argument is missing : -m <markov_file> ? "<<endl;
      error=1;
    }
  if((SpM==0)&&(Poly==0)&&(Spl==0)&&(SpH==0)&&(PolyM==0))
    {
      cerr<<" Argument is missing : -SpM? -SpH? -Spl? -Poly? -PolyM?"<<endl;
      error=1;
    }
  if (SpM+Poly+Spl+SpH+PolyM>1)
    {
      cerr<<" You have to use ONE and only ONE of these: -Poly or -PolyM or -SpM or -Spl or -SpH."<<endl;
      error=1;
    }
  if (error == 1)
    {
      cerr<<endl;
      cerr<<" PVALUEdmm -h for help."<<endl<<endl;;
      return 1;
    }

  //==============//
  // Le programme //
  //==============//

  Translator alpha(alphabet_file);

  //========================================
  //Construction of a DMM by a model file
  Drifting_Markov * DMM;
  int bad_model=0;	 
  ifstream mf(config_file);
  int method;
  mf>>method;
  mf.close();

  if (Poly==1)
    {
      if(method!=0)
	{
	  cerr<<"Error: You do not use Polynomial method."<<endl;
	  bad_model=1;
	}
      else
	{
	  DMM = new Polynomial(config_file); 
	}	
    } 
  if (PolyM==1)
    {
      if(method!=4)
	{
	  cerr<<"Error: You do not use Polynomial_M method."<<endl;
	  bad_model=1;
	}
      else
	{
	  DMM = new Polynomial_M(config_file); 
	}	
    }
  if (SpM==1)
    {
      if(method!=1)
	{
	  cerr<<"Error: You do not use Splines_M method."<<endl;
	  bad_model=1;
	}
      else
	{
	  DMM = new Splines_M(config_file);	
	}
    }
  if (SpH==1)
    {
      if(method!=2)
	{
	  cerr<<"Error: You do not use Splines_H method."<<endl;
	  bad_model=1;
	}
      else
	{
	  DMM = new Splines_H(config_file);
	}
    } 
  if (Spl==1)
    {
      if(method!=3)
	{
	  cerr<<"Error: You do not use Splines method."<<endl;
	  bad_model=1;
	}
      else
	{
	  DMM = new Splines(config_file);
	}
    }

  if(bad_model==1)
    {
      delete DMM;
      return 0;
    }
  //========================================
 
  
  int order=DMM->order();

  //Algorithm of G. Nuel
 
  int i,j,k,t;
  int sequence_size= DMM->length();
  int alphabet_size= alpha.tell_alphabet_size();

  Word mot(word,alphabet_file);
  double expectation=mot.expect(sequence_size,*DMM);
  cout<<word<<" "<<Nobs<<" "<<expectation;
  if (expectation>Nobs)
    {
      under=1;
    }
  else
    {
      over=1;
    }
 
  //Number of starting states
  int nbst;
  if(order==0)
    {
      nbst=alphabet_size;
    }
  else
    {
      nbst = int(pow(double(alphabet_size), double(order)));
    }

  /***********************/
  /*** Initialization  ***/
  /***********************/
  //P and Q
  ifstream PQ(markov_file);
  char line[256];
  int PQ_order, PQ_alphabet_size;
  int * startcode = new int [nbst];	 
  PQ.getline(line,256);
  PQ.getline(line,256);
  PQ>>PQ_alphabet_size>>PQ_order;
  if ((PQ_order!= order)||(PQ_alphabet_size!= alphabet_size))
    {
      cout<<endl<<"Error... Alphabet size or order bad defined"<<endl;
      return 1;  
    }
  PQ.getline(line,256); 
  PQ.getline(line,256);
  for(i=0;i<nbst;i++)
    {
      PQ>>startcode[i];
    }
  PQ.getline(line,256);
  int nfst;
  PQ.getline(line,256);
  PQ>>nfst;
  PQ.getline(line,256);
  PQ.getline(line,256);
  for(i=0;i<nfst;i++)
    {
      PQ.getline(line,256);
    }
  PQ.getline(line,256);
  int nstates, nz, Pnz, Qnz;
  PQ>>nstates>>nz>>Pnz>>Qnz;
  PQ.getline(line,256); 
  PQ.getline(line,256);
  int * dataP = new int [3*Pnz];
  int * fromP = &dataP[0];
  int * toP = &dataP[Pnz];
  int * contentP = &dataP[2*Pnz];

  int * dataQ = new int [3*Qnz];
  int * fromQ = &dataQ[0];
  int * toQ = &dataQ[Qnz];
  int * contentQ = &dataQ[2*Qnz];

  for(i=0;i<Pnz;i++)
    {
      PQ>>fromP[i]>>toP[i]>>contentP[i];
    }
  PQ.getline(line,256); 
  PQ.getline(line,256);
  for(i=0;i<Qnz;i++)
    {
      PQ>>fromQ[i]>>toQ[i]>>contentQ[i];
    }
  PQ.close();
	  
  //++++++++++++++++++++
  //
  //  overrepresented
  //
  //++++++++++++++++++++

  if (over==1)
    {
      if (Nobs == 0)
	{
	  cout<<" "<<0;
	  cout<<" "<<1<<endl;
	}
      else
	{
	  //Initialization of x, y, z, mu.
	  double ** x= new double* [Nobs];
	  x[0]= new double[Nobs*nstates];

	  double ** y= new double* [Nobs];
	  y[0]= new double[Nobs*nstates];

	  double * z= new double[nstates];

	  for (i=1; i<Nobs; i++)
	    {
	      x[i] = x[i-1]+ nstates;
	      y[i] = y[i-1]+ nstates;
	    } 
	  for (i=0; i<Nobs; i++)
	    {
	      for(j=0;j<nstates;j++)
		{
		  x[i][j]=0;
		  y[i][j]=0;
		  z[j]=0;
		}
	    }
	      
	  int n=sequence_size;
	  for(i=0; i<Qnz;i++)
	    {
	      x[0][fromQ[i]]+=DMM->Pi(n-3,contentQ[i]);  
	    }

	  /*****************/
	  /*** Main loop ***/
	  /*****************/
 	      
	  for(t=0;t<n-2;t++)
	    {
	      //Stock of Pi.
	      int tstoc=Pnz+Qnz;
	      double * Pi_stoc= new double [tstoc];
	      for(j=0;j<Pnz;j++)
		{
		  Pi_stoc[j]=DMM->Pi(n-3-t,contentP[j]);
		}
	      for(j=Pnz;j<tstoc;j++)
		{
		  Pi_stoc[j]=DMM->Pi(n-3-t,contentQ[j-Pnz]);
		}
		
	      for(j=0;j<Nobs;j++)
		{
		  for(k=0;k<nstates;k++)
		    {
		      y[j][k]=x[j][k];
		    }
		}
	      //x[i]=0 for the product of matrices
	      for(j=0;j<Nobs;j++)
		{
		  for(k=0;k<nstates;k++)
		    {
		      x[j][k]=0;
		    }
		}
	      for(j=0;j<Pnz;j++)
		{
		  x[0][fromP[j]]+=Pi_stoc[j]*y[0][toP[j]];		      
		} 
	      for(i=1;i<Nobs;i++)
		{
		  for(j=0;j<Pnz;j++)
		    {
		      x[i][fromP[j]]+=Pi_stoc[j]*y[i][toP[j]]; 
		    }
		  for(j=0;j<Qnz;j++)
		    {
		      x[i][fromQ[j]]+=Pi_stoc[Pnz+j]*y[i-1][toQ[j]];
		    }
		}
		  
	      for(j=0;j<nstates;j++)
		{
		  z[j]+=x[Nobs-1][j];
		}
		  
	      delete[] Pi_stoc;
	    }

	  double pvalue=0;
	  double * mu= new double[nbst];

	  //Stationary laws of Pi_0
	  if(order==0)
	    {
	      int i;
	      for (i=0;i<nbst;i++)
		{
		  mu[i]=DMM->Pi(0,0,i);
		}	 
	    }
	  else
	    {
	      int i,j;
	      int nbc=alphabet_size;
	      int nbl= int(pow(double(alphabet_size),order));
	      Markov Pi_t(alphabet_size,order);
	      for(i=0;i<nbl;i++)
		{
		  for(j=0;j<nbc;j++)
		    {
		      Pi_t(j+nbc*i)=DMM->Pi(0,i,j);
		    }
		}
	      Pi_t.compute_stat_law(true);
	      for(i=0;i<nbst;i++)
		{
		  mu[i]=Pi_t.Mu(i);
		}
	    }

	  for(i=0;i<nbst;i++)
	    { 
	      pvalue+=mu[i]*z[startcode[i]];
	    }   
	  cout<<" "<< -log(pvalue)/log(10);
	  cout<<" "<<pvalue<<endl;

	  delete[] x[0];
	  delete[] y[0];
	  delete[] x;
	  delete[] y;
	  delete[] z;
	  delete[] mu;
	}
    }
  //++++++++++++++++++++
  //
  //  underrepresented
  //
  //++++++++++++++++++++

  else
    {
      //Initialization of x, y, z, mu.
      double ** x= new double* [Nobs+1];
      x[0]= new double[(Nobs+1)*nstates];

      double ** y= new double* [Nobs+1];
      y[0]= new double[(Nobs+1)*nstates];

      for (i=1; i<Nobs+1; i++)
	{
	  x[i] = x[i-1]+ nstates;
	  y[i] = y[i-1]+ nstates;
	} 
      for (i=0; i<Nobs+1; i++)
	{
	  for(j=0;j<nstates;j++)
	    {
	      x[i][j]=1;
	      y[i][j]=0;
	    }
	}

      /******************/
      /*** Main loop  ***/
      /******************/
      int n=sequence_size;
      for(t=0;t<n-1;t++)
	{	 
	  //Stock of Pi.
	  int tstoc=Pnz+Qnz;
	  double * Pi_stoc= new double [tstoc];
	  for(j=0;j<Pnz;j++)
	    {
	      Pi_stoc[j]=DMM->Pi(n-2-t,contentP[j]);
	    }
	  for(j=Pnz;j<tstoc;j++)
	    {
	      Pi_stoc[j]=DMM->Pi(n-2-t,contentQ[j-Pnz]);
	    }
	      
	  for(j=0;j<Nobs+1;j++)
	    {
	      for(k=0;k<nstates;k++)
		{
		  y[j][k]=x[j][k];
		}
	    }
	  //x[i]=0 for the product of matrices
	  for(j=0;j<Nobs+1;j++)
	    {
	      for(k=0;k<nstates;k++)
		{
		  x[j][k]=0;
		}
	    }
	  for(j=0;j<Pnz;j++)
	    {
	      x[0][fromP[j]]+=Pi_stoc[j]*y[0][toP[j]];
	    }
	  for(i=1;i<Nobs+1;i++)
	    {
	      for(j=0;j<Pnz;j++)
		{
		  x[i][fromP[j]]+=Pi_stoc[j]*y[i][toP[j]];
		}
	      for(j=0;j<Qnz;j++)
		{
		  x[i][fromQ[j]]+=Pi_stoc[Pnz+j]*y[i-1][toQ[j]];
		}
	    }
	  delete[] Pi_stoc;
	}

      double pvalue=0;
      double * mu= new double[nbst];


      //Stationary laws of Pi_0
      if(order==0)
	{
	  int i;
	  for (i=0;i<nbst;i++)
	    {
	      mu[i]=DMM->Pi(0,0,i);
	    }	 
	}
      else
	{
	  int i,j;
	  int nbc=alphabet_size;
	  int nbl= int(pow(double(alphabet_size),order));
	  Markov Pi_t(alphabet_size,order);
	  for(i=0;i<nbl;i++)
	    {
	      for(j=0;j<nbc;j++)
		{
		  Pi_t(j+nbc*i)=DMM->Pi(0,i,j);
		}
	    }
	  Pi_t.compute_stat_law(true);
	  for(i=0;i<nbst;i++)
	    {
	      mu[i]=Pi_t.Mu(i);
	    }
	}
	      
      for(i=0;i<nbst;i++)
	{  
	  pvalue+=mu[i]*x[Nobs][startcode[i]];
	}   
      cout<<" "<< log(pvalue)/log(10);
      cout<<" "<< pvalue<<endl;
    
      delete[] x[0];
      delete[] y[0];
      delete[] x;
      delete[] y;	     
      delete[] mu;
    }
  delete[] dataP;
  delete[] dataQ;
  delete[] startcode;
  delete DMM;
    
  return 0;
}        

  
